/* START OF FILE: Sound_Tap_Channel.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Sound_Tap_Channel.c - Tap a Sound channel for AIFF disk file playback to monitor progress.
 ** (C) Copyright 1995 James Merkle
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1995.05.17		(James Merkle)
 **						Initial coding started from scratch
 **
 **  1995.06.08		(James Merkle)
 **						Totally finished. Works perfect. (Except for loss of first buffer caused by a late tap install)
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 ** 
 **	This set of routines allows semi-accurate tapping of data stream being fed to a sound channel.
 **	It is true that the data is raw and may be compressed and has not been processed yet, but
 **	it is used when I tap into the AIFF disk playback routines using files normally not compressed.
 **	Tapping is performed on the audio channel by inserting monitor on the "Get More Data". This is the channels
 **	"Get more data" and not the higher lever "user" defined double buffer callback for more data.
 ** The double buffer callback for the user could not be used because it is used by AIFF play routines anyway,
 ** and "Get More Data" is in every sound component.
 **	Putting the tap at that stage allows audio monitoring of speech synthesis without creating amplitute 
 **	tables for phonemes, as well as highly useful for monitoring AIFF file play from disk.
 **	
 **	It is too hard to create a sound component that avoids switching CPU modes ( and also is
 **	called no matter what and is also called by speech synthesis. One way is to Target and close 
 **	the "Mixer" component and intercept and delegate all calls to the original. But this is not
 **	officially permitted in Sound Manager 3.0 and the Mixer is usually opened and is unclosable.
 **	
 **	Targeting a rate converter would work but Apples mixer is allowed to skip over all rate converters if
 **	the destination output claims it can rate convert in hardware.
 **	
 **	Targeting a size/rate converter would work but Apples mixer is allowed to skip over all size/rate converters if
 **	the destination output claims it can rate convert sizes in hardware.
 **	
 **	Targeting an output device component might work but Apples mixer is a tool that creates multiple copies of
 ** output device component instantiations if the tool claims to be multichannel, just as the Mixer
 **	is a special component that can mix from multiple inputs it can also maintain multiple outputs.
 **	
 **	Tapping the 840av DSP using ARTA and DSP programming can be done as shown in "Levels" example from DTS but
 **	That only works on 840av Macs and not on any standard PowerMacs. It also is not input channel specific.
 **	
 **	Renaming the mixer component to a fake name before it registers and creating a phantom stub Mixer that
 **	calls the renamed mixer would work. In fact it does work and a hack exists doing so that changes the Macs CLUT
 **	when sound emits creating psychedelia light show. But this technique is sadly too goddamn skanky especially if 
 **	the mixer is moved into ROM and is not easily overridden in a way that allows editing of it (Pippin?). One way
 **	to override the 8 and 16 bit dual purpose Mixer component is to write a early boot patch to load my Mixer shell
 **	when Apples Mixer loads and have a hack in component manager to rename any other mixer something wierd.
 **	I think that is too risky especially if the Mixer component needs to look itself up for global space and sees that
 **	it is no longer a Mixer component.
 **	
 **	In Sound Manager 2.0 and 1.0 there was better ways using "SndAddModifier" but version 3.0
 **  is a subset of Sound Manager 2.0 with some extensions added. This method should work for a couple years.
 **	
 **	I am certain I found the best and safest solution and It totally depends on only one small undocumented feature.
 **	That undocumented feature is the offset I found for "source_Sound_PB" in the channel structure info
 **	
 **	
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */


#include <AIFF.h>
#include <Sound.h>
#include <SoundComponents.h>

#include "Shell_Globals.h"
#include "Error_Utilities.h"
#include "Memory_Utilities.h"
#include "Shell_Assembly.h"
#include "Sound_Play.h"

#include "OS_Utilities.h"
#include "Time_Utilities.h"

/* ===
#include "Dreaming_Stuff.h"
=== */

#include "Shell_Globals.h"

#include "Sound_Tap_Channel.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */

#if (ORIGINAL_MAC_SYSTEM == 1)
/*  */
/* Unique little THINK C constructs to save and restore A5 worlds */
/*  */
#define RememberA5()	do { __Get_A5(); asm { move.l A5,(a1) } } while (0)

#define Setup_A5()			do { asm { move.l A5,-(sp) } __Get_A5(); asm { move.l (a1),A5 } } while (0)
#define Restore_A5()		do { asm { move.l (sp)+,A5 } } while (0)

#endif /*  (ORIGINAL_MAC_SYSTEM == 1) */

#define k_TICKS_PER_SECOND	(60UL)

#define k_AVAILABLE_STACK_BYTES_FOR_AMPLITUDE_SCAN	(4096)

/*  */
/* The taps are made generously large just for the heck of it (64K).*/
/* Many devices will be 1024 or so, but we need the most accurate */
/* possible and some devices (file based AIFF reading) might use 64K */
/* if the user selects a foolishly too large buffer space. I select a buffer space of */
/* 56K which, oddly enough, results in 2 bouble buffers of about 26K. 26K is */
/* much less than 64K so we are safe. There are two of these buffers. */
/* 
/* double backs but our tap is above the double back and in main feeder. */
/* I hope a device uses 64K or slightly less. If too small terrible inacuracies */
/* of sound tapping will result (too old for a sample) */
/*  */
#define k_MAX_MEMORY_FOR_SOUND_TAP 	(64L * 1024L)

#define k_BARE_MINIMUM_USEFUL_SAMPLES 	(512)

#define k_LOW_THREE_DECIMAL_BITS	(3UL)

/*  */
/* if one value of 0x0003FFFF is multiplied by 0x00004000 */
/* the result will still fit in a 32 bit word. The second value is the */
/* complement and at 60 ticks per second is good enough for */
/* 273 seconds without resorting to 64 bit math nor floating point. */
/*  */
	
#define k_LOW_THREE_DECIMAL_BITS_COMPLEMENT		(0x4000UL)

/*  */
/* Withut this adjustment, data being tapped may be too early */
/* because the stream has passed our tap a little and is partially */
/* in other tiny component buffers and in the hardware chip buffer. */
/* to be more exact we need to second guess and sample back into our */
/* older tap buffer if needed and try to always read this many */
/* samples behind. Note that this is not byte or channel based and */
/* may not be precise onder some situations, but it is really */
/* quite good enough. Lips are perfectly on sync with it. */
/*  */
#define k_SAMPLES_ASSUMED_ALREADY_IN_COMPONENT_STREAM		(1024)

/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */



/*  */
/* This structure is probably only valid in Sound Manager 3.0! */
/* And because it is mostly undocumented and supposed to be opaque, */
/* it is wise to not use it for any purposes. I think nearly all the fields are */
/* correctly disassembled and noted but I only depend on */
/* "double_Buffer_For_Sound_Play" in my own programs */
/* and that is the only one I ever use */
/*  */
typedef struct {
	ComponentInstance			output_Device_Last_Component;
	Component					sound_Chain_Output_Component;
	ProcessSerialNumber		owner_Process;		
	SoundSource					mixer_Source_ID;	
	
	/*  */
	/* The only field I care about or use... */
	/*  */
	SoundParamBlock			source_Sound_PB;
	
	SndDoubleBufferHeader		double_Buffer_For_Sound_Play;
	
	/*  */
	/* All the rest of structure may change too likely over the next few years ... */
	/* But I am confident I mapped out the initial fields correctly. */
	/*  */
	
}Sound_Manager_Private_Rec, *Sound_Manager_Private_Ptr;


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */

static void __Get_A5(void);

pascal Boolean Swap_More_Data_Callback_1(SoundParamBlockPtr	*sound_Source_Ptr);

pascal Boolean Swap_More_Data_Callback_2(SoundParamBlockPtr	*sound_Source_Ptr);

pascal Boolean Swap_More_Data_Callback_3(SoundParamBlockPtr	*sound_Source_Ptr);

Safe_Boolean Get_Average_Wave_Amplitude(Boolean twos_Complement_Storage
													, CARDINAL_32 bytes_Per_Sample
													, CARDINAL_32 samples_In_Buffer
													, CARDINAL_8 *sample_Data
													, CARDINAL_8 *amplitude);

Safe_Boolean Extract_Estimated_Current_Samples_From_Tap(CARDINAL_32 channel_Desired
																	, CARDINAL_32 total_Channels
																	, CARDINAL_32 sample_Size
																	, CARDINAL_32 start_Sample_Point
																	, CARDINAL_32 original_Samples_Needed
																	, CARDINAL_8 *source_Input
																	, CARDINAL_32 source_Input_Available_Samples
																	, CARDINAL_32 max_Sample_Size_Accectable
																	, Ptr samples_Destination
																	, CARDINAL_32 *samples_Returned
																	, CARDINAL_32 *sample_Size_Returned);



/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */
	
	CARDINAL_32				g_Active_Taps_Opened;
	CARDINAL_32				g_Active_Taps_Available;
	Audio_Progress_Tap_Rec	g_Audio_Taps[k_SIMULTANEOUS_TAPS];

/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8	*unit_Name;


/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
#if (ORIGINAL_MAC_SYSTEM == 1)
static void __Get_A5(void)
{
	asm {
		bsr.s	@1
		dc.l	0			;  store A5 here
@1		move.l	(sp)+,a1
	}
}
#endif
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Initialize_Sound_Tap_Channel(void)
{
	SIMPLE_LOOP		loop;
	CARDINAL_8			*static_Unit_Name = "\pSound_Tap_Channel";
/* END VAR */
	
	
	unit_Name = static_Unit_Name;
	
	g_Active_Taps_Opened = 0;
	
#if !defined(ORIGINAL_MAC_SYSTEM)
#error "UNDEFINED PARAMETER!"
#endif
#if (k_SIMULTANEOUS_TAPS != 3)
#error "number of taps differs from number of planned callbacks!"
#endif
	
	g_Active_Taps_Available = k_SIMULTANEOUS_TAPS;

	for (loop = 0; (loop < k_SIMULTANEOUS_TAPS); loop++) {
		g_Audio_Taps[loop].info.active_And_In_Use = SAFE_FALSE;
	}
	
	/*  */
	/* If PowerPC then we need to create memory structures for creating */
	/* proc pointers. Regretfully they are assigned one by one. */
	/*  */
#if (ORIGINAL_MAC_SYSTEM == 0)

	get_More_Data_Function_UPP[0] = (SoundParamUPP)NewRoutineDescriptor(
								(ProcPtr)(&Swap_More_Data_Callback_1)
								, uppSoundParamProcInfo
								, GetCurrentISA());
					
	get_More_Data_Function_UPP[1] = (SoundParamUPP)NewRoutineDescriptor(
								(ProcPtr)(&Swap_More_Data_Callback_2)
								, uppSoundParamProcInfo
								, GetCurrentISA());
					
	get_More_Data_Function_UPP[2] = (SoundParamUPP)NewRoutineDescriptor(
								(ProcPtr)(&Swap_More_Data_Callback_3)
								, uppSoundParamProcInfo
								, GetCurrentISA());
					
#endif
	
	
	RememberA5();
	
	return(SAFE_TRUE);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Destroy_Sound_Tap_Channel(void)
{
	Safe_Boolean		function_Result;
/* END VAR */


/*  */
/* Global persistant structures are deleted here */
/*  */

	function_Result = SAFE_TRUE;
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Destroy_Audio_Tap(Audio_Progress_Tap_Ptr audio_Tap_Ptr)
{
	SIMPLE_LOOP		loop;
	SIMPLE_LOOP		buffer_Index;
	Boolean				found_Item;
	CARDINAL_32		items_To_Locate;
	CARDINAL_32		items_Located;
	Safe_Boolean			function_Result;
/* END VAR */

	
	found_Item = FALSE;
	items_To_Locate = g_Active_Taps_Opened;
	items_Located = 0;
	for (loop = 0; ((loop < k_SIMULTANEOUS_TAPS) && (found_Item != TRUE) && (items_Located < items_To_Locate)); loop++) {
		if (g_Audio_Taps[loop].info.active_And_In_Use == SAFE_TRUE) {
			items_Located += 1;
			
			if (&g_Audio_Taps[loop] == audio_Tap_Ptr) {
				found_Item = TRUE;
				
				for(buffer_Index = 0; (buffer_Index < 2); buffer_Index++) {
					if (g_Audio_Taps[loop].info.tap_Buffer[buffer_Index].component_Recent_Buffer_Memory_In_Use == MEMORY_INITIALIZED) {					
						if (Destroy_Handle(&g_Audio_Taps[loop].info.tap_Buffer[buffer_Index].component_Recent_Buffer
										, &g_Audio_Taps[loop].info.tap_Buffer[buffer_Index].component_Recent_Buffer_Memory_In_Use) != SAFE_TRUE) {
							/*  */
							/* Regretfully, just ignore deletion errors and clear out entry */
							/*  */
						}
					}
				}

				g_Audio_Taps[loop].info.active_And_In_Use = SAFE_FALSE;
				g_Active_Taps_Opened = g_Active_Taps_Opened - 1;
			}
		}
	} /* for (loop = 0; ((loop < k_SIMULTANEOUS_TAPS) */

	if (found_Item == TRUE) {
		function_Result = SAFE_TRUE;
	}
	else {
		function_Result = SAFE_FALSE;
	}
	
	return(function_Result);
	
} /* Safe_Boolean Destroy_Audio_Tap ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Install_Tap_On_Get_More_Data(SndChannelPtr	active_Snd_Channel_Ptr
											, CARDINAL_32 user_Reference_Info
											, Audio_Progress_Tap_Ptr *resulting_Audio_Tap_Ptr)
/*  */
/* This substitutes the ProcPtr for more data that Apple installs */
/* for async file playing with our own that merely calls */
/* the original one from inside here. If the program is aborted, */
/* other procedures stop file playback and therefore a routine */
/* to fix the swapped pointer back to normal is not needed. */
/*  */
{
	CARDINAL_8						*code_Name;
	Sound_Manager_Private_Ptr	private_Info_Ptr;
	Audio_Progress_Tap_Ptr		audio_Tap_Ptr;
	Safe_Boolean						function_Result;
	CARDINAL_32					memory_Requested;
	SIMPLE_LOOP					loop;
	SIMPLE_LOOP					buffer_Index;
	SignedByte						temp_State;
	Boolean							found_Item;
/* END VAR */


	*resulting_Audio_Tap_Ptr = nil;
	code_Name = "\pInstall_Tap_On_Get_More_Data";

	private_Info_Ptr = (Sound_Manager_Private_Ptr)active_Snd_Channel_Ptr->firstMod;
	if ((CARDINAL_32)private_Info_Ptr->source_Sound_PB.moreRtn == 0) {
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}

	/*  */
	/* Check if remedied and a slot open now. */
	/*  */
	if (g_Active_Taps_Opened >= k_SIMULTANEOUS_TAPS) {
		goto Normal_Function_Exit;
	}
	
	/*  */
	/* Find an open slot */
	/*  */
	found_Item = FALSE;
	for (loop = 0; ((loop < k_SIMULTANEOUS_TAPS) && (found_Item != TRUE)); loop++) {
		if (g_Audio_Taps[loop].info.active_And_In_Use != SAFE_TRUE) {
			found_Item = TRUE;
			audio_Tap_Ptr = &g_Audio_Taps[loop];
		}
	}

	if (found_Item != TRUE) {
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}
	
	audio_Tap_Ptr->info.next_Buffer = 0;
	
	
	/*  */
	/* Undefined and unused, but can be modified for future use */
	/*  */
	audio_Tap_Ptr->info.user_Reference_Info = user_Reference_Info;
	
	for(buffer_Index = 0; (buffer_Index < 2); buffer_Index++) {
	
		audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Buffer = nil;
		audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Buffer_Memory_In_Use = MEMORY_NOT_INITIALIZED;
		memory_Requested = k_MAX_MEMORY_FOR_SOUND_TAP;
		audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Buffer_Max_Size = memory_Requested;
	
		if (Create_Handle(&memory_Requested
									, 0
									, SAFE_TRUE
									, SAFE_FALSE
									, &audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Buffer
									, &audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Buffer_Memory_In_Use) != SAFE_TRUE) {
			Display_Low_Memory_Error_Message(unit_Name, code_Name, memory_Requested);
			goto Error_Handler;
		}
		
		/*  */
		/* Move it and lock it down because it is written at interrupt time. */
		/*  */
		if (Begin_Use_Of_Handle(audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Buffer
										, SAFE_TRUE
										, &temp_State) != SAFE_TRUE) {
			goto Error_Handler;
		}
			
		Fill_Byte_Memory_With_Byte((Ptr)&audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Info, sizeof(SoundComponentData), 0);
				
		/*  */
		/* Indicate that buffer is not filled (too old) */
		/*  */
		audio_Tap_Ptr->info.tap_Buffer[buffer_Index].tick_Time_Last_Called = 0;
		audio_Tap_Ptr->info.tap_Buffer[buffer_Index].data_untrusted_In_Transition = SAFE_FALSE;
		
	} /* for(buffer_Index = 0; (buffer_Index < 2); buffer_Index++) */
	
	/*  */
	/* We are done. Keep this one.*/
	/*  */
	audio_Tap_Ptr->info.tick_Time_First_Setup = TickCount();
	audio_Tap_Ptr->info.tap_Buffer_Total_Samples_Counted = 0;
	
	audio_Tap_Ptr->info.active_And_In_Use = SAFE_TRUE;
	g_Active_Taps_Opened += 1;
	
	/*  */
	/* Swap interrupt callbacks now! */
	/*  */
#if (ORIGINAL_MAC_SYSTEM == 0)
	audio_Tap_Ptr->original_Get_More_Data_UPP = private_Info_Ptr->source_Sound_PB.moreRtn;
	 private_Info_Ptr->source_Sound_PB.moreRtn = audio_Tap_Ptr->get_More_Data_Function_UPP;
#else
	audio_Tap_Ptr->original_Get_More_Data_Function = (ProcPtr)private_Info_Ptr->source_Sound_PB.moreRtn;
	
	switch (g_Active_Taps_Opened) {
		case 1:
			 private_Info_Ptr->source_Sound_PB.moreRtn = (SoundParamUPP)&Swap_More_Data_Callback_1;
			break;
			
		case 2:
			 private_Info_Ptr->source_Sound_PB.moreRtn = (SoundParamUPP)&Swap_More_Data_Callback_2;
			break;
			
		case 3:
			 private_Info_Ptr->source_Sound_PB.moreRtn = (SoundParamUPP)&Swap_More_Data_Callback_3;
			break;
			
		default:
			audio_Tap_Ptr->info.active_And_In_Use = SAFE_FALSE;
			g_Active_Taps_Opened = g_Active_Taps_Opened - 1;
			goto Error_Handler;
	}
	
#endif

	*resulting_Audio_Tap_Ptr = audio_Tap_Ptr;
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
	
Normal_Function_Exit:
	return(function_Result);

} /* Safe_Boolean Install_Tap_On_Get_More_Data... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Get_Current_Amplitude_From_Tap(Audio_Progress_Tap_Ptr audio_Tap_Ptr
														, CARDINAL_32 channel_Desired
														, CARDINAL_32 second_Ratio_For_Window
														, CARDINAL_8 *amplitude)
/*  */
/* This routine is similar to routine in sound input manager to  */
/* estimate recent amplitude of a channel (volume) */
/* It dynamically extracts rather than passively returning a */
/* max amplitude over a 512 byte range. */
/* It is also heavily over commented because it is a part of a */
/* very complex process */
/*  */
{
	CARDINAL_32				samples_Needed;
	CARDINAL_32				max_Sample_Size_Accectable;
 	CARDINAL_32				samples_Returned;
 	CARDINAL_32 				sample_Size_Returned;
 	CARDINAL_32				bytes_Per_Second;
 	CARDINAL_32				sample_Data_Buffer_Size;
 	CARDINAL_32				samples_Per_Second;
 	CARDINAL_32				bytes_Per_Sample;
 	CARDINAL_32				current_Tick_Time;
 	CARDINAL_32				elapsed_Time;
 	CARDINAL_32				total_Samples_To_Skip;
 	CARDINAL_32				total_Buffers_Available;
 	CARDINAL_32				samples_to_Ignore;
 	CARDINAL_32				start_Sample_Point;
 	DIGITS_SPEED_REAL			temp_Floating_Value;
 	SIMPLE_LOOP				buffer_Index;
	Safe_Boolean					function_Result;
 	CARDINAL_8					sample_Data[k_AVAILABLE_STACK_BYTES_FOR_AMPLITUDE_SCAN];
/* END VAR */

	
	*amplitude = 0;

	/*  */
	/* Limit time sample from one 60th of a second to half a second. */
	/* everything else is crap anyway. 30 is best */
	/*  */
	if (second_Ratio_For_Window > 60) {
		second_Ratio_For_Window = 60;
	}
	else {
		if (second_Ratio_For_Window < 2) {
			second_Ratio_For_Window = 2;
		}
	}
	
	/*  */
	/* Check if no buffers ever filled yet */
	/*  */
	if (audio_Tap_Ptr->info.tap_Buffer_Total_Samples_Counted == 0) {
		goto Error_Handler;
	}

	/*  */
	/* Use information from captured component info to */
	/* calculate how many samples should have elapsed since */
	/* we started tapping. */
	/*  */
	samples_Per_Second = audio_Tap_Ptr->info.tap_Buffer[0].component_Recent_Info.sampleRate;
	
	/*  */
	/* Preserve low three bits of rate for extended accuracy for large calculation */
	/* (accurate within 70th to 10th of second) and will hold 2.2 hours at max Sound Manager rate.*/
	/* This is fine because ticks are only accurate for us within 120th second. */
	/*  */
	samples_Per_Second = (samples_Per_Second >> (16UL - k_LOW_THREE_DECIMAL_BITS));

	current_Tick_Time = TickCount();
	
	/*  */
	/* Calculate ticks elapsed since first buffer was tapped. */
	/*  */
	elapsed_Time = Calculate_Total_Ticks(audio_Tap_Ptr->info.tick_Time_First_Setup, current_Tick_Time);
	
	/*  */
	/* Calculate number of samples we wish to skip over to obtain */
	/* exact position in our buffers to start extracting audio data from. */
	/* This will help us calculate which buffer to use. */
	/*  */
	
	if (elapsed_Time < k_LOW_THREE_DECIMAL_BITS_COMPLEMENT) {
		/*  */
		/* For this common state, we can use integer math. */
		/* if one value of 0x0003FFFF is multiplied by 0x00004000 */
		/* the result will still fit in a 32 bit word. The second value is the */
		/* complement and at 60 ticks per second is good enough for */
		/* 273 seconds without resorting to 64 bit math nor floating point. */
		/*  */
		
		total_Samples_To_Skip = (elapsed_Time * samples_Per_Second) / k_TICKS_PER_SECOND;
	}
	else {
		temp_Floating_Value = (1.0 * elapsed_Time);
		temp_Floating_Value = temp_Floating_Value * samples_Per_Second;
		temp_Floating_Value = temp_Floating_Value / (1.0 * k_TICKS_PER_SECOND);
		total_Samples_To_Skip = temp_Floating_Value;
	}
	
	total_Samples_To_Skip = (total_Samples_To_Skip >> k_LOW_THREE_DECIMAL_BITS);
	
	/*  */
	/* At this point we would add or subtract the amount */
	/* of lead time needed to allow for being behind by a buffer at start */
	/* or for inter-component lag time. It would be a few milliseconds at most. */
	/* The hardware lag is 1024 samples or 50% or 1024 at best. */
	/* Because I now use two buffers instead of one, I can point backwards in time */
	/* as many samples as I wish to, but in these cases I only need to point backwards */
	/* a portion of one buffer. */
	/*  */
	if (total_Samples_To_Skip >= k_SAMPLES_ASSUMED_ALREADY_IN_COMPONENT_STREAM) {
		total_Samples_To_Skip = total_Samples_To_Skip - k_SAMPLES_ASSUMED_ALREADY_IN_COMPONENT_STREAM;
	}

	/*  */
	/* Actually sample 1 30th of a second */
	/* at 22Khz it is 733 samples at 44Khz it is double and at */
	/* lower rates it is less of course. */
	/*  */
	samples_Needed = ((samples_Per_Second / second_Ratio_For_Window) >> k_LOW_THREE_DECIMAL_BITS);
	
	if (total_Samples_To_Skip > audio_Tap_Ptr->info.tap_Buffer_Total_Samples_Counted) {
		/*  */
		/* There is not enough info in all captured data to accommodate */
		/* request because the time elapsed is too far ahead in time than */
		/* the buffers. This can happen when debugging. Additionally, */
		/* we could be off by a little bit. Accommodate slop by fetching */
		/* mildly stale data from last buffer point no further away from true */
		/* endpoint greater than equal to size of resolution of sample size */
		/* of our seconds ratio. (second_Ratio_For_Window) */
		/*  */
		if (total_Samples_To_Skip > (samples_Needed + audio_Tap_Ptr->info.tap_Buffer_Total_Samples_Counted - 1UL)) {
			goto Error_Handler;
		}
	}
	
	/*  */
	/* If two buffers filled then this is already pointing to oldest buffer.*/
	/* otherwise it is pointing to an empty buffer and needs to be avoided. */
	/*  */
	buffer_Index = audio_Tap_Ptr->info.next_Buffer;

	/*  */
	/* Start extraction from buffers. Check which buffer we need to */
	/* use. We could use time to help us, but we have something */
	/* far more accurate. We will be merely jumping back into the */
	/* older buffer by subtracting the number of bytes held in the newer buffer. */
	/*  */
	
	total_Buffers_Available = 2;
	if (audio_Tap_Ptr->info.tap_Buffer[1].tick_Time_Last_Called == 0) {
		/*  */
		/* Eliminate 1 in 4 billion chance of error when Mac running many days */
		/* by looking at a second field. */
		/*  */
		if (audio_Tap_Ptr->info.tap_Buffer[1].component_Recent_Info.sampleCount == 0) {
			total_Buffers_Available = 1;
		}
	}
	
	if (total_Buffers_Available > 1) {

		if ((audio_Tap_Ptr->info.tap_Buffer[0].component_Recent_Info.numChannels < 1) 
			|| (audio_Tap_Ptr->info.tap_Buffer[0].component_Recent_Info.numChannels > 32)
			|| (audio_Tap_Ptr->info.tap_Buffer[1].component_Recent_Info.numChannels < 1) 
			|| (audio_Tap_Ptr->info.tap_Buffer[1].component_Recent_Info.numChannels > 32)) {
			/*  */
			/* Component info is insane. We did this test before trusting its data */
			/*  */
			goto Error_Handler;
		}

		buffer_Index += 1;
		buffer_Index = buffer_Index & 0x01;
		
		if (total_Samples_To_Skip < (audio_Tap_Ptr->info.tap_Buffer_Total_Samples_Counted 
										- (CARDINAL_32)audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Info.sampleCount)) {
										
			samples_to_Ignore = (CARDINAL_32)audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Info.sampleCount;

			/*  */
			/* We definitely need the older buffer, move back pointer to the older buffer */
			/* and we will use it.
			/*  */
			buffer_Index += 1;
			buffer_Index = buffer_Index & 0x01;
		}
		else {
			samples_to_Ignore = 0;
		}
		
		/*  */
		/* Calculate where to end up in the buffer by subtracting its length */
		/* and samples_to_Ignore from the total count. */
		/*  */
		start_Sample_Point = audio_Tap_Ptr->info.tap_Buffer_Total_Samples_Counted 
					- (CARDINAL_32)audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Info.sampleCount;
					
		start_Sample_Point = start_Sample_Point - samples_to_Ignore;
		
		/*  */
		/* subtract this point from where we want to be to get true deviance. */
		/*  */
		start_Sample_Point = total_Samples_To_Skip - start_Sample_Point;
	
	}
	else {
		start_Sample_Point = total_Samples_To_Skip;
		buffer_Index = 0;

		if ((audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Info.numChannels < 1) 
			|| (audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Info.numChannels > 32)){
			/*  */
			/* Component info is insane. */
			/*  */
			goto Error_Handler;
		}
		
	} /* NOT if (total_Buffers_Available > 1) */
	
	/*  */
	/* Check for a very rare thing, but is possible if we took too long */
	/* to read this data and the buffer contents are in the middle of an I/O */
	/* that will disrupt its contents by filling with even newer data! */
	/*  */
	if (audio_Tap_Ptr->info.tap_Buffer[buffer_Index].data_untrusted_In_Transition == SAFE_TRUE) {
		goto Error_Handler;
	}
	
	/*  */
	/* Check for a very rare thing, but is possible if we took too long */
	/* to read this data and the buffer has just been filled with*/
	/* newer data from interrupt! This is very possible when taking longer than a second */
	/* while debugging or when not polling often enough to keep up */
	/*  */
	if ((CARDINAL_32)Calculate_Total_Ticks(audio_Tap_Ptr->info.tap_Buffer[buffer_Index].tick_Time_Last_Called, current_Tick_Time) > 0x40000000) {
		goto Error_Handler;
	}
	
	/*  */
	/* We know what buffer to use and the offset we want to use. */
	/* The offset may be slightly larger than the buffer contents but */
	/* if too far (unexpected) then we can just error out. */
	/*  */
				
	if ((audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Info.format != kOffsetBinary)
		&& (audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Info.format != kTwosComplement)){
		
		/*  */
		/* Component info shows compressed audio stream and we */
		/* are too early in the component chain to do anything about it. */
		/* This code only can average uncompressed data. */
		/*  */
		goto Error_Handler;
		
	}
	
	/*  */
	/* Provide room for 12 bit resolution and 16, 24 or 32 bit resolution sound */
	/* But round up to full bytes for vital elbow room to avoid having to mask */
	/* reading when buffer is returned. Of course this does not really save time */
	/* because buffer is being expanded in that case and work is performed lower down. */
	/*  */
	max_Sample_Size_Accectable = audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Info.sampleSize;
	if ((max_Sample_Size_Accectable & 0x0007) != 0) {
		max_Sample_Size_Accectable = max_Sample_Size_Accectable + 7;
		max_Sample_Size_Accectable = (max_Sample_Size_Accectable & 0xFFFFFFF8UL);
	}
	
	sample_Data_Buffer_Size = k_AVAILABLE_STACK_BYTES_FOR_AMPLITUDE_SCAN;

	/*  */
	/* It is critical to trim request if it is too big for our storage! */
	/*  */	
	if (((samples_Needed * max_Sample_Size_Accectable) >> 8) > sample_Data_Buffer_Size) {
		samples_Needed = ((sample_Data_Buffer_Size << 8) / max_Sample_Size_Accectable);
	}

	/*  */
	/* It is critical to trim request if it is more than 32 thousand samples because */
	/* loop structures use SIGNED loop 16 bit incrementors and besides that */
	/* more than 32 thousand is wasteful use of CPU and is not needed for */
	/* mere analog intensity measurement (power spectrum with one bin). */
	/*  */
	if ((samples_Needed & 0xFFFF8000) != 0) {
		samples_Needed = samples_Needed & 0x00007FFF;
	}
		
	/*  */
	/* Call the workhorse routine to strip out a channel of raw */
	/* sound from component buffer. */
	/*  */
	if (Extract_Estimated_Current_Samples_From_Tap(channel_Desired
															, audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Info.numChannels
															, audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Info.sampleSize
															, start_Sample_Point
															, samples_Needed
															, (CARDINAL_8 *)(*audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Buffer)
															, audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Info.sampleCount
															, max_Sample_Size_Accectable
															, (Ptr)&sample_Data[0]
															, &samples_Returned
															, &sample_Size_Returned) != SAFE_TRUE)	{
		goto Error_Handler;
	}
	
	/*  */
	/* First truncate samples returned to modulo 4 so that */
	/* if 8 bit, then 32 bit words can be used for type conversion */
	/* From 2's complement and natural Apple Audio */
	/* or for loop unrolling if 16, 24, or32 bits anyway. */
	/*  */
	samples_Returned = samples_Returned & 0xFFFFFFFC;

	/*  */
	/* Cool, now we just add up the amplitudes using deviations from */
	/* silence and adjusting if twos complement first. */
	/* Then we divide by amount of samples and then we apply */
	/* the function that helps adjust fact that max volume is not see in this */
	/* measurement yet we want it multiplied by two or so and have */
	/* an activation power spectrum curve apllied to it as well. */
	/*  */
		
	/*  */
	/* HACK HACK HACK do conversion here soon */
	/*  */
	
	/*  */
	/* Add up the samples */
	/*  */
	
	bytes_Per_Sample = sample_Size_Returned >> 3UL;
	if ((bytes_Per_Sample << 3UL)  < sample_Size_Returned) {
		/*  */
		/* If sample occupies less bits than given, adjust. Expected with 12 bit sound? */
		/*  */
		bytes_Per_Sample += 1;
	}
	
	if (audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Info.format == kTwosComplement) {
		if (Get_Average_Wave_Amplitude(TRUE
												, bytes_Per_Sample
												, samples_Returned
												, &sample_Data[0]
												, amplitude) != SAFE_TRUE) {
			goto Error_Handler;
		}
	}
	else {
		if (Get_Average_Wave_Amplitude(FALSE
												, bytes_Per_Sample
												, samples_Returned
												, &sample_Data[0]
												, amplitude) != SAFE_TRUE) {
			goto Error_Handler;
		}	
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
	
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean Get_Current_Amplitude_From_Tap... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Extract_Estimated_Current_Samples_From_Tap(CARDINAL_32 channel_Desired
																	, CARDINAL_32 total_Channels
																	, CARDINAL_32 sample_Size
																	, CARDINAL_32 start_Sample_Point
																	, CARDINAL_32 original_Samples_Needed
																	, CARDINAL_8 *source_Input
																	, CARDINAL_32 source_Input_Available_Samples
																	, CARDINAL_32 max_Sample_Size_Accectable
																	, Ptr samples_Destination
																	, CARDINAL_32 *samples_Returned
																	, CARDINAL_32 *sample_Size_Returned)
/*  */
/* This routine copies a chunk of bytes to another piece of ram */
/* and then while piece is safely snatched they operates on the data */
/* estimating amplitude. The data being snatched is already a frozen copy */
/* safely snatched now because there was no way to do it without interrupts */
/* conflicting. Interrupts can try to thwart attempt and should not cause problems */
/* because of our special flag "tap_Buffer[buffer_Index].data_untrusted_In_Transition" */
/*  */
{
	CARDINAL_32	samples_Needed;
	CARDINAL_32	packet_Bit_Size;
	CARDINAL_32	packet_Byte_Size;
	CARDINAL_32	start_Bit_Offset;
	CARDINAL_32	start_Byte_Offset;
	CARDINAL_32	sample_Byte_Size;
	CARDINAL_32	samples_Destination_Address;
	CARDINAL_32	actual_Start_Address;
	CARDINAL_32	temp_Value;
	SIMPLE_LOOP	loop;
	Safe_Boolean		function_Result;
/* END VAR */


	if ((max_Sample_Size_Accectable > 32) || (max_Sample_Size_Accectable < 8)) {
		goto Error_Handler;
	}
	
	*samples_Returned = 0;
	*sample_Size_Returned = sample_Size;
	
	samples_Needed = original_Samples_Needed;
	
	if (samples_Needed > source_Input_Available_Samples) {
		samples_Needed = source_Input_Available_Samples;
		if (samples_Needed < k_BARE_MINIMUM_USEFUL_SAMPLES) {
			goto Error_Handler;
		}
	}

	if ((start_Sample_Point + samples_Needed) > source_Input_Available_Samples) {
		/*  */
		/* Pin to end of buffer unless way too far off */
		/*  */
	
		if (source_Input_Available_Samples >= samples_Needed) {
			if (start_Sample_Point < source_Input_Available_Samples) {
				start_Sample_Point = source_Input_Available_Samples - samples_Needed;
			}
			else {
				/*  */
				/* If we just accept the end of the buffer we are performing a disservice. */
				/* There is not a single byte accurately in this buffer that we want. */
				/* it is important to allow a little bit of slop  (probabably about 50% of */
				/* one ticks worth of information.) */
				/* This calculation actually saves much more CPU time than it wastes to calculate */
				/* but instead, a single slop buffer of one fraction of the desired */
				/* original sample request is allowed. */
				/*  */
				if ((start_Sample_Point <= source_Input_Available_Samples + original_Samples_Needed)) {
					start_Sample_Point = source_Input_Available_Samples - samples_Needed;
				}
				else {
					/*  */
					/* Too much time has elapsed to benefit from using this buffer. */
					/*  */
					goto Error_Handler;
				}
			}
		}
		else {
			/*  */
			/* This was already adjusted at start of function and should not happen */
			/*  */
			goto Error_Handler;
		}
	
	} /* if ((start_Sample_Point + samples_Needed) > source_Input_Available_Samples)  */
	
	if ((total_Channels != 1) || (sample_Size != 8)) {
	
		/*  */
		/*  */
		/*  */
		/*  */
		/*  */
		/* HACK HACK HACK : For Debugging only! Zero out result for safety */
		/*  */
		Fill_Byte_Memory_With_Byte(samples_Destination, ((samples_Needed * max_Sample_Size_Accectable) >> 3L), 0);
	
	
		packet_Bit_Size = (CARDINAL_32)total_Channels;
		packet_Bit_Size = packet_Bit_Size * (CARDINAL_32)sample_Size;

		start_Bit_Offset = start_Sample_Point * packet_Bit_Size;
		
		if (total_Channels != 1) {
			if (channel_Desired != 0) {
				/*  */
				/* Increment to beginning of proper phase */
				/*  */
				start_Bit_Offset += (channel_Desired * sample_Size);
			}
		}
		
		if ((start_Bit_Offset & 0x03) != 0) {
			/*  */
			/* HACK HACK HACK : We cannot deal with nibble offsets yet */
			/*  */
			goto Error_Handler;
		}
		
		if ((sample_Size & 0x03) != 0) {
			/*  */
			/* HACK HACK HACK : We cannot deal with nibbles yet */
			/* but may add soon. */
			/*  */
			actual_Start_Address = (CARDINAL_32)source_Input;
			
		/* ===
			for (loop = 0; (loop < samples_Needed); loop++) {
			
				actual_Start_Address
			
				start_Bit_Offset += packet_Bit_Size;
				samples_Destination_Address += sample_Size;
			}
		==== */
			
			/*  */
			/*  */
			/*  */
			/*  */
			/*  */
			/*  */
			/* HACK HACK HACK : IGNORE This type of file for now. Enhance soon */
			/*  */
			DebugStr("\pUnfinished programming in Extract_Estimated_Current_Samples_From_Tap");
			
		}
		else {
			/*  */
			/* Samples in channel are 1, 2, 3, or 4 bytes in size. */
			/* This handles extraction from any number of channels too. */
			/*  */
			
			actual_Start_Address = (CARDINAL_32)source_Input;
			
			packet_Byte_Size = packet_Bit_Size >> 3;
			sample_Byte_Size = sample_Size >> 3;
			start_Byte_Offset = start_Bit_Offset >> 3;
			samples_Destination_Address = (CARDINAL_32)samples_Destination;
			
			switch (sample_Byte_Size) {
				case 1:
					for (loop = 0; (loop < samples_Needed); loop++) {
						*(CARDINAL_8 *)samples_Destination_Address = *(CARDINAL_8 *)(actual_Start_Address + start_Byte_Offset);
					
						start_Byte_Offset += packet_Byte_Size;
						samples_Destination_Address += sample_Byte_Size;
					}
				break;
				
				case 2:
					for (loop = 0; (loop < samples_Needed); loop++) {
						*(CARDINAL_16 *)samples_Destination_Address = *(CARDINAL_16 *)(actual_Start_Address + start_Byte_Offset);
					
						start_Byte_Offset += packet_Byte_Size;
						samples_Destination_Address += sample_Byte_Size;
					}
				break;
				
				case 3:
					/*  */
					/* Twenty four bit sound. (some 18 and 20 bit A-D converters are common in DSP world) */
					/*  */
					for (loop = 0; (loop < samples_Needed); loop++) {
					
						temp_Value = *(CARDINAL_32 *)samples_Destination_Address;
						temp_Value = temp_Value & 0x000000FF;
						temp_Value = temp_Value | (*(CARDINAL_32 *)(actual_Start_Address + start_Byte_Offset) << 8);
						*(CARDINAL_32 *)samples_Destination_Address = temp_Value;
						
						start_Byte_Offset += packet_Byte_Size;
						samples_Destination_Address += sample_Byte_Size;
					}
				break;
				
				case 4:
					for (loop = 0; (loop < samples_Needed); loop++) {
						*(CARDINAL_32 *)samples_Destination_Address = *(CARDINAL_32 *)(actual_Start_Address + start_Byte_Offset);
					
						start_Byte_Offset += packet_Byte_Size;
						samples_Destination_Address += sample_Byte_Size;
					}
				break;
				
				default:
					goto Error_Handler;
			}
		}
		
		/*  */
		/* All done! */
		/*  */
		*samples_Returned = samples_Needed;

	}
	else {
		/*  */
		/* Simple single channel 8 bit. Pull back buffer if needed. */
		/*  */
		
		*samples_Returned = samples_Needed;
		actual_Start_Address = (CARDINAL_32)source_Input + start_Sample_Point;
		BlockMoveData((Ptr)actual_Start_Address, samples_Destination, samples_Needed);
		
	} /* NOT if ((total_Channels != 1) || (sample_Size != 8))  */
	

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
	
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean Extract_Estimated_Current_Samples_From_Tap... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Get_Average_Wave_Amplitude(Boolean twos_Complement_Storage
													, CARDINAL_32 bytes_Per_Sample
													, CARDINAL_32 samples_In_Buffer
													, CARDINAL_8 *sample_Data
													, CARDINAL_8 *amplitude)
/*  */
/* This routine should be written in assembly to conserve CPU! */
/* It is only called by Get_Current_Amplitude_From_Tap */
/*  */
{
	register CARDINAL_8		*sample_Ptr;
	register CARDINAL_8		data;
	register CARDINAL_32		temp_Value;
	register CARDINAL_32		total_Amplitude;
	register CARDINAL_32		array_Index_Location;
	register SIMPLE_LOOP		loop;
	CARDINAL_32				average_Amplitude;
	Safe_Boolean					function_Result;
/* END VAR */


	function_Result = SAFE_FALSE;
	*amplitude = 0;
	
	if ((bytes_Per_Sample > 4) || (bytes_Per_Sample < 1)) {
		goto Error_Handler;
	}

	sample_Ptr = sample_Data;
	
	total_Amplitude = 0;
	array_Index_Location = 0;
	
	if (bytes_Per_Sample == 1) {
		/*  */
		/* Add absolute values (deviations) from center of wave */
		/*  */
		if (twos_Complement_Storage == TRUE) {
			for (loop = 0; (loop < samples_In_Buffer); loop++) {
				data = sample_Ptr[loop];
				if (data & 0x80) {
					total_Amplitude -= (INTEGER_8)data;
				}
				else {
					total_Amplitude += data;
				}
			}
		}
		else {
			for (loop = 0; (loop < samples_In_Buffer); loop++) {
				data = (sample_Ptr[loop] ^ 0x80);
				if (data & 0x80) {
					total_Amplitude -= (INTEGER_8)data;
				}
				else {
					total_Amplitude += data;
				}
			}
		}
		
		/*  */
		/* Double the amplitude because shape of wave includes */
		/* 50% of its power under 50% of the midpoint of the wave */
		/* IRRESPECTIVE of the fact that we added only deviations */
		/* from wave midpoint. This statement is in error if the wave is a */
		/* square wave, but I feel most wave data is fairly sinosoidal */
		/* and needs amplification for accuracy. We will pin it to max if */
		/* needed after conversion */
		/*  */
		total_Amplitude = total_Amplitude << 1;
		
		
		/*  */
		/*  */
		/*  */
		/*  */
		/*  */
		/*  */
		/*  */
		/*  */
		/*  */
		/*  */
		/*  */
		/*  */
		/*  */
		/*  */
		/*  */
		/* HACK HACK HACK make it 4 times louder for hell of it!!! */
		/*  */
		total_Amplitude = total_Amplitude << 1;
		
		
		
		average_Amplitude = (total_Amplitude / samples_In_Buffer);
		/*  */
		/* Pin to byte maximum */ 
		/*  */
		if ((average_Amplitude & 0xFFFFFF00) != 0) {
			average_Amplitude = 0x00FF;
		}
		*amplitude = average_Amplitude;
	}
	else {
		if (bytes_Per_Sample == 2) {
			for (loop = 0; (loop < samples_In_Buffer); loop = loop + 1) {
			
				total_Amplitude += *(CARDINAL_16 *)&sample_Ptr[array_Index_Location];
			
				array_Index_Location += 2;
			}
			average_Amplitude = (total_Amplitude / samples_In_Buffer);
			
			/*  */
			/* Double the amplitude because shape of wave includes */
			/* 50% of its power under 50% of the midpoint of the wave */
			/* if sinosoidal characteristic. */
			/*  */
			average_Amplitude = average_Amplitude << 1;
			
			/*  */
			/* Pin to byte maximum */ 
			/*  */
			if ((average_Amplitude & 0xFFFFFF00) != 0) {
				average_Amplitude = 0x00FF;
			}
			*amplitude = average_Amplitude;
		}
		else {
			if (bytes_Per_Sample == 3) {
			
				/*  */
				/* HACK HACK HACK Needs better solution. Can be remedied easily */
				/* by feeder promoting to 32 bit if room exists */
				/* Ugh! how annoying! Misaligned data! Very slow on PowerPC */
				/* semi-slow on Mac, and illegal on Mac+ */
				/*  */
				for (loop = 0; (loop < samples_In_Buffer); loop = loop + 1) {
				
					temp_Value = *(CARDINAL_32 *)&sample_Ptr[array_Index_Location];
					total_Amplitude += (temp_Value & 0x00FFFFFF);
				
					array_Index_Location += 3;
				}
				average_Amplitude = (total_Amplitude / samples_In_Buffer);
				
				average_Amplitude = average_Amplitude << 1;
				
				/*  */
				/* Pin to byte maximum */ 
				/*  */
				if ((average_Amplitude & 0xFFFFFF00) != 0) {
					average_Amplitude = 0x00FF;
				}
				*amplitude = average_Amplitude;
			}
			else {
				for (loop = 0; (loop < samples_In_Buffer); loop = loop + 1) {
				
					temp_Value = *(CARDINAL_32 *)&sample_Ptr[array_Index_Location];
				
					array_Index_Location += bytes_Per_Sample;
				}
				average_Amplitude = (total_Amplitude / samples_In_Buffer);
				
				average_Amplitude = average_Amplitude << 1;
				
				/*  */
				/* Pin to byte maximum */ 
				/*  */
				if ((average_Amplitude & 0xFFFFFF00) != 0) {
					average_Amplitude = 0x00FF;
				}
			}
		}
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
	
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean Get_Average_Wave_Amplitude... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Delay_Tap_Until_Good_Opportunity(Ptr	generic_Caste_Ptr)
/*  */
/* This is a special purpose routine that waits for channel activity to */
/* show that some of our callbacks have been called or that the */
/* channel is now capable of having its "moreRtn" swapped. */
/* When starting up a file play, I try very hard to not lose any of the */
/* first buffer. Currently there is no solution becaue the first buffer is filled */
/* in conjunction with the channel issuing the start of a file play command */
/* Even after a pause, even with a callback, the only soluton is to do it */
/* in a non-deterministic manner using fast polling interrupt time routine */
/* but that has never been tried yet. This is good enough for now. */
/* if losing a buffers worth of sound (26K in my setup) is unacceptable, then */
/* use overlapping sound file playing and always place a leader with waste at the start of the sound */
/*  */
{
	CARDINAL_32	starting_Callback_Count;
	CARDINAL_32	starting_Tick_Time;
	CARDINAL_32	elapsed_Time;
	Safe_Boolean		function_Result;
	Sound_Manager_Private_Ptr	private_Info_Ptr;
	Async_AIFF_Ptr	playback;
	Boolean			still_Searching;
/* END VAR */	


	function_Result = SAFE_FALSE;
	
	playback = (Async_AIFF_Ptr)generic_Caste_Ptr;
	
	if (playback->audio_Tap_Initialized != SAFE_TRUE) {
		
		starting_Callback_Count = playback->callback_Count;
		starting_Tick_Time = TickCount();
		if (playback->callback_Debug_Info_More_Data_Ptr == 0) {
		
			still_Searching = TRUE;
			do {
				Delay_For_One_Tick_Yielding_To_System();
		/* ===
				if (playback->callback_Count != starting_Callback_Count) {
					still_Searching = FALSE;
				}
		=== */
		
				private_Info_Ptr = (Sound_Manager_Private_Ptr)playback->active_Snd_Channel_Ptr->firstMod;
				if ((CARDINAL_32)private_Info_Ptr->source_Sound_PB.moreRtn != 0) {
					still_Searching = FALSE;
				}
					
				elapsed_Time = Calculate_Total_Ticks(starting_Tick_Time, TickCount());
				if (elapsed_Time > (1UL * 60UL)) {
					still_Searching = FALSE;
				}
	
			} while (still_Searching == TRUE);
		
		} /* if (playback->callback_Debug_Info_More_Data_Ptr == 0) */
	
		if (playback->audio_Tap_Initialized != SAFE_TRUE) {
			function_Result = Install_Tap_On_Get_More_Data(playback->active_Snd_Channel_Ptr, 0, &playback->audio_Tap_Ptr);
				/*  */
				/* Not normally expected to fail if number of taps avaiable */
				/* is equal to number of open sounds playing. */
				/*  */
			if (function_Result == SAFE_TRUE) {
				playback->audio_Tap_Initialized = SAFE_TRUE;
			}
		}

	} /* if (playback->audio_Tap_Initialized != SAFE_TRUE) */
	
	return(function_Result);
		
} /* Safe_Boolean Delay_Tap_Until_Good_Opportunity ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
#if (ORIGINAL_MAC_SYSTEM == 1)
pascal Boolean Swap_More_Data_Callback_1(SoundParamBlockPtr	*sound_Source_Ptr)
/*  */
/* This routine is a "SoundParamUPP" that is called almost the same way a */
/* completion routine is called but instead is used to call the function */
/* that loads more data into a sound routine. It actually just invokes the */
/* address of the real caller that was swapped when the tap was started. */
/*  */
{
	CARDINAL_32					index;
	Audio_Progress_Tap_Ptr		audio_Tap_Ptr;
	CARDINAL_32					bytes_To_Copy;
	pascal Boolean (*address_Of_More_Data_Routine)(SoundParamBlockPtr *);
	volatile Boolean					function_Result;
	SndChannelPtr					active_Snd_Channel_Ptr;
	Sound_Manager_Private_Ptr	private_Info_Ptr;
	SoundComponentData			*real_Component_Data;
	CARDINAL_16					buffer_Index;
/* END VAR */


	Setup_A5();
	asm {
		movem.l	d0-d1/a0-a2,  -(SP)		; Save registers that might be altered by compiler (not needed)
	}
	

	/*  */
	/* Each instance of Swap_More_Data_Callback_X is identical except */
	/* for this assignment. */
	/*  */
	index = 0;
	
	
	function_Result = FALSE;
	
	audio_Tap_Ptr = &g_Audio_Taps[index];
	if (audio_Tap_Ptr->info.active_And_In_Use == SAFE_TRUE) {
	
		buffer_Index = audio_Tap_Ptr->info.next_Buffer;
		audio_Tap_Ptr->info.tap_Buffer[buffer_Index].data_untrusted_In_Transition = SAFE_TRUE;

		address_Of_More_Data_Routine = (void *)audio_Tap_Ptr->original_Get_More_Data_Function;
	
		asm {
			movem.l	(SP)+,d0-d1/a0-a2	; Restore altered registers
		} /* asm */
		Restore_A5();
		function_Result = address_Of_More_Data_Routine(sound_Source_Ptr);
		Setup_A5();
		asm {
			movem.l	d0-d1/a0-a2,  -(SP)		; Save registers that might be altered by compiler (not needed)
		}
		
		if (function_Result != FALSE) {
		
			real_Component_Data = &(*sound_Source_Ptr)->desc;
			
			/*  */
			/* That is the official way to get the component info. The other way */
			/* shown below results in same data and is just for an example, but */
			/* it demonstrates how to get to other parameters if the direct information */
			/* is too limited in scope to use. */
			/*  */
		/* ====
			active_Snd_Channel_Ptr = (SndChannelPtr)((*sound_Source_Ptr)->refCon);
			private_Info_Ptr = (Sound_Manager_Private_Ptr)active_Snd_Channel_Ptr->firstMod;
			real_Component_Data = &private_Info_Ptr->source_Sound_PB.desc;
		==== */
		
			if ((real_Component_Data->numChannels > 0) && (real_Component_Data->numChannels < 33)) {
				audio_Tap_Ptr->info.tap_Buffer[buffer_Index].tick_Time_Last_Called = TickCount();
				audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Info = *real_Component_Data;
			
				bytes_To_Copy = (CARDINAL_32)real_Component_Data->numChannels;
				bytes_To_Copy = bytes_To_Copy * (CARDINAL_32)real_Component_Data->sampleSize;
				bytes_To_Copy = bytes_To_Copy * (CARDINAL_32)real_Component_Data->sampleCount;
				if (bytes_To_Copy > audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Buffer_Max_Size) {
					bytes_To_Copy = audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Buffer_Max_Size;
				}
				if (audio_Tap_Ptr->info.tap_Buffer_Total_Samples_Counted == 0) {
					audio_Tap_Ptr->info.tick_Time_First_Setup = audio_Tap_Ptr->info.tap_Buffer[buffer_Index].tick_Time_Last_Called;
				}
				audio_Tap_Ptr->info.tap_Buffer_Total_Samples_Counted += (CARDINAL_32)real_Component_Data->sampleCount;

				BlockMoveData(audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Info.buffer , *audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Buffer, bytes_To_Copy);
				audio_Tap_Ptr->info.next_Buffer += 1;
				/*  */
				/* Modulo 2 wrap around to first buffer again. */
				/*  */
				audio_Tap_Ptr->info.next_Buffer = (audio_Tap_Ptr->info.next_Buffer & 0x0001);
			}
		} /* if (function_Result != FALSE)  */
		
		audio_Tap_Ptr->info.tap_Buffer[buffer_Index].data_untrusted_In_Transition = SAFE_FALSE;
	}
	
	asm {
		movem.l	(SP)+,d0-d1/a0-a2	; Restore altered registers
	} /* asm */
	Restore_A5();
	
	return(function_Result);
	
} /* pascal void Swap_More_Data_Callback_1 ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
pascal Boolean Swap_More_Data_Callback_2(SoundParamBlockPtr	*sound_Source_Ptr)
{
	CARDINAL_32					index;
	Audio_Progress_Tap_Ptr		audio_Tap_Ptr;
	CARDINAL_32					bytes_To_Copy;
	pascal Boolean (*address_Of_More_Data_Routine)(SoundParamBlockPtr *);
	volatile Boolean					function_Result;
	SndChannelPtr					active_Snd_Channel_Ptr;
	Sound_Manager_Private_Ptr	private_Info_Ptr;
	SoundComponentData			*real_Component_Data;
	CARDINAL_16					buffer_Index;
/* END VAR */


	Setup_A5();
	asm {
		movem.l	d0-d1/a0-a2,  -(SP)		; Save registers that might be altered by compiler (not needed)
	}
	
	index = 1;
	
	function_Result = FALSE;
	
	audio_Tap_Ptr = &g_Audio_Taps[index];
	if (audio_Tap_Ptr->info.active_And_In_Use == SAFE_TRUE) {
	
		buffer_Index = audio_Tap_Ptr->info.next_Buffer;
		audio_Tap_Ptr->info.tap_Buffer[buffer_Index].data_untrusted_In_Transition = SAFE_TRUE;

		address_Of_More_Data_Routine = (void *)audio_Tap_Ptr->original_Get_More_Data_Function;
	
		asm {
			movem.l	(SP)+,d0-d1/a0-a2	; Restore altered registers
		} /* asm */
		Restore_A5();
		function_Result = address_Of_More_Data_Routine(sound_Source_Ptr);
		Setup_A5();
		asm {
			movem.l	d0-d1/a0-a2,  -(SP)		; Save registers that might be altered by compiler (not needed)
		}

		
		if (function_Result != FALSE) {
		
			real_Component_Data = &(*sound_Source_Ptr)->desc;
			
			/*  */
			/* That is the official way to get the component info. The other way */
			/* shown below results in same data and is just for an example, but */
			/* it demonstrates how to get to other parameters if the direct information */
			/* is too limited in scope to use. */
			/*  */
		/* ====
			active_Snd_Channel_Ptr = (SndChannelPtr)((*sound_Source_Ptr)->refCon);
			private_Info_Ptr = (Sound_Manager_Private_Ptr)active_Snd_Channel_Ptr->firstMod;
			real_Component_Data = &private_Info_Ptr->source_Sound_PB.desc;
		==== */
		
			if ((real_Component_Data->numChannels > 0) && (real_Component_Data->numChannels < 33)) {
				audio_Tap_Ptr->info.tap_Buffer[buffer_Index].tick_Time_Last_Called = TickCount();
				audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Info = *real_Component_Data;
			
				bytes_To_Copy = (CARDINAL_32)real_Component_Data->numChannels;
				bytes_To_Copy = bytes_To_Copy * (CARDINAL_32)real_Component_Data->sampleSize;
				bytes_To_Copy = bytes_To_Copy * (CARDINAL_32)real_Component_Data->sampleCount;
				if (bytes_To_Copy > audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Buffer_Max_Size) {
					bytes_To_Copy = audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Buffer_Max_Size;
				}
				if (audio_Tap_Ptr->info.tap_Buffer_Total_Samples_Counted == 0) {
					audio_Tap_Ptr->info.tick_Time_First_Setup = audio_Tap_Ptr->info.tap_Buffer[buffer_Index].tick_Time_Last_Called;
				}
				audio_Tap_Ptr->info.tap_Buffer_Total_Samples_Counted += (CARDINAL_32)real_Component_Data->sampleCount;

				BlockMoveData(audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Info.buffer , *audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Buffer, bytes_To_Copy);
				audio_Tap_Ptr->info.next_Buffer += 1;
				audio_Tap_Ptr->info.next_Buffer = (audio_Tap_Ptr->info.next_Buffer & 0x0001);
			}
		} /* if (function_Result != FALSE)  */
		
		audio_Tap_Ptr->info.tap_Buffer[buffer_Index].data_untrusted_In_Transition = SAFE_FALSE;
	}
	
	asm {
		movem.l	(SP)+,d0-d1/a0-a2	; Restore altered registers
	} /* asm */
	Restore_A5();
	
	return(function_Result);
	
} /* pascal void Swap_More_Data_Callback_2 ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
pascal Boolean Swap_More_Data_Callback_3(SoundParamBlockPtr	*sound_Source_Ptr)
{
	CARDINAL_32					index;
	Audio_Progress_Tap_Ptr		audio_Tap_Ptr;
	CARDINAL_32					bytes_To_Copy;
	pascal Boolean (*address_Of_More_Data_Routine)(SoundParamBlockPtr *);
	volatile Boolean					function_Result;
	SndChannelPtr					active_Snd_Channel_Ptr;
	Sound_Manager_Private_Ptr	private_Info_Ptr;
	SoundComponentData			*real_Component_Data;
	CARDINAL_16					buffer_Index;
/* END VAR */


	Setup_A5();
	asm {
		movem.l	d0-d1/a0-a2,  -(SP)		; Save registers that might be altered by compiler (not needed)
	}
	
	index = 2;
	
	function_Result = FALSE;
	
	audio_Tap_Ptr = &g_Audio_Taps[index];
	if (audio_Tap_Ptr->info.active_And_In_Use == SAFE_TRUE) {
	
		buffer_Index = audio_Tap_Ptr->info.next_Buffer;
		audio_Tap_Ptr->info.tap_Buffer[buffer_Index].data_untrusted_In_Transition = SAFE_TRUE;

		address_Of_More_Data_Routine = (void *)audio_Tap_Ptr->original_Get_More_Data_Function;
	
		asm {
			movem.l	(SP)+,d0-d1/a0-a2	; Restore altered registers
		} /* asm */
		Restore_A5();
		function_Result = address_Of_More_Data_Routine(sound_Source_Ptr);
		Setup_A5();
		asm {
			movem.l	d0-d1/a0-a2,  -(SP)		; Save registers that might be altered by compiler (not needed)
		}


		if (function_Result != FALSE) {
		
			real_Component_Data = &(*sound_Source_Ptr)->desc;
			
			/*  */
			/* That is the official way to get the component info. The other way */
			/* shown below results in same data and is just for an example, but */
			/* it demonstrates how to get to other parameters if the direct information */
			/* is too limited in scope to use. */
			/*  */
		/* ====
			active_Snd_Channel_Ptr = (SndChannelPtr)((*sound_Source_Ptr)->refCon);
			private_Info_Ptr = (Sound_Manager_Private_Ptr)active_Snd_Channel_Ptr->firstMod;
			real_Component_Data = &private_Info_Ptr->source_Sound_PB.desc;
		==== */
		
			if ((real_Component_Data->numChannels > 0) && (real_Component_Data->numChannels < 33)) {
				audio_Tap_Ptr->info.tap_Buffer[buffer_Index].tick_Time_Last_Called = TickCount();
				audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Info = *real_Component_Data;
			
				bytes_To_Copy = (CARDINAL_32)real_Component_Data->numChannels;
				bytes_To_Copy = bytes_To_Copy * (CARDINAL_32)real_Component_Data->sampleSize;
				bytes_To_Copy = bytes_To_Copy * (CARDINAL_32)real_Component_Data->sampleCount;
				if (bytes_To_Copy > audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Buffer_Max_Size) {
					bytes_To_Copy = audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Buffer_Max_Size;
				}
				if (audio_Tap_Ptr->info.tap_Buffer_Total_Samples_Counted == 0) {
					audio_Tap_Ptr->info.tick_Time_First_Setup = audio_Tap_Ptr->info.tap_Buffer[buffer_Index].tick_Time_Last_Called;
				}
				audio_Tap_Ptr->info.tap_Buffer_Total_Samples_Counted += (CARDINAL_32)real_Component_Data->sampleCount;

				BlockMoveData(audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Info.buffer , *audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Buffer, bytes_To_Copy);
				audio_Tap_Ptr->info.next_Buffer += 1;
				audio_Tap_Ptr->info.next_Buffer = (audio_Tap_Ptr->info.next_Buffer & 0x0001);
			}
		} /* if (function_Result != FALSE)  */
		
		
		audio_Tap_Ptr->info.tap_Buffer[buffer_Index].data_untrusted_In_Transition = SAFE_FALSE;
	}
	
	asm {
		movem.l	(SP)+,d0-d1/a0-a2	; Restore altered registers
	} /* asm */
	Restore_A5();
	
	return(function_Result);
	
} /* pascal void Swap_More_Data_Callback_3 ... */
/*=**************************************************************************=*/
#else

/*=**************************************************************************=*/
pascal Boolean Swap_More_Data_Callback_1(SoundParamBlockPtr	*sound_Source_Ptr)
{
/*  */
/* This routine is a "SoundParamUPP" that is called almost the same way a */
/* completion routine is called but instead is used to call the function */
/* that loads more data into a sound routine. It actually just invokes the */
/* address of the real caller that was swapped when the tap was started. */
/*  */
{
	CARDINAL_32					index;
	Audio_Progress_Tap_Ptr		audio_Tap_Ptr;
	CARDINAL_32					bytes_To_Copy;
	volatile Boolean					function_Result;
	SoundComponentData			*real_Component_Data;
	CARDINAL_16					buffer_Index;
/* END VAR */

	
	/*  */
	/* Each instance of Swap_More_Data_Callback_X is identical except */
	/* for this assignment. */
	/*  */
	index = 0;
	function_Result = FALSE;
	
	audio_Tap_Ptr = &g_Audio_Taps[index];
	if (audio_Tap_Ptr->info.active_And_In_Use == SAFE_TRUE) {
	
		buffer_Index = audio_Tap_Ptr->info.next_Buffer;
		audio_Tap_Ptr->info.tap_Buffer[buffer_Index].data_untrusted_In_Transition = SAFE_TRUE;
		function_Result = CallSoundParamProc(audio_Tap_Ptr->original_Get_More_Data_UPP, sound_Source_Ptr);

		
		if (function_Result != FALSE) {
		
			real_Component_Data = &(*sound_Source_Ptr)->desc;		
			if ((real_Component_Data->numChannels > 0) && (real_Component_Data->numChannels < 33)) {
				audio_Tap_Ptr->info.tap_Buffer[buffer_Index].tick_Time_Last_Called = TickCount();
				audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Info = *real_Component_Data;
			
				bytes_To_Copy = (CARDINAL_32)real_Component_Data->numChannels;
				bytes_To_Copy = bytes_To_Copy * (CARDINAL_32)real_Component_Data->sampleSize;
				bytes_To_Copy = bytes_To_Copy * (CARDINAL_32)real_Component_Data->sampleCount;
				if (bytes_To_Copy > audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Buffer_Max_Size) {
					bytes_To_Copy = audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Buffer_Max_Size;
				}
				if (audio_Tap_Ptr->info.tap_Buffer_Total_Samples_Counted == 0) {
					audio_Tap_Ptr->info.tick_Time_First_Setup = audio_Tap_Ptr->info.tap_Buffer[buffer_Index].tick_Time_Last_Called;
				}
				audio_Tap_Ptr->info.tap_Buffer_Total_Samples_Counted += (CARDINAL_32)real_Component_Data->sampleCount;

				BlockMoveData(audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Info.buffer , *audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Buffer, bytes_To_Copy);
				audio_Tap_Ptr->info.next_Buffer += 1;
				audio_Tap_Ptr->info.next_Buffer = (audio_Tap_Ptr->info.next_Buffer & 0x0001);
			}
		} /* if (function_Result != FALSE)  */
		
		
		audio_Tap_Ptr->info.tap_Buffer[buffer_Index].data_untrusted_In_Transition = SAFE_FALSE;
	}
		
	return(function_Result);
	
} /* pascal void Swap_More_Data_Callback_1 ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
pascal Boolean Swap_More_Data_Callback_2(SoundParamBlockPtr	*sound_Source_Ptr)
{
{
	CARDINAL_32					index;
	Audio_Progress_Tap_Ptr		audio_Tap_Ptr;
	CARDINAL_32					bytes_To_Copy;
	volatile Boolean					function_Result;
	SoundComponentData			*real_Component_Data;
	CARDINAL_16					buffer_Index;
/* END VAR */

	
	index = 1;
	function_Result = FALSE;
	
	audio_Tap_Ptr = &g_Audio_Taps[index];
	if (audio_Tap_Ptr->info.active_And_In_Use == SAFE_TRUE) {
		buffer_Index = audio_Tap_Ptr->info.next_Buffer;
		audio_Tap_Ptr->info.tap_Buffer[buffer_Index].data_untrusted_In_Transition = SAFE_TRUE;
		
		function_Result = CallSoundParamProc(audio_Tap_Ptr->original_Get_More_Data_UPP, sound_Source_Ptr);
		
		if (function_Result != FALSE) {
		
			real_Component_Data = &(*sound_Source_Ptr)->desc;		
			if ((real_Component_Data->numChannels > 0) && (real_Component_Data->numChannels < 33)) {
				audio_Tap_Ptr->info.tap_Buffer[buffer_Index].tick_Time_Last_Called = TickCount();
				audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Info = *real_Component_Data;
			
				bytes_To_Copy = (CARDINAL_32)real_Component_Data->numChannels;
				bytes_To_Copy = bytes_To_Copy * (CARDINAL_32)real_Component_Data->sampleSize;
				bytes_To_Copy = bytes_To_Copy * (CARDINAL_32)real_Component_Data->sampleCount;
				if (bytes_To_Copy > audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Buffer_Max_Size) {
					bytes_To_Copy = audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Buffer_Max_Size;
				}
				if (audio_Tap_Ptr->info.tap_Buffer_Total_Samples_Counted == 0) {
					audio_Tap_Ptr->info.tick_Time_First_Setup = audio_Tap_Ptr->info.tap_Buffer[buffer_Index].tick_Time_Last_Called;
				}
				audio_Tap_Ptr->info.tap_Buffer_Total_Samples_Counted += (CARDINAL_32)real_Component_Data->sampleCount;

				BlockMoveData(audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Info.buffer , *audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Buffer, bytes_To_Copy);
				audio_Tap_Ptr->info.next_Buffer += 1;
				audio_Tap_Ptr->info.next_Buffer = (audio_Tap_Ptr->info.next_Buffer & 0x0001);
			}
		} /* if (function_Result != FALSE)  */
		
		audio_Tap_Ptr->info.tap_Buffer[buffer_Index].data_untrusted_In_Transition = SAFE_FALSE;
	}
		
	return(function_Result);
	
} /* pascal void Swap_More_Data_Callback_2 ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
pascal Boolean Swap_More_Data_Callback_3(SoundParamBlockPtr	*sound_Source_Ptr)
{
{
	CARDINAL_32					index;
	Audio_Progress_Tap_Ptr		audio_Tap_Ptr;
	CARDINAL_32					bytes_To_Copy;
	volatile Boolean					function_Result;
	SoundComponentData			*real_Component_Data;
	CARDINAL_16					buffer_Index;
/* END VAR */

	
	index = 2;
	function_Result = FALSE;
	
	audio_Tap_Ptr = &g_Audio_Taps[index];
	if (audio_Tap_Ptr->info.active_And_In_Use == SAFE_TRUE) {
		buffer_Index = audio_Tap_Ptr->info.next_Buffer;
		audio_Tap_Ptr->info.tap_Buffer[buffer_Index].data_untrusted_In_Transition = SAFE_TRUE;
		
		function_Result = CallSoundParamProc(audio_Tap_Ptr->original_Get_More_Data_UPP, sound_Source_Ptr);
		
		if (function_Result != FALSE) {
		
			real_Component_Data = &(*sound_Source_Ptr)->desc;		
			if ((real_Component_Data->numChannels > 0) && (real_Component_Data->numChannels < 33)) {
				audio_Tap_Ptr->info.tap_Buffer[buffer_Index].tick_Time_Last_Called = TickCount();
				audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Info = *real_Component_Data;
			
				bytes_To_Copy = (CARDINAL_32)real_Component_Data->numChannels;
				bytes_To_Copy = bytes_To_Copy * (CARDINAL_32)real_Component_Data->sampleSize;
				bytes_To_Copy = bytes_To_Copy * (CARDINAL_32)real_Component_Data->sampleCount;
				if (bytes_To_Copy > audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Buffer_Max_Size) {
					bytes_To_Copy = audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Buffer_Max_Size;
				}
				if (audio_Tap_Ptr->info.tap_Buffer_Total_Samples_Counted == 0) {
					audio_Tap_Ptr->info.tick_Time_First_Setup = audio_Tap_Ptr->info.tap_Buffer[buffer_Index].tick_Time_Last_Called;
				}
				audio_Tap_Ptr->info.tap_Buffer_Total_Samples_Counted += (CARDINAL_32)real_Component_Data->sampleCount;

				BlockMoveData(audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Info.buffer , *audio_Tap_Ptr->info.tap_Buffer[buffer_Index].component_Recent_Buffer, bytes_To_Copy);
				audio_Tap_Ptr->info.next_Buffer += 1;
				audio_Tap_Ptr->info.next_Buffer = (audio_Tap_Ptr->info.next_Buffer & 0x0001);
			}
		} /* if (function_Result != FALSE)  */
		
		audio_Tap_Ptr->info.tap_Buffer[buffer_Index].data_untrusted_In_Transition = SAFE_FALSE;
	}
		
	return(function_Result);
	
} /* pascal void Swap_More_Data_Callback_3 ... */
#endif /* NOT if (ORIGINAL_MAC_SYSTEM == 1) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
#if (ORIGINAL_MAC_SYSTEM == 1)
pascal void Set_Sound_Callback(SndChannelPtr active_Snd_Channel_Ptr, SndCommand *command)
/*  */
/* Globals not actually relied upon in this routine yet. */
/* But are avaliable if needed */
/*  */
{
	CARDINAL_32	previous_Global_Reference;
	CARDINAL_32	saved_Global_Reference;
	Async_AIFF_Ptr	playback;
	Sound_Manager_Private_Ptr	private_Info_Ptr;
/* END VAR */	

	playback = (Async_AIFF_Ptr)active_Snd_Channel_Ptr->userInfo;
	previous_Global_Reference = playback->saved_Global_Reference;

#if (k_USES_A5_FOR_GLOBALS == 1)

	asm {
		move.l 	a5, saved_Global_Reference(a6)		; Save original a5
		move.l	previous_Global_Reference(a6), A5
	}
	

	private_Info_Ptr = (Sound_Manager_Private_Ptr)active_Snd_Channel_Ptr->firstMod;
	playback->callback_Debug_Info_More_Data_Ptr = (CARDINAL_32)private_Info_Ptr->source_Sound_PB.moreRtn;
	playback->callback_Count += 1;
	
	asm {
		move.l	saved_Global_Reference(a6), A5
	}

#else
	asm {
		move.l 	a4, saved_Global_Reference(a6)		; Save original a4
		move.l	previous_Global_Reference(a6), A4
	}

	Mark_Playback_Flag_Completed(&playback->playing_AIFF_Now);
	
	asm {
		move.l	saved_Global_Reference(a6), A4
	}
#endif /* NOT #if (k_USES_A5_FOR_GLOBALS == 1) */

	
} /* pascal void Set_Sound_Callback ... */
#else
/*=**************************************************************************=*/


/*=**************************************************************************=*/
pascal void Set_Sound_Callback(SndChannelPtr active_Snd_Channel_Ptr, SndCommand *command)
{
	Async_AIFF_Ptr	playback;
	Sound_Manager_Private_Ptr	private_Info_Ptr;
/* END VAR */	

	playback = active_Snd_Channel_Ptr->userInfo;
	
	private_Info_Ptr = (Sound_Manager_Private_Ptr)active_Snd_Channel_Ptr->firstMod;
	playback->callback_Debug_Info_More_Data_Ptr = (CARDINAL_32)private_Info_Ptr->source_Sound_PB.moreRtn;
	playback->callback_Count += 1;
		
} /* pascal void Set_Sound_Callback ... */
#endif /* NOT if (ORIGINAL_MAC_SYSTEM == 1) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Sound_Tap_Channel.c */
