/* START OF FILE: Dreaming_Analog_Card.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */ 
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Dreaming_Analog_Card.c - Audio Routines to support Poor Man's Video Phone
 ** (C) Copyright 1993 James Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **	1993.05.03		(James Merkle)
 **						Initial coding started, some from other routines
 **
 **	1994.04.03		(James Merkle)
 **						Total system re-analysis. All new settings. Seems to enjoy having
 **						large voltage (close to +/- 5) and minimal gain (10X or less) and small packets (500) with little FIFO held	
 **						and small rate (10000/sec) and reasonable minor samples (100) within tapped packets.
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 **	
 ** Oscilloscope.c	-	This example performs a single channel data acquisition 
 **					operation and continuously graphs the data as it is acquired
 **					on the NB-MIO-16, NB-MIO-16X, Lab-NB, Lab-LC board using NI_DAQ_MAC interface
 **					routines.	Analog trigger conditions are specified in DAQ2TTap 
 **					to trigger when the analog value at channel 0 crosses 0 with a 
 **					negative slope.	A trigger position is specified to occur 
 **					half-way through the buffer.	Both the pre-triggered and 
 **					posttriggered data are displayed on the graph.
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */


#include <GestaltEqu.h>
#include <Sound.h>

#include "Error_Utilities.h"
#include "OS_Utilities.h"
#include "Shell_Dialogs.h"
#include "Shell_Globals.h"
#include "Memory_Utilities.h"
#include "Sound_Utilities.h"
#include "String_Utilities.h"


#include "Core_Specific.h"
#include "Core_Universal.h" /* Non_Specific_Support_Routines */
#include "Shell_Globals.h"

#include "NI_DAQ_MAC.h"

#include "Dreaming_Stuff.h"
#include "Dreaming_Analog_Card.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */

#define k_DEFAULT_BOARD_SLOT			5

/*  */
/* Set  k_EXPECT_SCXI_EQUIPMENT to "0" if raw card used*/
/*  */
#define k_EXPECT_SCXI_EQUIPMENT						0

#define k_DEFAULT_SCXI_CHASSIS							1
#define k_DEFAULT_SCXI_MODULE_SLOT				1


/*  */
/* Best if set to "1" */
/* but experimentally "100,50,20,10,5,2,1" or "5" is not too bad */
/* 1, 10, 100, 500 for NB-MIO 16 */
/* 1, 2, 4, 8 for NB-MIO 16H */ /* NORMALLY 1 with speaker amplifier */
/*  */
#define k_DESIRED_LAB_GAIN							(10L)
#define k_DESIRED_MIO_16_GAIN					(1L)

/*  */
/* use 10s timebase(first part, normally "2") */		
/*  */
#define	k_TIMEBASE_LAB_NB						(2L)
#define	k_TIMEBASE_MIO_16						(2L)

/*  */
/* use 10s timebase(second part, normally "30L")*/
/* When set to "9" at 10s, then yields 10,000 samples sec */
/* 30 = 3,333 sec */
/* 6 = 16,666 per sec */
/* sampling rate = 1 /( samp_interval * timebase) */
/* faster rates are possible with an NB-DMA board */
/*  */

/*  */
/* Because 600 is minimum packet and 30 packets per second */
/* needed for PERFECT dynamic streaming TTAPs we need 30 * 600 */
/* samples per second, but with extra lines we may need less, but regardless */
/* this results in need of at least a minimum of "6" at 100,000 base */
/* thus 16,666 */
/*  */
/* 20 norm */
#define	k_SAMPLE_INTERVAL_LAB_NB		(20L)

/*  */
/* (X) with base of 2 and rate of 10 then 10,000 samples per second */
/* with base of 1 and rate of 100 then 10,000 samples per second */
/* with base of 1 and rate of 50 then 22,222 samples per second */
/* with base of 1 and rate of 180 then 5555 samples per second */
/* (X) with base of 1 and rate of 120 then 8333 samples per second  (8333/ block of 500) = only 16.6 per second */
/* (X) with base of 1 and rate of 100 then 10000 samples per second  (10000/ block of 300) =  33 per second */
/* (X) with base of 2 and rate of 20 then 5,000 samples per second  (5000/ block of 110) =  45 per second */
/*  */
#define	k_SAMPLE_INTERVAL_MIO_16		(20L)

/*  */
/* number of samples in each block : 20 blocks in buffer */
/* BEWARE "k_MAXIMUM_MINI_SAMPLES" must be equal or less than */
/*  */
#define SAMPLES_IN_BLOCK_LAB_NB	 (500L)	
#define SAMPLES_IN_BLOCK_MIO_16	 (110L)	

/*  */
/* number of samples in circular buffer */
/* should be multiple of "SAMPLES_IN_BLOCK_LAB_NB" */
/* Channel * number of buffers * samples in block */
/* this number is NOT BYTES it is SAMPLES */
/* Best at "4" but set to "2" each for efficient non delays */
/* MIO 16 had trouble with it at 2 so it is set to 3 */
/*  */
#define TOTAL_BUFFER_SIZE_LAB_NB (1L  * 2L * SAMPLES_IN_BLOCK_LAB_NB)

#define TOTAL_BUFFER_SIZE_MIO_16 (1L  * 10L * SAMPLES_IN_BLOCK_MIO_16)

#define k_LOWEST_SLOT							0
#define k_HIGHEST_SLOT						15

/*  */
/* since data is acquired continuously, count is ignored */
/*  */
#define COUNT	0L

/*  */
/* Prevent activation triggering */
/*  */
#define k_TAP_COMMAND_IS_MISSING_ON_LAB	1

/*  */
/* ANALOG TRIGGER - trigger on a negative slope "1" */
/* I feel "0" is better overall */
/*  */
#define k_ACTIVATION_SLOPE_DIRECTION			0	     

/*  */
/*	when channel's value crosses 0 */
/*  */
#define VALUE								0

/*  */
/* enable continuous data acquisition */
/*  */
#define		k_CONTINUOUS_SAMPLING_WANTED		1
	
/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */

int Display_Card_Error(Str255 error_String);

void Reset(void);

int my_kbhit(void);

int main(void);

void chkerr(char *s, int16 err);

Safe_Boolean Activate_Continuous_Sampling(void);

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */

Safe_Boolean					g_Digital_Audio_Card_In_Use;

/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8	*unit_Name = "\pDreaming_Analog_Card";

CARDINAL_32				g_Failure_Count_For_AD_FIFO_Overflow_On_Card;

INTEGER_16					g_Board_In_Use;
INTEGER_16					g_Board_Type;
INTEGER_16					g_Gain;

Memory_Allocation		g_Analog_Buffer_Memory_In_Use;
Handle							g_Analog_Buffer;
CARDINAL_32				g_Buffer_Size;

INTEGER_16					g_Channel;
INTEGER_32					g_Total_Channels_Scanning;

Safe_Boolean					g_SCXI_In_Use;
INTEGER_32					g_SCXI_Chassis;
INTEGER_32					g_SCXI_Module;
INTEGER_16					g_Total_SCXI_Channels	;
INTEGER_16					g_Channels_Scanned_List[12];
INTEGER_16					g_Start_Channel_For_Scan_List[12];
INTEGER_16					g_Modules_Scanning_List[12];


INTEGER_16					g_Multiplexor_Mode;
INTEGER_16					g_16_Channel_Vector[16];
INTEGER_16					g_16_Gain_Vector[16];

/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Display_Card_Error(Str255 error_String) {

	Perform_OK_Dialog(error_String);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Start_Sampling_From_Card(CARDINAL_32	total_Channels_Scanning)
{
	INTEGER_16	result_Code, status;
	INTEGER_16	hack_Gain;
	INTEGER_16 	i;
	INTEGER_16 	*buffer;
	INTEGER_16	channel;
	INTEGER_16	slope;
	INTEGER_16	level;
	INTEGER_16	trigger_Skip_Position;
	Boolean			located;
	INTEGER_16	index;
	INTEGER_16	samples;
	INTEGER_16		board_Type;

	INTEGER_32 samp_interval, num_graphPts;
	SignedByte			handle_State;
	CARDINAL_32	memory_Requested;
	CARDINAL_32	final_Tick;
	Str255					message_String;
	Safe_Boolean		function_Result;
	Boolean				use_Normal_DAQ2Tap_Command;
	CARDINAL_8		*code_Name;
/* END VAR */


	code_Name = "\pStart_Sampling_From_Card";
	function_Result = SAFE_FALSE;
	
	if (g_Digital_Audio_Card_In_Use == SAFE_TRUE) {
		if (Stop_Sampling_From_Card() != SAFE_TRUE) {
			/*  */
			/* Ignore result_Code messages */
			/*  */
		}
	}
	
	g_Digital_Audio_Card_In_Use = SAFE_FALSE;

	buffer = NULL;
	
/*  */
/* NOTE: set global flag to clear board if program fails to end properly */
/* We MUST call "result_Code = DAQ2Clear(g_Board_In_Use);" */
/*  */
	
	g_Board_In_Use = k_DEFAULT_BOARD_SLOT;
	g_Board_Type = 0;
	
	g_Failure_Count_For_AD_FIFO_Overflow_On_Card = 0;

	located = FALSE;
	board_Type = 0;
	Board_ID(g_Board_In_Use, &board_Type);
	if ((board_Type == LAB_NB) || (board_Type == Lab_LC) || (board_Type == NB_MIO_16)){
		located = TRUE;
		g_Board_In_Use = k_DEFAULT_BOARD_SLOT;
		g_Board_Type = board_Type;
	}
	
	/*  */
	/* clear any current acquisition, and hunt for a card, indirectly the lazy way */
	/*  */
	if (located != TRUE) {
		located = FALSE;
		for (index = k_LOWEST_SLOT; ((index <= k_HIGHEST_SLOT) && (located != TRUE)); index++) {
			board_Type = 0;
			Board_ID(index, &board_Type);
			if ((board_Type == LAB_NB) || (board_Type == Lab_LC) || (board_Type == NB_DMA_8) || (board_Type == NB_MIO_16)){
				located = TRUE;
				g_Board_Type = board_Type;
				g_Board_In_Use = index;
			}
		}
	}
	
	if (located != TRUE) {
		Get_NS_String(k_DREAM_ERROR_STRINGS, k_ERROR_NO_CARD_AT_ALL_LOCATED, message_String);
		Perform_OK_Dialog(message_String);
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}
	
	located = FALSE;
	result_Code = Board_Reset(g_Board_In_Use);
	if (result_Code == 0) {
		located = TRUE;
	}
	
	if (located != TRUE) {
		Get_NS_String(k_DREAM_ERROR_STRINGS, k_ERROR_NO_CARD_LOCATED, message_String);
		Perform_OK_Dialog(message_String);
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}
	

#if (k_EXPECT_SCXI_EQUIPMENT == 1)
		g_SCXI_Chassis = k_DEFAULT_SCXI_CHASSIS;
		g_SCXI_Module = k_DEFAULT_SCXI_MODULE_SLOT;
		g_SCXI_In_Use = SAFE_TRUE;
		
		result_Code = SCXI_Reset(g_SCXI_Chassis, g_SCXI_Module);
	
		if (result_Code != 0){
			Get_NS_String(k_DREAM_ERROR_STRINGS, k_ERROR_SCXI_MODULE_AND_CHASSIS_NOT_DETECTED, message_String);
			Perform_OK_Dialog(message_String);
			g_SCXI_In_Use = SAFE_FALSE;
			g_SCXI_Chassis = 0;
			g_SCXI_Module = 0;
		} 
	
#else 
		g_SCXI_In_Use = SAFE_FALSE;
		g_SCXI_Chassis = 0;
		g_SCXI_Module = 0;
		
#endif /* if (k_EXPECT_SCXI_EQUIPMENT == 1) */

	result_Code = DAQ2Clear(g_Board_In_Use);
	if ((result_Code != 0) && (result_Code != -110)){
		Get_NS_String(k_DREAM_ERROR_STRINGS, k_ERROR_CARD_WOULD_NOT_CLEAR_DAQ, message_String);
		Perform_OK_Dialog(message_String);
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}	
		
/*  */
/* acquire triggered data on analog input channel 0 */
/*  */
	g_Channel = 0;
	g_Total_Channels_Scanning = total_Channels_Scanning;

	/*  */
	/* Initialized here but reset in following routine for effect */
	/*  */

	switch (g_Board_Type) {
		
		case NB_MIO_16:	
			g_Gain = k_DESIRED_MIO_16_GAIN;
		break;
		
		case LAB_NB:
		case Lab_LC:
			g_Gain = k_DESIRED_LAB_GAIN;
		break;
		
		default:
			g_Gain = 1;
			goto Error_Handler;
			
	} /* switch (g_Board_Type) */

		

	/*  */
	/* acquire memory for acquisition buffers */
	/*  */
	switch (g_Board_Type) {
		
		case NB_MIO_16:
			memory_Requested = (CARDINAL_32)sizeof(INTEGER_16) * (CARDINAL_32)(TOTAL_BUFFER_SIZE_LAB_NB);
			memory_Requested = memory_Requested * ((CARDINAL_32)g_Total_Channels_Scanning);
		break; /* NB_MIO_16 */
		
		
		case LAB_NB:
		case Lab_LC:
			memory_Requested = (CARDINAL_32)sizeof(INTEGER_16) * (CARDINAL_32)(TOTAL_BUFFER_SIZE_MIO_16);
			memory_Requested = memory_Requested * ((CARDINAL_32)g_Total_Channels_Scanning);
		break;
		
		default:
			chkerr("Unknown card", -64);
			goto Error_Handler;
			
	} /* switch (g_Board_Type) */

	
	g_Buffer_Size = memory_Requested;
	
	g_Analog_Buffer_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	g_Analog_Buffer = nil;
	
	if (Create_Handle(&memory_Requested
						, (CARDINAL_32)(10240L)
						, ALLOW_PURGING
						, DISALLOW_PARTIAL_AMOUNT
						, (Handle *)&g_Analog_Buffer
						, &g_Analog_Buffer_Memory_In_Use) != SAFE_TRUE) {
						
		Display_Low_Memory_Error_Message(unit_Name, code_Name, g_Buffer_Size);
		
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}
	/*  */
	/* Move it high and lock it down for good (until deleted) */
	/*  */
	if (Begin_Use_Of_Handle((Handle)g_Analog_Buffer
					, MOVE_HIGH_REQUIRED
					, &handle_State) != SAFE_TRUE) {
		goto Error_Handler;
	}

	buffer = (INTEGER_16 *)*g_Analog_Buffer;
	
			

	g_Digital_Audio_Card_In_Use = SAFE_TRUE;

	if (Reset_Sampling_From_Audio_Card() == SAFE_TRUE) {
		
		/*  */
		/* acquire the most recent block of data to graph */
		/*  */
		
		Delay(60L, (INTEGER_32 *)&final_Tick);
	
		/*  */
		/* This is it!!! This is an example of what to do for a grab */
		/* Try it out and shut down monitoring if an result_Code is detected */
		/* Channel is always "g_Channel" because that is the only channel we scan? */
		/*  */
		
		if (k_ACTIVATION_SLOPE_DIRECTION == 0 ) {
			if (g_SCXI_In_Use == SAFE_TRUE) {
				channel = g_Channel;
			}
			else {
				channel = 0;
			}
			slope = 0;
			level = 0;
			trigger_Skip_Position = 0;
		}
		else {
			if (g_SCXI_In_Use == SAFE_TRUE) {
				channel = g_Channel;
			}
			else {
				channel = 0;
			}
			slope = k_ACTIVATION_SLOPE_DIRECTION;
			level = VALUE;
			trigger_Skip_Position = (k_SKIP_SAMPLES);
			if (trigger_Skip_Position < 0) {
				trigger_Skip_Position = 0;
			}
		}

		use_Normal_DAQ2Tap_Command = TRUE;

	/*  */
	/* set memory for acquisition buffers */
	/*  */
	switch (g_Board_Type) {
		
		case NB_MIO_16:
			samples = SAMPLES_IN_BLOCK_MIO_16;
		break; /* NB_MIO_16 */
		
		case LAB_NB:
		case Lab_LC:
			samples = SAMPLES_IN_BLOCK_LAB_NB;
		break;
		
		default:
			chkerr("Unknown card", -64);
			goto Error_Handler;
			
	} /* switch (g_Board_Type) */


	switch (g_Board_Type) {
		
		case NB_MIO_16:
			; /* NULL */
		break; /* NB_MIO_16 */
		
		
		case LAB_NB:
		case Lab_LC:
#if (!defined(k_TAP_COMMAND_IS_MISSING_ON_LAB))
#error "UNDEFINED PARAMETER!"
#endif
			if (k_TAP_COMMAND_IS_MISSING_ON_LAB == 1) {
				use_Normal_DAQ2Tap_Command = FALSE;
			}
		break;
		
		default:
			chkerr("unknown card", -64);
			goto Error_Handler;
			
	} /* switch (g_Board_Type) */

	samples = samples * g_Total_Channels_Scanning;


#pragma mark INITIAL_GRAB_START

		if ((use_Normal_DAQ2Tap_Command != TRUE) || (slope != 0)) {
	 		result_Code = DAQ2TTap(g_Board_In_Use
						, channel
						, slope
						, level
						, (INTEGER_16 *)*g_Analog_Buffer
						, trigger_Skip_Position
						, samples);
		}
		else {
	 		result_Code = DAQ2Tap(g_Board_In_Use
						, channel
						, slope
						, level
						, (INTEGER_16 *)*g_Analog_Buffer
						, samples);
		}

		if (result_Code != noErr) {
			if (result_Code != -112) {
				chkerr("DAQ2TTap",result_Code);
			}
			else {


			}
		}
	
			if (result_Code == noErr) {
	/*  */
	/* scale digital values to voltages */
	/*  */
	
			} /*if (result_Code == noErr)*/
			else {
				if (Stop_Sampling_From_Card() != SAFE_TRUE) {
					/*  */
					/* Ignore result_Code messages */
					/*  */
				}
				else {
					goto Error_Handler;
				}
			}
			
	
	} /* if (result_Code == 0) */
	else {
		g_Digital_Audio_Card_In_Use = SAFE_FALSE;
	}
		
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
	g_Digital_Audio_Card_In_Use = SAFE_FALSE;


Normal_Function_Exit:

	return(function_Result);

}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Reset_Sampling_From_Audio_Card(void)
/*  */
/* This should only be called if g_Digital_Audio_Card_In_Use */
/* is valid or about to be set true. */
/* Right before calling this you may want to alter "g_Gain". */
/* Right after calling this, you should delay for half a second before */
/* relying on cards values again */
/*  */
{
	Safe_Boolean		function_Result;
	INTEGER_16		result_Code;
	INTEGER_16 		count_Mode;
	INTEGER_16		polarity;
	SIMPLE_LOOP	loop;
	CARDINAL_8		*code_Name;
	INTEGER_16		sample_Mux_Time;
	INTEGER_16		sample_Mux_Interval;
	INTEGER_16		scan_Mux_Time;
	INTEGER_16		scan_Mux_Interval;
/* END VAR */


	code_Name = "\pReset_Sampling_From_Audio_Card";

	if (g_Digital_Audio_Card_In_Use != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}

	function_Result = SAFE_FALSE;
	
	/*  */
	/* enable continuous data acquisition */
	/*  */	
	count_Mode = k_CONTINUOUS_SAMPLING_WANTED;	

	/*  */
	/* clear any current acquisition */
	/*  */	
	result_Code = DAQ2Clear(g_Board_In_Use);
	if ((result_Code != 0) && (result_Code != -110)){
		chkerr("DAQ2Clear", result_Code);
		goto Error_Handler;
	}

	/*  */
	/* start acquisition */
	 /* NOTE: since data is acquired continuously, pass in 0 for buffer and count parameters */
	/*  */
	
	/*  */
	/* semi pointless to include "g_Analog_Buffer" */
	/*  */
	
			
	switch (g_Board_Type) {
		
		case NB_MIO_16:
		
				/*  */
				/* Polarity can be "0" or "1" */
				/* "0" is preferred default (differential) */
				/*  */
				polarity = 0;
				
				result_Code = AI_Config(g_Board_In_Use
														, 0
														, 10
														, polarity);
				if (result_Code != 0) {
					chkerr("DAQ2Config", result_Code);
					goto Error_Handler;
				}
		

			/*  */
			/* Vital */
			/*  */
			result_Code = DAQ_Config(g_Board_In_Use
												, 0
												, 0
												, 0);
			if (result_Code != 0) {
				chkerr("DAQ2Config", result_Code);
				goto Error_Handler;
			}
			
			/*  */
			/* configure for continuous block mode data acquisition */
			/*  */
			result_Code = DAQ2Config(g_Board_In_Use
												,count_Mode
												,(TOTAL_BUFFER_SIZE_MIO_16) * g_Total_Channels_Scanning
												, SAMPLES_IN_BLOCK_MIO_16);
		
			if (result_Code != 0) {
				chkerr("DAQ2Config", result_Code);
				goto Error_Handler;
			}
			
			g_Multiplexor_Mode = 0;
			
			for (loop = 0; (loop < 16); loop++) {
				g_16_Channel_Vector[loop] = loop;
			}
			
			g_16_Channel_Vector[0] = 0;
			g_16_Channel_Vector[1] = 2;
			g_16_Channel_Vector[2] = 4;
			g_16_Channel_Vector[3] = 6;

			
			for (loop = 0; (loop < 16); loop++) {
				g_16_Gain_Vector[loop] = g_Gain;
			}
						
			if (g_SCXI_In_Use == SAFE_TRUE) {
			
				g_Total_SCXI_Channels = 0;
				
				g_Modules_Scanning_List[g_Total_SCXI_Channels] = g_SCXI_Module;
				g_Channels_Scanned_List[g_Total_SCXI_Channels] = g_Total_Channels_Scanning;
				g_Start_Channel_For_Scan_List[g_Total_SCXI_Channels] = 0;
				g_Total_SCXI_Channels +=1;
				
				result_Code = SCXI_SCAN_Setup(g_SCXI_Chassis
																, 1
																, &g_Modules_Scanning_List[0]
																, &g_Channels_Scanned_List[0]
																, &g_Start_Channel_For_Scan_List[0]
																, g_Board_In_Use
																, 0);
				if (result_Code != 0) {
					chkerr("SCXI_SCAN_Setup", result_Code);
					goto Error_Handler;
				}

				/*  */
				/* Set up the mux counter value to equal the total number of channels */
				/*	to be scanned in one pass through the module scan list, because */
				/*	we are only going to scan one channel on the DAQ board */
				/*  */
				result_Code = SCXI_MuxCtr_Setup (g_Board_In_Use, 1, 1, g_Total_SCXI_Channels);
				if (result_Code != 0) {
					chkerr("SCXI_MuxCtr_Setup", result_Code);
					goto Error_Handler;
				}
			}
			
		break; /* NB_MIO_16 */
		
		
		
		case LAB_NB:
		case Lab_LC:

			if (g_Digital_Audio_Card_In_Use != SAFE_TRUE) {
				/*  */
				/* Polarity can be "0" or "1" */
				/* "0" is preferred default */
				/*  */	
				polarity = 0;
				
				result_Code = AI_Config(g_Board_In_Use
														, 1
														, 10
														, polarity);
				if (result_Code != 0) {
					chkerr("DAQ2Config", result_Code);
					goto Error_Handler;
				}
			}
		
			/*  */
			/* Vital */
			/*  */
			result_Code = DAQ_Config(g_Board_In_Use
												, 0
												, 0
												, 0);
			if (result_Code != 0) {
				chkerr("DAQ2Config", result_Code);
				goto Error_Handler;
			}
			
			/*  */
			/* configure for continuous block mode data acquisition */
			/*  */
			result_Code = DAQ2Config(g_Board_In_Use
												,count_Mode
												,(TOTAL_BUFFER_SIZE_LAB_NB) * g_Total_Channels_Scanning
												, SAMPLES_IN_BLOCK_LAB_NB);
		
			if (result_Code != 0) {
				chkerr("DAQ2Config", result_Code);
				goto Error_Handler;
			}

		break;
		
		default:
			chkerr("DAQ2Config", -64);
			goto Error_Handler;
			
	} /* switch (g_Board_Type) */

	if (Activate_Continuous_Sampling() != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	/*  */
	/* start a single channel data acquisition */
	 /* NOTE: since data is acquired continuously, pass in 0 for buffer and count parameters */
	/*  */
/* =====
	result_Code = DAQ_Start(g_Board_In_Use, g_Channel, g_Gain, nil, COUNT, timebase, samp_interval);
	if (result_Code != 0) {
		chkerr("DAQ_Start", result_Code);
	}
====== */
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);

} /* Safe_Boolean Reset_Sampling_From_Audio_Card ... *
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Activate_Continuous_Sampling(void)
/*  */
/* This should only be called if g_Digital_Audio_Card_In_Use */
/* is valid or about to be set true. */
/* Right before calling this you may want to alter "g_Gain". */
/* Right after calling this, you should delay for half a second before */
/* relying on cards values again */
/*  */
{
	INTEGER_16		timebase;
	INTEGER_32		samp_interval;
	Safe_Boolean		function_Result;
	CARDINAL_8		*code_Name;
	OSErr					result_Code;
/* END VAR */


	code_Name = "\pActivate_Continuous_Sampling";
	function_Result = SAFE_FALSE;

	if (g_Digital_Audio_Card_In_Use != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}
			
	switch (g_Board_Type) {
		
		case NB_MIO_16:
		
			/*  */
			/* use 10s timebase(first part, normally "2"*/		
			/*  */
			timebase = k_TIMEBASE_MIO_16;	
			
			/*  */
			/* use 10s timebase(second part, normally "30L"*/		
			/*  */
			samp_interval = k_SAMPLE_INTERVAL_MIO_16;	
			
			/*  */
			/* sampling rate = 1 /( samp_interval * timebase) */
			/* 1200 sec/samp */
			/* faster rates are possible with an NB-DMA board */
			/*  */

			if (g_Total_Channels_Scanning > 1) {
			
				if (g_SCXI_In_Use != SAFE_TRUE) {
					result_Code = SCAN_Setup(g_Board_In_Use, g_Total_Channels_Scanning, &(g_16_Channel_Vector[0]), &(g_16_Gain_Vector[0]), g_Multiplexor_Mode);
				}
				else {
					result_Code = SCAN_Setup(g_Board_In_Use, 1, &(g_16_Channel_Vector[0]), &(g_16_Gain_Vector[0]), g_Multiplexor_Mode);
				}
			
				if (result_Code != 0) {
					chkerr("Scan_Setup", result_Code);
				}
/*====			
				sample_Mux_Time = timebase;
				sample_Mux_Interval = samp_interval;
				scan_Mux_Time = timebase;
				scan_Mux_Interval = 30;
				
				result_Code = SCAN_IntStart(g_Board_In_Use, nil, COUNT,  sample_Mux_Time, sample_Mux_Interval, scan_Mux_Time, scan_Mux_Interval);
==== */			
				result_Code = SCAN_Start(g_Board_In_Use, nil, COUNT,  timebase, samp_interval);
			}
			else {
			
				result_Code = DAQ_Start(g_Board_In_Use, g_Channel, g_Gain, nil, COUNT, timebase, samp_interval);
			
			
			} /* if (g_Total_Channels_Scanning < 2) */

			if (result_Code != 0) {
				chkerr("Scan_IntStart", result_Code);
			}
		break; /* NB_MIO_16 */
		
		
		case LAB_NB:
		case Lab_LC:

			/*  */
			/* use 10s timebase(first part, normally "2"*/		
			/*  */
			timebase = k_TIMEBASE_LAB_NB;	
			
			/*  */
			/* use 10s timebase(second part, normally "30L"*/		
			/*  */
			samp_interval = k_SAMPLE_INTERVAL_LAB_NB;
			
			/*  */
			/* sampling rate = 1 /( samp_interval * timebase) */
			/* 1200 sec/samp */
			/* faster rates are possible with an NB-DMA board */
			/*  */
			if (g_Total_Channels_Scanning > 1) {
				if (g_SCXI_In_Use != SAFE_TRUE) {
					result_Code = Lab_SCAN_Start(g_Board_In_Use, g_Total_Channels_Scanning, g_Gain, nil, COUNT, timebase, samp_interval);
				}
				else {
					result_Code = Lab_SCAN_Start(g_Board_In_Use, 1, g_Gain, nil, COUNT, timebase, samp_interval);
				}
				
				if (result_Code != 0) {
					chkerr("Lab_SCAN_Start", result_Code);
				}
			}
			else {
				result_Code = DAQ_Start(g_Board_In_Use, g_Channel, g_Gain, nil, COUNT, timebase, samp_interval);
				if (result_Code != 0) {
					chkerr("Lab_SCAN_Start", result_Code);
				}
			}
		break;
		
		default:
			chkerr("DAQ2Config", -64);
			goto Error_Handler;
			
	} /* switch (g_Board_Type) */

	
	/*  */
	/* start a single channel data acquisition */
	 /* NOTE: since data is acquired continuously, pass in 0 for buffer and count parameters */
	/*  */
/* =====
	result_Code = DAQ_Start(g_Board_In_Use, g_Channel, g_Gain, nil, COUNT, timebase, samp_interval);
	if (result_Code != 0) {
		chkerr("DAQ_Start", result_Code);
	}
====== */


	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);

}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Fetch_Tap_Of_Digital_Audio_Card(INTEGER_16 channel
					, CARDINAL_16 *minibuffer
					, CARDINAL_16 samples
					, Boolean			*no_New_Data_Yet)
{
	Safe_Boolean		function_Result;
	CARDINAL_8		*code_Name;
	SIMPLE_LOOP	loop;
	INTEGER_16		slope;
	INTEGER_16		level;
	INTEGER_16		trigger_Skip_Position;
	OSErr					result_Code;
	Boolean				use_Normal_DAQ2Tap_Command;
/* END VAR */


	code_Name = "\pFetch_Tap_Of_Digital_Audio_Card";
	function_Result = SAFE_FALSE;
	*no_New_Data_Yet = TRUE;
	
	if (g_Digital_Audio_Card_In_Use == SAFE_TRUE ) {

	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/* HACK TEMP PARANOIA CLEARING */
	/*  */
/* ===
	for (loop = 0; (loop < samples); loop++) {
		minibuffer[loop] = 0;
	}
==== */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */

	/*  */
	/* Trigger channel does not manner, no slope specified */
	/*  */
	
	/*  */
	/* Channel is always "g_Channel" because that is the only channel we scan? */
	/*  */

		if (k_ACTIVATION_SLOPE_DIRECTION == 0 ) {
		
			if (g_SCXI_In_Use == SAFE_TRUE) {
				channel = g_Channel;
			}
			
			slope = 0;
			level = 0;
			trigger_Skip_Position = 0;
		}
		else {
		
			if (g_SCXI_In_Use == SAFE_TRUE) {
				channel = g_Channel;
			}
			
			slope = k_ACTIVATION_SLOPE_DIRECTION;
			level = VALUE;
			trigger_Skip_Position = (k_SKIP_SAMPLES - 1);
			if (trigger_Skip_Position < 0) {
				trigger_Skip_Position = 0;
			}
		}

		use_Normal_DAQ2Tap_Command = TRUE;
		
	switch (g_Board_Type) {
		
		case NB_MIO_16:
			; /* NULL */
		break; /* NB_MIO_16 */
		
		
		case LAB_NB:
		case Lab_LC:
#if (!defined(k_TAP_COMMAND_IS_MISSING_ON_LAB))
#error "UNDEFINED PARAMETER!"
#endif
			if (k_TAP_COMMAND_IS_MISSING_ON_LAB == 1) {
				use_Normal_DAQ2Tap_Command = FALSE;
			}
		break;
		
		default:
			chkerr("DAQ2Config", -64);
			goto Error_Handler;
			
	} /* switch (g_Board_Type) */


#pragma mark SAMPLE_GRAB_START

		if ((use_Normal_DAQ2Tap_Command != TRUE) || (slope != 0)) {
	 		result_Code = DAQ2TTap(g_Board_In_Use
						, channel
						, slope
						, level
						, (INTEGER_16 *)minibuffer
						, trigger_Skip_Position
						, samples);
		}
		else {
	 		result_Code = DAQ2Tap(g_Board_In_Use
						, channel
						, slope
						, level
						, (INTEGER_16 *)minibuffer
						, samples);
		}

		if (result_Code == noErr) {
			*no_New_Data_Yet = FALSE;
		}
		else {
		
			switch (result_Code) {
			case -112:
				/*  */
				/* cntNotAvailErr */
				/*  */
				*no_New_Data_Yet = TRUE;
			break;
			
			case -74:
				/*  */
				/* no data acquisition in progress */
				/*  */
				*no_New_Data_Yet = TRUE;
			break;
			
			case -75:
			case -166:
				/*  */
				/* AD FIFO Overflowed because sample rate too fast */
				/*  */
				
				/*  */
				/* Increment count. Warnings will be posted at program end */
				/*  */
				g_Failure_Count_For_AD_FIFO_Overflow_On_Card += 1;
				
				*no_New_Data_Yet = TRUE;
				
				if (Activate_Continuous_Sampling() != SAFE_TRUE) {
					goto Error_Handler;
				}
				else {
					if ((use_Normal_DAQ2Tap_Command != TRUE) || (slope != 0)) {
				 		result_Code = DAQ2TTap(g_Board_In_Use
									, channel
									, slope
									, level
									, (INTEGER_16 *)minibuffer
									, trigger_Skip_Position
									, samples);
					}
					else {
				 		result_Code = DAQ2Tap(g_Board_In_Use
									, channel
									, slope
									, level
									, (INTEGER_16 *)minibuffer
									, samples);
					}
			
					if (result_Code == noErr) {
						*no_New_Data_Yet = FALSE;
					}
					else {
						goto Error_Handler;
					}
				}
				
			break;
			
			default:
				chkerr("DAQ2TTap",result_Code);
				*no_New_Data_Yet = TRUE;
			}
		}
		
	}
	else {
		goto Error_Handler;
	}	
	
	
/* ====
	err = SCAN_Demux (boardID, dataBuf, sampleCount, binaryArrays);
	chkerr("SCAN_Demux",err);
==== */

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);

}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Test_Digital_Audio_Card(void)
{
	Safe_Boolean		function_Result;
	CARDINAL_8		*code_Name;
	INTEGER_16		channel;
	INTEGER_16		slope;
	INTEGER_32		samples;
	INTEGER_16		level;
	INTEGER_16		trigger_Skip_Position;
	OSErr					result_Code;
/* END VAR */


	code_Name = "\pTest_Digital_Audio_Card";
	function_Result = SAFE_FALSE;
	
	if (Start_Sampling_From_Card(g_Voices.audio_Channels_Needed_On_Cards) != SAFE_TRUE) {
		if (Stop_Sampling_From_Card() != SAFE_TRUE) {
		
		}
	}
	
	
	/*  */
	/* Trigger channel does not manner, no slope specified */
	/*  */
		if (k_ACTIVATION_SLOPE_DIRECTION == 0 ) {
			if (g_SCXI_In_Use == SAFE_TRUE) {
				channel = g_Channel;
			}
			else {
				channel = 0;
			}

			slope = 0;
			level = 0;
			trigger_Skip_Position = 0;
		}
		else {
		
			if (g_SCXI_In_Use == SAFE_TRUE) {
				channel = g_Channel;
			}
			else {
				channel = 0;
			}
			
			slope = k_ACTIVATION_SLOPE_DIRECTION;
			level = VALUE;
			trigger_Skip_Position = (k_SKIP_SAMPLES);
			if (trigger_Skip_Position < 0) {
				trigger_Skip_Position = 0;
			}
		}

	switch (g_Board_Type) {
		
		case NB_MIO_16:
			samples = SAMPLES_IN_BLOCK_MIO_16;
		break; /* NB_MIO_16 */
		
		case LAB_NB:
		case Lab_LC:
			samples = SAMPLES_IN_BLOCK_LAB_NB;
		break;
		
		default:
			chkerr("Unknown card", -64);
			goto Error_Handler;
			
	} /* switch (g_Board_Type) */

		result_Code = DAQ2TTap(g_Board_In_Use
					, channel
					, slope
					, level
					, (INTEGER_16 *)(*g_Analog_Buffer)
					, trigger_Skip_Position
					, samples);
		
	if (result_Code != -112) {
		chkerr("DAQ2TTap",result_Code);
		if (result_Code == noErr) {
/*  */
/* scale digital values to voltages */
/*  */

		} /*if (result_Code == noErr)*/
		
	} /* if (result_Code != -112) */
	
	
	
	if (Stop_Sampling_From_Card() != SAFE_TRUE) {
	
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);

}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Stop_Sampling_From_Card(void)
{
	Safe_Boolean		function_Result;
	CARDINAL_8		*code_Name;
	OSErr					result_Code;
	Str255					message_1;
	Str255					message_2;
	Str255					message_3;
/* END VAR */


	code_Name = "\pStop_Sampling_From_Card";
	function_Result = SAFE_FALSE;
	
	if (g_Digital_Audio_Card_In_Use == SAFE_TRUE ) {
		result_Code = DAQ2Clear(g_Board_In_Use);
		chkerr("DAQ2Clear", result_Code);
		
		g_Digital_Audio_Card_In_Use = SAFE_FALSE;
		
		if (g_Analog_Buffer != nil)	{
			if (Destroy_Handle((Handle *)&(g_Analog_Buffer)
												, &(g_Analog_Buffer_Memory_In_Use)) != SAFE_TRUE) {
				Display_Serious_Error_Message(unit_Name, code_Name, "\pError with DisposeHandle", "\pSystem Corrupted?");
				goto Error_Handler;
			}	
		}
		
		if (g_Failure_Count_For_AD_FIFO_Overflow_On_Card > 30) {
			Get_NS_String(k_DREAM_ERROR_STRINGS, k_ERROR_LAB_CARD_TOO_MANY_RESTARTS_NEEDED_1, message_1);
			Get_NS_String(k_DREAM_ERROR_STRINGS, k_ERROR_LAB_CARD_TOO_MANY_RESTARTS_NEEDED_2, message_3);
			Make_String_Long_String(&message_2[0], &message_1[0], g_Failure_Count_For_AD_FIFO_Overflow_On_Card, NO_LIMIT, &message_3[0]);
			SysBeep(20);
			Perform_OK_Dialog(message_2);
		}
		
	} /* if (g_Digital_Audio_Card_In_Use == SAFE_TRUE ) */
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);

}	/* main */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void chkerr(char *s, INTEGER_16 err)
{

	if (err) {
		if (err == -1) {
				switch (err) {
					case -37:	Display_Card_Error("\p***bdNamErr: Bad file name for NI_DAQ_MAC");
								Display_Card_Error("\p***Check the driver name string in Device Manager calls");
								break;
					default:	Display_Card_Error("\p***Check Operating System Error codes.");
								break;
				}
		}
		else {	
			switch (err) {
				case -60:	Display_Card_Error("\pNot an NB Series board result_Code");
							break;
				case -61:	Display_Card_Error("\pBad board number result_Code");
							break;
				case -62:	Display_Card_Error("\pBad gain result_Code");
							break;
				case -63:	Display_Card_Error("\pBad channel number result_Code.");
							break;
				case -64:	Display_Card_Error("\pNo support result_Code.");
							Display_Card_Error("\pThis function is not supported by the board");
							break;
				case -65:	Display_Card_Error("\pBad port number result_Code");
							break;
				case -66:	Display_Card_Error("\pPort not configured for output result_Code");
							break;
				case -67:	Display_Card_Error("\pPort does not support latched mode");
							break;
				case -68:	Display_Card_Error("\pPort can not be assigned to group");
							break;
				case -69:	Display_Card_Error("\pIllegal input parameter value");
							break;
				case -70:	Display_Card_Error("\pTimeout result_Code");
							break;
				case -71:	Display_Card_Error("\pOut of Range result_Code");
							break;			
				case -72:	Display_Card_Error("\pData Acquisition in progress result_Code");
							break;
				case -73:	Display_Card_Error("\pCounter in use result_Code");
							break;
				case -74:	Display_Card_Error("\pNo data aquisition in progress result_Code");
							break;
				case -75:	Display_Card_Error("\pOverflow result_Code");
							break;
				case -76:	Display_Card_Error("\pOverrun result_Code");
							break;
				case -77:	Display_Card_Error("\pBad count result_Code.");
							Display_Card_Error("\pCount must be multiple of number of channels ");
							break;
				case -78:	Display_Card_Error("\pBad type result_Code");
							break;
				case -79:	Display_Card_Error("\pNo Count operation result_Code");
							break;
				case -80:	Display_Card_Error("\pCounter reserved result_Code");
							break;
				case -81:	Display_Card_Error("\pPort assigned to group result_Code");
							break;
				case -82:	Display_Card_Error("\pNo port assigned to group result_Code");
							break;
				case -83:	Display_Card_Error("\pGroup not configured for output result_Code");
							break;
				case -84:	 Display_Card_Error("\pGroup not configured for input result_Code");
							break;
				case -85:	 Display_Card_Error("\pBlock digital I/O already in progress result_Code");
							break;
				case -86:	 Display_Card_Error("\pNo block digital I/O in progress result_Code");
							break;
				case -87:	 Display_Card_Error("\pOne group may not use external handshaking while the other is performing timed I/O");
							break;
				case -88:	 Display_Card_Error("\pDMA transfer requires port 0 in 8-bit, 0 & 1 in 16-bit, or all ports in 32-bit");
							break;
				case -90:	Display_Card_Error("\pBad signal direction result_Code");
							break;
				case -91:	Display_Card_Error("\pThe specified RTSI trigger line is in use");
							break;
				case -92:	Display_Card_Error("\pNo RTSI trigger lines are available");
							break;
				case -93:	Display_Card_Error("\pThe specified signal is in use");
							break;
				case -94:	Display_Card_Error("\pAn NB-DMA-8-G card is needed for waveform generation");
							break;
				case -95:	Display_Card_Error("\pA DMA channel is not available for use");
							break;
				case -96:	Display_Card_Error("\pA setup call is required prior to this operation");
							break;
				case -97:	Display_Card_Error("\pThe analog output channel is in use");
							break;
				case -98:	Display_Card_Error("\pA waveform load call is required prior to this operation");
							break;
				case -99:	Display_Card_Error("\pThe specified channel has been assigned to a group");
							break;
				case -100:	Display_Card_Error("\pNo waveform operation has been executed");
							break;
				case -101:	Display_Card_Error("\pOnly one WF_Grp_Setup call per board is allowed");
							break;
				case -102:	Display_Card_Error("\pGroup waveform generation is in progress");
							break;
				case -103:	Display_Card_Error("\pSpecified analog trigger mode is not supported.");
							break;
				case -104:	Display_Card_Error("\pSpecified number of analog triggers is not supported.");
							break;
				case -105:	Display_Card_Error("\pSpecified hystresis window is out of range from the specified analog trigger level.");
							break;
				case -106:	Display_Card_Error("\pShared trigger requires that data acq. and waveform gen. be externally triggered.");
							break;
				case -107:	Display_Card_Error("\pContinuous data acquisition or waveform generation not possible without DMA.");
							break;
				case -108:	Display_Card_Error("\pNot enough physical memory");
							break;
				case -109:	Display_Card_Error("\pSpecified block size in DAQ2Config invalid for rate selected in DAQ_Start (DMA only)");
							break;
				case -110:	Display_Card_Error("\pNI_DAQ_MAC has not been configured for double-buffered mode");
							break;
				case -111:	Display_Card_Error("\pAcquired data was overwritten in the circular double-buffered acquisition buffer");
							break;
				case -112:	Display_Card_Error("\pThe number of samples requested is not yet available");
							break;
				case -113:	Display_Card_Error("\pThe double buffer data acquisition completed before the number of samples requested was available");
							break;
				case -114:	Display_Card_Error("\pSpecified block size is invalid or not an integer multiple of number of channels");
							break;
				case -115:	Display_Card_Error("\pDAQ2MemConfig was executed after DAQ2Config");
							break;
				case -116:	Display_Card_Error("\pRTSI_DisConn is not needed; signal is connected to trigger line.");
							break;
				case -117:	Display_Card_Error("\pCannot drive a trigger signal to the I/O connector in \
									the current trigger configuration of the NB-A2100 or NB-A2150.");
							break;
				case -118:	Display_Card_Error("\pCannot enable digital trigger from both the RTSI bus and the I/O connector.");
							break;
				case -119:	Display_Card_Error("\pDelayed trigger mode is not supported in current trigger configuration of NB-A2100.");
							break;
				case -120:	Display_Card_Error("\pNo trigger value was found");
							break;
				case -121:	Display_Card_Error("\pAnalog trigger did not occur before the specified time expired");
							break;
				case -122:	Display_Card_Error("\pPre-triggering not allowed in combination with analog input trigger");
							break;
				case -123:	Display_Card_Error("\pSpecified a delay after the trigger when collecting data so the delay value is ignored");
							break;
				case -124:	Display_Card_Error("\pNumber of samples after pretrigger is not an integer multiple of the number of channels");
							break;
				case -125:	Display_Card_Error("\pMust call MAI_Arm befor MAI_Read for external sample clock");
							break;
				case -126:	Display_Card_Error("\pIllegal RTSI connection");
							break;
				case -127:	Display_Card_Error("\pExpected data is not avalable in FIFO");
							break;
				case -128:	Display_Card_Error("\pNo trigger was enabled");
							break;
				case -129:	Display_Card_Error("\pOperation not allowed while the board is armed -- call MAI_Arm to disarm first");
							break;
				case -130:	Display_Card_Error("\pNot enough pre-samples were collected");
							break;
				case -131:	Display_Card_Error("\pScan interval must be the length of one sample interval times the number of channels being scanned + 2sec");
							break;
				case -132:	Display_Card_Error("\pTrigger position is out of range");
							break;
				case -133:	Display_Card_Error("\pTotal number of samples is invalid or not an even number");
							break;
				case -134:	Display_Card_Error("\pInvalid number of pretrigger scans specified");
							break;
				case -135:	Display_Card_Error("\pInvalid number of posttrigger scans specified");
							break;
				case -136:	Display_Card_Error("\pFrame size exceeded maximum number of DMA transfers");
							break;
				case -137:	Display_Card_Error("\pInvalid scan rate specified");
							break;
				case -138:	Display_Card_Error("\pScan rate is ignored when using an external sample clock");
							break;
				case -139:	Display_Card_Error("\pAttempted to get an invalid frame or scan from the acquisition buffer");
							break;
				case -140:	Display_Card_Error("\pDMA config result_Code");
							break;
				case -141:	Display_Card_Error("\pGain/offset DAC value out of range during calibration");
							break;
				case -142:	Display_Card_Error("\pGain/offset calibration was unable to converge");
							break;
				case -143:	Display_Card_Error("\pUnable to read value from NB-A2000	EEPROM");
							break;
				case -144:	Display_Card_Error("\pUnable to write value to NB-A2000 EEPROM");
							break;
				case -145:	Display_Card_Error("\pHardware result_Code when reading value from NB-A2000 EEPROM");
							break;
				case -146:	Display_Card_Error("\pInvalid address for NB-A2000 EEPROM read / write");
							break;
				case -147:	Display_Card_Error("\pExternal reference value out of range during initial calibration");
							break;
				case -148:	Display_Card_Error("\pInternal reference value out of range during initial calibration");
							break;
				case -149:	Display_Card_Error("\pUnable to collect calibration data from board");
							break;
				case -150:	Display_Card_Error("\pHardware initialization result_Code");
							break;
				case -151:	Display_Card_Error("\pClock source not consistent with operation attempted");
							break;
				case -152:	Display_Card_Error("\pSample clock signal can not be received from and driven on the external sample clock line simultaneously");
							break;
				case -153:	Display_Card_Error("\pBuffer size or base address is not 16 32-bit word aligned for use with block mode DMA");
							break;
				case -154:	Display_Card_Error("\pBoard already setup for master-slave.");
							break;
				case -155:	Display_Card_Error("\pOperation failed because resource is reserved by NI-DAQ.");
							break;
				case -156:	Display_Card_Error("\pOperation failed because resource is reserved by the user.");
							break;
				case -157:	Display_Card_Error("\pOperation failed because A2150 or A2100 slot in Master Slave Clock Configuration.");
							break;			
				case -159:	Display_Card_Error("\pNumber of points selected for waveform buffer or block is invalid.");
							break;
				case -160:	Display_Card_Error("\pCannot do D/A output because the D/A FIFO is disabled.");
							break;
				case -161:	Display_Card_Error("\pSpecified buffer number is invalid.");
							break;
				case -162:	Display_Card_Error("\pSpecified group number is invalid.");
							break;
				case -163:	Display_Card_Error("\pCannot execute this function when buffered waveform generation is in progress.");
							break;
				case -164:	Display_Card_Error("\pCannot execute this function when buffered waveform generation has completed.");
							break;
				case -165:	Display_Card_Error("\pD/A FIFO Underflow result_Code.	If NB-DMA-8 board is used, RTSI cable must be connected.");
							break;
				case -166:	Display_Card_Error("\pDMA chaining operation malfunctioned and halted waveform generation.");
							break;
				case -167:	Display_Card_Error("\pUnderwrite result_Code. Waveform data was generated more than desired number of times.");
							break;
				case -168:	Display_Card_Error("\pCannot execute when last block has already been loaded.");
							break;
				case -169:	Display_Card_Error("\pBuffered waveform generation has stopped to prevent regeneration of data.");
							break;
				case -175:	Display_Card_Error("\pLine not configured for output in case of write.");
							break;
				case -180:	Display_Card_Error("\pCould not make the virtual memory buffer contiguous for use in a DMA data acquisition or waveform generation.");
							break;
				case -181:	Display_Card_Error("\pAnalog triggering and pretriggering flags must be disabled before starting block mode data acquisition.");
							break;
				case -182:	Display_Card_Error("\pSince external signal is used for later update mode, this call had no effect.");
							break;
				case -183:	Display_Card_Error("\pThe SCXI chassis ID does not correspond to a configured chassis.");
							break;
				case -184:	Display_Card_Error("\pThe SCXI module slot specified is invalid or corresponds to an empty slot.");
							break;
				case -185:	Display_Card_Error("\pThe NI-DAQ Preferences file could not be found or has been corrupted.");
							break;
				case -186:	Display_Card_Error("\pThe SCXI configuration parameters specified indicate\n ");
							Display_Card_Error("\pan invalid SCXI configuration, or the current function\n ");
							Display_Card_Error("\pcannot be executed because of the current SCXI configuration.");
							break;
				case -187:	Display_Card_Error("\pSCXI communication result_Code; either the chassis communication is\n ");
							Display_Card_Error("\pdisabled, or the driver could not successfully communicate with the chassis ");
							break;
				case -188:	Display_Card_Error("\pEither the operating mode specified in an SCXI config call is invalid,\n ");
							Display_Card_Error("\pa module is in the wrong opMode to execute the current function.");
							break;
				case -189:	Display_Card_Error("\pOne of the SCXI modules specified for a function is not supported for\n");
							Display_Card_Error("\pthe operation; the rest of the function was executed for the\n");
							Display_Card_Error("\pspecified modules that are supported\n");
							break;
				case -190:	Display_Card_Error("\pThe data acquisition board specified for an SCXI \n");
							Display_Card_Error("\poperation is the wrong board type for the operation");
							break;
				case -191:	Display_Card_Error("\pAn attempt was made to drive the Track/Hold trigger line on the\n");
							Display_Card_Error("\pSCXIbus with more than one module, or a SCXI_Track_Hold_Control call\n");
							Display_Card_Error("\pwas made and the Track/Hold setup is not correct \n");
							break;
				case -300:	Display_Card_Error("\pWARNING: Wrote to a port where some lines are configured for input.");
							break;
				case -500:	Display_Card_Error("\punexpected result_Code");
							break;
				default:	Display_Card_Error("\pConsult NI_DAQ_MAC result_Code");
							break;
			}
		}
	}
	
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Dreaming_Analog_Card.c */
