/* [AVL_Tree_Test.c] START OF FILE */

/* [---BEGIN---] FILE HEADER COMMENTS */

/**
 ** SCREEN VIEW = Syntax-12, Tabs=6 chars
 ** PRINTER VIEW = Syntax-12, -12, Tabs=0.30 inch
 ** ========================================================================
 ** AVL_Tree_Test.c v0.8 
 ** Written by James Merkle from two magazine articles
 ** [AVL_Tree_Test.c] - main and supporting routines
 ** 1990 by James Merkle
 ** ========================================================================
 **
 **
 ** =============
 ** Revision History
 ** =============
 **  90.11.10			(James Merkle)
 **						Initial coding started 
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 **	Modified heavily from two sources (Micro C, MacTutor)
 **	Original purpose : To test AVL speed and efficiency
 **
 **	Current use
 **											: manage source code structures and entities in RapidoCode
 **
 **	Other uses	(If modified)
 **											: General purpose LONGINT-keyed tree manager
 **											: manage open file handles in Buffered IO unit
 **											: manage control objects in Patch-O-Matic
 **											: manage buffered sectors in buffered SCSI disk driver
 **
 **	AVL stands for "g.m. Adelson-Velskii and e.m. Landis" , two Russian mathematicians from 1962. 
 **	What they proposed was that although a properly balanced binary tree is the fastest tree in the world, 
 **	the algorithm to keep it balanced when fed data in linear and random order is so slow, that a compromise 
 **	made leaving some of the nodes unbalanced, resulted in a quite efficient little routine. Each node can have at most
 **	a difference of 1 node length between the right and left subtrees.  These lengths are kept within the nodes 
 **	themselves in the form of a balance factor number requiring at most only two bits for stable storage of 
 **	the differences of -1,0, and 1.  When the balance factor would become +2 or -2,  a rotation of nodes occurs
 **	to restabilize the balance factor.
 **
 ** Sadly, The more complicated, yet critical "Deletion" routine still needs to be written.  This AVL tree can only add.
 **
 **/

/* [----END----] FILE HEADER COMMENTS */
/*=********************************************************************=*/

/*=********************************************************************=*/
/* [---BEGIN---] FILE INCLUDE AREA */

/* #include <stdio.h> 
#include <stdlib.h>
#include <string.h> */

#include "AVL_Tree_Manager.h"
#include "System_Dependent_Hooks.h"
#include "Non_Specific_Support_Routines.h"

/* [----END----] FILE INCLUDE AREA */
	
	
/*=********************************************************************=*/
/* [---BEGIN---] PROTOTYPES-------------------------- */

/*------------ LOCAL PROTOTYPES -------------------*/
	
/* extern AVL_node *Create_AVL_Node(char *thekey, char *thedata); */
/* extern CARDINAL_16 insert_AVL_Node(node *parent, char *thekey, char *thedata); */
/* extern node *Find_Tree_Node(node *AVL_Tree, char *thekey); */
/* extern void Dispose_Of_The_Tree_Recursive(node *AVL_Tree); */

	
/* [----END----] PROTOTYPES-------------------------- */
/*=********************************************************************=*/


static char		*unit_Name = "\pAVL_Tree_Manager";


/*=********************************************************************=*/


/*=********************************************************************=*/
/* create and initialize a new AVL_node */
/* Caution : str55 is used as first parameter, but second parameter is like variable_String_65537 */
AVL_node *Create_AVL_Node(char *thekey, char *thedata)
	{
	
	AVL_node		*thenode;
	CARDINAL_16	thelength;
	char				*code_Name;
/* END VAR */

	code_Name="\pCreate_AVL_Node";

	
	thenode = (AVL_node *)NewPtr(sizeof(AVL_node));
	if (thenode == NIL) {
		Display_Serious_Error_Message(unit_Name, code_Name,
										 "\pcall to NewPtr", "\pOut of memory?");
		  thenode = NIL;
		 goto END_OF_FUNCTION;
	}
	
	thelength = 1 + (CARDINAL_8)thekey[0];
	thenode->item_Key = (char *)NewPtr(thelength);
	if (thenode->item_Key == NIL) {
		Display_Serious_Error_Message(unit_Name, code_Name,
											 "\pcall to NewPtr", "\pOut of memory?");
		DisposPtr((Ptr)thenode);
		  thenode = NIL;
		 goto END_OF_FUNCTION;
	}
	
	BlockMove((Ptr)thekey, (Ptr)thenode->item_Key, thelength);
	
	thenode->balance = 0;
	thenode->left_Node_Ptr = NIL;
	thenode->right_Node_Ptr = NIL;
	
	/*=--------------------------------------------------------------------=*/
	/* Note : this computation used in case non-aligned memory on 68000 CPU */
	/*=--------------------------------------------------------------------=*/
	thelength = 2 + (CARDINAL_8)thedata[1];
	thelength = ((CARDINAL_8)thedata[0] << 8) | thelength;
	
	thenode->item_Data = (char *)NewPtr(thelength);
	if (thenode->item_Data == NIL) {
		Display_Serious_Error_Message(unit_Name, code_Name,
										 "\pcall to NewPtr", "\pOut of memory?");
		DisposPtr((Ptr)thenode->item_Key);
		DisposPtr((Ptr)thenode);
		 thenode = NIL;
		 goto END_OF_FUNCTION;
	}
	
	BlockMove((Ptr)thedata, (Ptr)thenode->item_Data, thelength);
	
END_OF_FUNCTION:
	
	return(thenode);
	
	} /* Create_AVL_Node */
/*=********************************************************************=*/


/*=********************************************************************=*/
/*=--------------------------------------------------------------------=*/
/*
**		Insert a new key into the table and correct balance
**		factors recursively.  The routine returns a queue of
**		path directions.
*/
/*=--------------------------------------------------------------------=*/

CARDINAL_16 Insert_AVL_Node(AVL_node *parent, char *thekey, char *thedata)
	{
	
	INTEGER_16	compare;
	AVL_node		*high;
	CARDINAL_16	result;
	AVL_node		*low;
	AVL_node		*child;
	char			*code_Name;
/* END VAR */

	code_Name="\pInsert_AVL_Node";

/*=--------------------------------------------------------------------=*/
/* if thekey matches this node, then return an error  (duplicate keys) */
/*=--------------------------------------------------------------------=*/
	
	compare = Compare_Pascal_Strings_Fully(thekey, parent->item_Key);
	if (compare == 0)
		return(ALREADY_PRESENT);

/*=--------------------------------------------------------------------=*/	
/*	if there's a slot for the new node, then create it and return
		1 if it is to the left of parent and 2 if to the right
	else determine high, the next step in the search path */
/*=--------------------------------------------------------------------=*/
	
	if (compare < 0) {
		high = parent->left_Node_Ptr;
		if (high == NIL) {
			parent->left_Node_Ptr = Create_AVL_Node(thekey, thedata);
			return(1);
		}
	}
	else
	{
		high = parent->right_Node_Ptr;
		if (high == NIL) {
			parent->right_Node_Ptr = Create_AVL_Node(thekey, thedata);
			return(2);
		}
	}
		
/*=--------------------------------------------------------------------=*/		
/*	now continue the search by calling insert recursively
//	if the result is 0 (no balancing needed at this level) or
//	the result is ALREADY_PRESENT (duplicate key), return */
/*=--------------------------------------------------------------------=*/	
	
	result = Insert_AVL_Node(high, thekey, thedata);
	if ((result == 0) || (result == ALREADY_PRESENT))
		return(result);

/*=--------------------------------------------------------------------=*/		
/*		the low 4 bits of result indicate the direction of the
//		search path leaving high; increment high's balance
//		if the path goes left, and decrement it if the path
//		goes right  */
/*=--------------------------------------------------------------------=*/	

	
	if (result % 16 == 1)
		high->balance++;
	else
		high->balance--;

/*=--------------------------------------------------------------------=*/		
/*	if the balance is now zero, no further correction of balances
//		is needed, so return
//	if it's 1, push the direction away from parent onto the
//		result stack and return it
//	if it's 2, continue to balancing transformation             */
/*=--------------------------------------------------------------------=*/	

	
	switch (high->balance)
		{
		case 0:
			return(0);
		case 1:
		case -1:
			return((result << 4) + ((compare < 0) ? 1 : 2));
		case 2:
		case -2:
			break;
		}
		
/*=--------------------------------------------------------------------=*/		
/*/	use the direction away from high to find low, then
//		switch on that direction and the next one
//	the easiest way to follow these transformations is to
//		refer to the pictures          */
/*=--------------------------------------------------------------------=*/	

	
	low = (result % 16 == 1) ? high->left_Node_Ptr : high->right_Node_Ptr;
	switch (result % 256)
		{
		
		case 0x11:	/*	left-left case    */
			
			high->left_Node_Ptr = low->right_Node_Ptr;
			low->right_Node_Ptr = high;
			if (compare < 0)
				parent->left_Node_Ptr = low;
			else
				parent->right_Node_Ptr = low;
			high->balance = 0;
			low->balance = 0;
			return(0);
		
		case 0x12:	/*  right-left case  */
			
			child = low->left_Node_Ptr;
			high->right_Node_Ptr = child->left_Node_Ptr;
			low->left_Node_Ptr = child->right_Node_Ptr;
			if (compare < 0)
				parent->left_Node_Ptr = child;
			else
				parent->right_Node_Ptr = child;
			child->left_Node_Ptr = high;
			child->right_Node_Ptr = low;
			child->balance = 0;
			low->balance = 0;
			high->balance = 0;
			result &= 0xF00;
			if (result == 0x100)
				low->balance = -1;
			else if (result == 0x200)
				high->balance = 1;
			return(0);
		
		case 0x21:	/*	left-right case  */
			
			child = low->right_Node_Ptr;
			low->right_Node_Ptr = child->left_Node_Ptr;
			high->left_Node_Ptr = child->right_Node_Ptr;
			if (compare < 0)
				parent->left_Node_Ptr = child;
			else
				parent->right_Node_Ptr = child;
			child->left_Node_Ptr = low;
			child->right_Node_Ptr = high;
			child->balance = 0;
			low->balance = 0;
			high->balance = 0;
			result &= 0xF00;
			if (result == 0x100)
				high->balance = -1;
			else if (result == 0x200)
				low->balance = 1;
			return(0);
		
		case 0x22:	/* right-right case */
			
			high->right_Node_Ptr = low->left_Node_Ptr;
			low->left_Node_Ptr = high;
			if (compare < 0)
				parent->left_Node_Ptr = low;
			else
				parent->right_Node_Ptr = low;
			high->balance = 0;
			low->balance = 0;
			return(0);
		
		}
	
	} /* Insert_AVL_Node */
/*=********************************************************************=*/


/*=********************************************************************=*/
AVL_node *Find_Tree_Node(AVL_node *AVL_Tree, char *thekey)
	{
	
	AVL_node		*thenode;
	INTEGER_16	compare;

/* END VAR */

	
	thenode = AVL_Tree;
	
	while (compare = (Compare_Pascal_Strings_Fully(thekey, thenode->item_Key))) {
		if (compare < 0)
			thenode = thenode->left_Node_Ptr;
		else
			thenode = thenode->right_Node_Ptr;
			
		if (thenode == NIL)
			return(NIL);
	}
	
	return(thenode);
	
	} /* Find_Tree_Node */
/*=********************************************************************=*/


/*=********************************************************************=*/
void Dispose_Of_The_Tree_Recursive(AVL_node *AVL_Tree)
	{
	
		if (AVL_Tree == NIL)
			return;
		
		Dispose_Of_The_Tree_Recursive(AVL_Tree->left_Node_Ptr);
		Dispose_Of_The_Tree_Recursive(AVL_Tree->right_Node_Ptr);
		
		DisposPtr((Ptr)AVL_Tree->item_Key);
		DisposPtr((Ptr)AVL_Tree->item_Data);
		DisposPtr((Ptr)AVL_Tree);
		
	} /*  Dispose_Of_The_Tree_Recursive */
/*=********************************************************************=*/


/*=********************************************************************=*/
/* END OF FILE: AVL_Tree_Manager.c */
