/* START OF FILE: Object_Inventory.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Object_Inventory.c - Manage inventory of available files and loaded files and components
 ** (C) Copyright 1994 James Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1994.02.01		(James Merkle)
 **						Initial coding started
 **
 **  1995.06.12		(James Merkle)
 **						Fixed foolish bug where in many loops in "Add_Potential_File_Item_To_Database" the 
 **						flag for "located" was not cleared before starting resulting in a truncated list. Never caused too much trouble
 **						because of luck.
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 **
 **	Depending on setting of "k_SEARCH_ENTIRE_VOLUME_FAST" this library
 **  can scan a hard disk for file items recuresively within a specific folder (App folder)
 ** or an entire volume (App volume). Entire volume uses re-entrant calls to PBCatSearch.
 **  The slower stack method uses very efficient stack based re-entrance as background task.
 **
 **
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include "About.h"

#include "Error_Utilities.h"
#include "Memory_Utilities.h"
#include "Program_Globals.h"
#include "String_Utilities.h"
#include "Volume_Utilities.h"
#include "Dreaming_File_Loading.h"
#include "Dreaming_Popup_Dialogs.h"
#include "Dreaming_Scene_Files.h"
#include "Dreaming_Backdrop_Files.h"
#include "Spin_O_Rama_Files.h"
#include "Slide_Show_Routines.h"
#include "Shell_Dialogs.h"

#include "Core_Globals.h"
#include "Core_Specific.h"
#include "Dreaming_Stuff.h"
#include "Core_Globals.h"
#include "Command_Redirector.h"

#include "Object_Inventory.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */

/*  */
/* Determines if PBCatSearch used for searching entire volume instead */
/* of precision based recursion from start point */
/*  */
#define k_SEARCH_ENTIRE_VOLUME_FAST		0

#define k_SEARCH_BUFFER_SIZE		(16384L)
#define k_MAXIMUM_FILE_MATCHES		10

#define k_MILLISECOND_CONVERSION		(17L)

#define k_AIFF_TYPE		'AIFF'



/*  */
/* Finder blows up at 100 so this should be enough */
/*  */
#define k_MAX_STACK_DEPTH_ALLOWED				128

#define k_MAX_FILE_MATCH_TYPES						10


/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */

	typedef struct {	
		FSSpec					search_Results[k_MAXIMUM_FILE_MATCHES];
	} Search_Results_Rec, *Search_Results_Ptr, **Search_Results_Hdl;
	
	/*  */
	/* Used in recursive search (not PBCat related) */
	/*  */
	typedef struct {
		Safe_Boolean			data_Valid;
		INTEGER_16			volume_Reference;	
		CARDINAL_32		file_Type_To_Match[k_MAX_FILE_MATCH_TYPES];
		CARDINAL_32		file_Type_To_Match_Total;
		CARDINAL_32		stop_Search_Time;
		CARDINAL_32		initialization_Directory;
		CARDINAL_32		initialization_Index;
		CARDINAL_16		return_Value_Determined;
		CARDINAL_16		stack_Total;
		CInfoPBRec			param_Block;
		
		INTEGER_16			stack_Of_Indices[k_MAX_STACK_DEPTH_ALLOWED];
	} Recursive_Search_Record, *Recursive_Search_Ptr, **Recursive_Search_Hdl;

/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */
															
														
void Test_Reentrant_Search(INTEGER_16 volume_Ref_Num_To_Search
								, INTEGER_32 directory_To_Search
								, Recursive_Search_Record	*pre_Cache);
								
void Continue_Reentrant_Search (Recursive_Search_Record	*pre_Cache);

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */

	Safe_Boolean		g_Scanning_Active;
	Safe_Boolean		g_Scanning_Needs_Run;
	Safe_Boolean		g_Search_Entire_Volume;
	
	
	Database_Control	g_Remote_Scenes;
	Database_Control	g_Puppets;
	Database_Control	g_Backdrops;
	Database_Control	g_Spin_Objects;
	Database_Control	g_Audio_Files;

/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8	*unit_Name;

	Handle					g_Search_Buffer_Hdl;
	Memory_Allocation		g_Search_Buffer_Hdl_Memory_Initialized;
	
	Search_Results_Hdl		g_Search_Results;
	Memory_Allocation		g_Search_Results_Memory_Initialized;

	Recursive_Search_Hdl	g_Stack_Search_Hdl;
	Memory_Allocation		g_Stack_Search_Memory_Initialized;

	HParamBlockRec			g_Hfs_Parameter_Block;
	CInfoPBRec				g_Search_Spec_1;
	CInfoPBRec				g_Search_Spec_2;
	CSParam					g_Search;
	Str63						g_File_Name;
	
	Safe_Boolean				g_Working_Directory_In_Use;
	INTEGER_16				g_Working_Directory;
	WDPBRec					g_Working_Directory_PB;

/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Initialize_Object_Inventory(void)
{
	CARDINAL_8			*static_Unit_Name = "\pObject_Inventory";
	CARDINAL_32		loop;
	SignedByte			original_State;
	CARDINAL_32		memory_Requested;
	Safe_Boolean			function_Result;
/* END VAR */

/*  */
/* Global persistant structures are created here */
/*  */

	unit_Name = static_Unit_Name;
	function_Result = SAFE_FALSE;


	g_Scanning_Active = SAFE_FALSE;
	g_Scanning_Needs_Run = SAFE_TRUE;


/*  */
/* Global persistant structures are created here */
/*  */

	g_Remote_Scenes.info.scenes.handle = nil;
	g_Remote_Scenes.memory_In_Use = MEMORY_NOT_INITIALIZED;
	g_Remote_Scenes.info.scenes.pointer = nil;
	
	memory_Requested = sizeof(Scenes_Array);
	if (Create_Handle( (CARDINAL_32 *)&memory_Requested
							, (CARDINAL_32)0L
							, ALLOW_PURGING
							, DISALLOW_PARTIAL_AMOUNT
							, (Handle *)&g_Remote_Scenes.info.scenes.handle
							, &g_Remote_Scenes.memory_In_Use) != SAFE_TRUE ) {
							
		Perform_OK_Dialog("\pMemory error allocating scene memory (not enough?) (4)");
		goto Error_Handler;
	}
	
	if (Begin_Use_Of_Handle((Handle)g_Remote_Scenes.info.scenes.handle
							, MOVE_HIGH_REQUIRED
							, &original_State) != SAFE_TRUE) {
		goto Error_Handler;
	}
		
	/*  */
	/* Because locked, we can access using a pointer */
	/*  */		
	g_Remote_Scenes.info.scenes.pointer = *g_Remote_Scenes.info.scenes.handle;
	
	g_Remote_Scenes.recent_Hdl = nil;
	g_Remote_Scenes.recent_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	memory_Requested = k_MAXIMUM_CACHED_DORAMNT_LIST_SIZE;
	if (Create_Handle( (CARDINAL_32 *)&memory_Requested
							, (CARDINAL_32)0L
							, ALLOW_PURGING
							, DISALLOW_PARTIAL_AMOUNT
							, (Handle *)&g_Remote_Scenes.recent_Hdl
							, &g_Remote_Scenes.recent_Memory_In_Use) != SAFE_TRUE ) {
							
		Perform_OK_Dialog("\pMemory error allocating scene memory (not enough?) (4)");
		goto Error_Handler;
	}
	
	if (Begin_Use_Of_Handle((Handle)g_Remote_Scenes.recent_Hdl
							, MOVE_HIGH_REQUIRED
							, &original_State) != SAFE_TRUE) {
		goto Error_Handler;
	}
	g_Remote_Scenes.recent_Length = 0;
	g_Remote_Scenes.recent_Valid = SAFE_FALSE;
	
	g_Remote_Scenes.total_Available = 0;

	for(loop= 0; (loop < k_MAXIMUM_DATABASE_SCENES); loop++) {
		g_Remote_Scenes.info.scenes.pointer->scenes[loop].active = SAFE_FALSE;
		
		g_Remote_Scenes.info.scenes.pointer->scenes[loop].type_Of_Source = k_FROM_ALGORITHM;
		g_Remote_Scenes.info.scenes.pointer->scenes[loop].algorithm_Id = k_BLACK_WINDOW;
	}
	
	/*  */
	/* Setup Empty Screen  */
	/*  */
	g_Remote_Scenes.info.scenes.pointer->scenes[g_Remote_Scenes.total_Available].active = SAFE_TRUE;
	g_Remote_Scenes.info.scenes.pointer->scenes[g_Remote_Scenes.total_Available].type_Of_Source = k_FROM_ALGORITHM;
	g_Remote_Scenes.info.scenes.pointer->scenes[g_Remote_Scenes.total_Available].algorithm_Id = k_NO_WINDOW;
	
	/*  */
	/* Names MUSt be 31 or less characters */
	/*  */
	Copy_Pascal_String(&k_NO_WINDOW_NAME[0], (g_Remote_Scenes.info.scenes.pointer->scenes[g_Remote_Scenes.total_Available].scene_Name));
	g_Remote_Scenes.total_Available += 1;

	/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
	/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */


	g_Puppets.info.puppets.handle = nil;
	g_Puppets.memory_In_Use = MEMORY_NOT_INITIALIZED;
	g_Puppets.info.puppets.pointer = nil;
	
	memory_Requested = sizeof(Puppets_Array);
	if (Create_Handle( (CARDINAL_32 *)&memory_Requested
							, (CARDINAL_32)0L
							, ALLOW_PURGING
							, DISALLOW_PARTIAL_AMOUNT
							, (Handle *)&g_Puppets.info.puppets.handle
							, &g_Puppets.memory_In_Use) != SAFE_TRUE ) {
							
		Perform_OK_Dialog("\pMemory error allocating puppet memory (not enough?) (4)");
		goto Error_Handler;
	}
	
	if (Begin_Use_Of_Handle((Handle)g_Puppets.info.puppets.handle
							, MOVE_HIGH_REQUIRED
							, &original_State) != SAFE_TRUE) {
		goto Error_Handler;
	}
		
	/*  */
	/* Because locked, we can access using a pointer */
	/*  */		
	g_Puppets.info.puppets.pointer = *g_Puppets.info.puppets.handle;
	

	g_Puppets.recent_Hdl = nil;
	g_Puppets.recent_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	memory_Requested = k_MAXIMUM_CACHED_DORAMNT_LIST_SIZE;
	if (Create_Handle( (CARDINAL_32 *)&memory_Requested
							, (CARDINAL_32)0L
							, ALLOW_PURGING
							, DISALLOW_PARTIAL_AMOUNT
							, (Handle *)&g_Puppets.recent_Hdl
							, &g_Puppets.recent_Memory_In_Use) != SAFE_TRUE ) {
							
		Perform_OK_Dialog("\pMemory error allocating puppet memory (not enough?) (4)");
		goto Error_Handler;
	}
	
	if (Begin_Use_Of_Handle((Handle)g_Puppets.recent_Hdl
							, MOVE_HIGH_REQUIRED
							, &original_State) != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	g_Puppets.recent_Length = 0;
	g_Puppets.recent_Valid = SAFE_FALSE;
	
	g_Puppets.total_Available = 0;

	for(loop= 0; (loop < k_MAXIMUM_DATABASE_PUPPETS); loop++) {
		g_Puppets.info.puppets.pointer->puppets[loop].active = SAFE_FALSE;
		
		g_Puppets.info.puppets.pointer->puppets[loop].type_Of_Source = k_FROM_ALGORITHM;
		g_Puppets.info.puppets.pointer->puppets[loop].algorithm_Id = k_BLACK_WINDOW;
	}
	
	/*  */
	/* Setup Empty Screen  */
	/*  */
	g_Puppets.info.puppets.pointer->puppets[g_Puppets.total_Available].active = SAFE_TRUE;
	g_Puppets.info.puppets.pointer->puppets[g_Puppets.total_Available].type_Of_Source = k_FROM_ALGORITHM;
	g_Puppets.info.puppets.pointer->puppets[g_Puppets.total_Available].algorithm_Id = k_NO_WINDOW;
	
	/*  */
	/* Names MUSt be 31 or less characters */
	/*  */
	Copy_Pascal_String(&k_NO_WINDOW_NAME[0], (g_Puppets.info.puppets.pointer->puppets[g_Puppets.total_Available].puppet_Name));
	g_Puppets.total_Available += 1;


	/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
	/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */


	g_Backdrops.info.backdrops.handle = nil;
	g_Backdrops.memory_In_Use = MEMORY_NOT_INITIALIZED;
	g_Backdrops.info.backdrops.pointer = nil;
	
	memory_Requested = sizeof(Backdrops_Array);
	if (Create_Handle( (CARDINAL_32 *)&memory_Requested
							, (CARDINAL_32)0L
							, ALLOW_PURGING
							, DISALLOW_PARTIAL_AMOUNT
							, (Handle *)&g_Backdrops.info.backdrops.handle
							, &g_Backdrops.memory_In_Use) != SAFE_TRUE ) {
							
		Perform_OK_Dialog("\pMemory error allocating backdrop memory (not enough?) (4)");
		goto Error_Handler;
	}
	
	if (Begin_Use_Of_Handle((Handle)g_Backdrops.info.backdrops.handle
							, MOVE_HIGH_REQUIRED
							, &original_State) != SAFE_TRUE) {
		goto Error_Handler;
	}
		
	/*  */
	/* Because locked, we can access using a pointer */
	/*  */		
	g_Backdrops.info.backdrops.pointer = *g_Backdrops.info.backdrops.handle;
	

	g_Backdrops.recent_Hdl = nil;
	g_Backdrops.recent_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	memory_Requested = k_MAXIMUM_CACHED_DORAMNT_LIST_SIZE;
	if (Create_Handle( (CARDINAL_32 *)&memory_Requested
							, (CARDINAL_32)0L
							, ALLOW_PURGING
							, DISALLOW_PARTIAL_AMOUNT
							, (Handle *)&g_Backdrops.recent_Hdl
							, &g_Backdrops.recent_Memory_In_Use) != SAFE_TRUE ) {
							
		Perform_OK_Dialog("\pMemory error allocating backdrop memory (not enough?) (4)");
		goto Error_Handler;
	}
	
	if (Begin_Use_Of_Handle((Handle)g_Backdrops.recent_Hdl
							, MOVE_HIGH_REQUIRED
							, &original_State) != SAFE_TRUE) {
		goto Error_Handler;
	}
	g_Backdrops.recent_Length = 0;
	g_Backdrops.recent_Valid = SAFE_FALSE;
	
	g_Backdrops.total_Available = 0;

	for(loop= 0; (loop < k_MAXIMUM_DATABASE_BACKDROPS); loop++) {
		g_Backdrops.info.backdrops.pointer->backdrops[loop].active = SAFE_FALSE;
		
		g_Backdrops.info.backdrops.pointer->backdrops[loop].type_Of_Source = k_FROM_ALGORITHM;
		g_Backdrops.info.backdrops.pointer->backdrops[loop].algorithm_Id = k_BLACK_WINDOW;
	}
	
	/*  */
	/* Setup Empty Screen  */
	/*  */
	g_Backdrops.info.backdrops.pointer->backdrops[g_Backdrops.total_Available].active = SAFE_TRUE;
	g_Backdrops.info.backdrops.pointer->backdrops[g_Backdrops.total_Available].type_Of_Source = k_FROM_ALGORITHM;
	g_Backdrops.info.backdrops.pointer->backdrops[g_Backdrops.total_Available].algorithm_Id = k_NO_WINDOW;
	
	/*  */
	/* Names MUSt be 31 or less characters */
	/*  */
	Copy_Pascal_String(&k_NO_WINDOW_NAME[0], (g_Backdrops.info.backdrops.pointer->backdrops[g_Backdrops.total_Available].backdrop_Name));
	g_Backdrops.total_Available += 1;



	/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
	/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */


	g_Spin_Objects.info.spin_Objects.handle = nil;
	g_Spin_Objects.memory_In_Use = MEMORY_NOT_INITIALIZED;
	g_Spin_Objects.info.spin_Objects.pointer = nil;
	
	memory_Requested = sizeof(Spin_Objects_Array);
	if (Create_Handle( (CARDINAL_32 *)&memory_Requested
							, (CARDINAL_32)0L
							, ALLOW_PURGING
							, DISALLOW_PARTIAL_AMOUNT
							, (Handle *)&g_Spin_Objects.info.spin_Objects.handle
							, &g_Spin_Objects.memory_In_Use) != SAFE_TRUE ) {
							
		Perform_OK_Dialog("\pMemory error allocating spin_Object memory (not enough?) (4)");
		goto Error_Handler;
	}
	
	if (Begin_Use_Of_Handle((Handle)g_Spin_Objects.info.spin_Objects.handle
							, MOVE_HIGH_REQUIRED
							, &original_State) != SAFE_TRUE) {
		goto Error_Handler;
	}
		
	/*  */
	/* Because locked, we can access using a pointer */
	/*  */		
	g_Spin_Objects.info.spin_Objects.pointer = *g_Spin_Objects.info.spin_Objects.handle;
	

	g_Spin_Objects.recent_Hdl = nil;
	g_Spin_Objects.recent_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	memory_Requested = k_MAXIMUM_CACHED_DORAMNT_LIST_SIZE;
	if (Create_Handle( (CARDINAL_32 *)&memory_Requested
							, (CARDINAL_32)0L
							, ALLOW_PURGING
							, DISALLOW_PARTIAL_AMOUNT
							, (Handle *)&g_Spin_Objects.recent_Hdl
							, &g_Spin_Objects.recent_Memory_In_Use) != SAFE_TRUE ) {
							
		Perform_OK_Dialog("\pMemory error allocating spin_Object memory (not enough?) (4)");
		goto Error_Handler;
	}
	
	if (Begin_Use_Of_Handle((Handle)g_Spin_Objects.recent_Hdl
							, MOVE_HIGH_REQUIRED
							, &original_State) != SAFE_TRUE) {
		goto Error_Handler;
	}
	g_Spin_Objects.recent_Length = 0;
	g_Spin_Objects.recent_Valid = SAFE_FALSE;
	
	g_Spin_Objects.total_Available = 0;

	for(loop= 0; (loop < k_MAXIMUM_DATABASE_SPIN_OBJECTS); loop++) {
		g_Spin_Objects.info.spin_Objects.pointer->spin_Objects[loop].active = SAFE_FALSE;
		
		g_Spin_Objects.info.spin_Objects.pointer->spin_Objects[loop].type_Of_Source = k_FROM_ALGORITHM;
		g_Spin_Objects.info.spin_Objects.pointer->spin_Objects[loop].algorithm_Id = k_BLACK_WINDOW;
	}
	
	/*  */
	/* Setup Empty Screen  */
	/*  */
	g_Spin_Objects.info.spin_Objects.pointer->spin_Objects[g_Spin_Objects.total_Available].active = SAFE_TRUE;
	g_Spin_Objects.info.spin_Objects.pointer->spin_Objects[g_Spin_Objects.total_Available].type_Of_Source = k_FROM_ALGORITHM;
	g_Spin_Objects.info.spin_Objects.pointer->spin_Objects[g_Spin_Objects.total_Available].algorithm_Id = k_NO_WINDOW;
	
	/*  */
	/* Names MUSt be 31 or less characters */
	/*  */
	Copy_Pascal_String(&k_NO_WINDOW_NAME[0], (g_Spin_Objects.info.spin_Objects.pointer->spin_Objects[g_Spin_Objects.total_Available].spin_Object_Name));
	g_Spin_Objects.total_Available += 1;



	/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
	/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */


	g_Audio_Files.info.sounds.handle = nil;
	g_Audio_Files.memory_In_Use = MEMORY_NOT_INITIALIZED;
	g_Audio_Files.info.sounds.pointer = nil;
	
	memory_Requested = sizeof(Sounds_Array);
	if (Create_Handle( (CARDINAL_32 *)&memory_Requested
							, (CARDINAL_32)0L
							, ALLOW_PURGING
							, DISALLOW_PARTIAL_AMOUNT
							, (Handle *)&g_Audio_Files.info.sounds.handle
							, &g_Audio_Files.memory_In_Use) != SAFE_TRUE ) {
							
		Perform_OK_Dialog("\pMemory error allocating Sounds_Array memory (not enough?) (4)");
		goto Error_Handler;
	}
	
	if (Begin_Use_Of_Handle((Handle)g_Audio_Files.info.sounds.handle
							, MOVE_HIGH_REQUIRED
							, &original_State) != SAFE_TRUE) {
		goto Error_Handler;
	}
		
	/*  */
	/* Because locked, we can access using a pointer */
	/*  */		
	g_Audio_Files.info.sounds.pointer = *g_Audio_Files.info.sounds.handle;
	

	g_Audio_Files.recent_Hdl = nil;
	g_Audio_Files.recent_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	memory_Requested = k_MAXIMUM_CACHED_DORAMNT_LIST_SIZE;
	if (Create_Handle( (CARDINAL_32 *)&memory_Requested
							, (CARDINAL_32)0L
							, ALLOW_PURGING
							, DISALLOW_PARTIAL_AMOUNT
							, (Handle *)&g_Audio_Files.recent_Hdl
							, &g_Audio_Files.recent_Memory_In_Use) != SAFE_TRUE ) {
							
		Perform_OK_Dialog("\pMemory error allocating Sounds_Array memory (not enough?) (4)");
		goto Error_Handler;
	}
	
	if (Begin_Use_Of_Handle((Handle)g_Audio_Files.recent_Hdl
							, MOVE_HIGH_REQUIRED
							, &original_State) != SAFE_TRUE) {
		goto Error_Handler;
	}
	g_Audio_Files.recent_Length = 0;
	g_Audio_Files.recent_Valid = SAFE_FALSE;
	
	g_Audio_Files.total_Available = 0;

	for(loop= 0; (loop < k_MAXIMUM_DATABASE_SOUND_OBJECTS); loop++) {
		g_Audio_Files.info.sounds.pointer->sound_Objects[loop].active = SAFE_FALSE;
		
		g_Audio_Files.info.sounds.pointer->sound_Objects[loop].type_Of_Source = k_FROM_ALGORITHM;
		g_Audio_Files.info.sounds.pointer->sound_Objects[loop].algorithm_Id = k_BLACK_WINDOW;
	}
	
	/*  */
	/* Setup Empty Screen  */
	/*  */
	g_Audio_Files.info.sounds.pointer->sound_Objects[g_Audio_Files.total_Available].active = SAFE_TRUE;
	g_Audio_Files.info.sounds.pointer->sound_Objects[g_Audio_Files.total_Available].type_Of_Source = k_FROM_ALGORITHM;
	g_Audio_Files.info.sounds.pointer->sound_Objects[g_Audio_Files.total_Available].algorithm_Id = k_NO_WINDOW;
	
	/*  */
	/* Names MUSt be 31 or less characters */
	/*  */
	Copy_Pascal_String(&k_NO_WINDOW_NAME[0], (g_Audio_Files.info.sounds.pointer->sound_Objects[g_Audio_Files.total_Available].sound_Object_Name));
	g_Audio_Files.total_Available += 1;



	/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
	/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
	

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
	
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Destroy_Object_Inventory(void)
{
	Safe_Boolean		function_Result;
/* END VAR */

/*  */
/* Global persistant structures are deleted here */
/*  */

	function_Result = SAFE_TRUE;
	return(function_Result);
}
/*=**************************************************************************=*/		

		
/*=**************************************************************************=*/		
Safe_Boolean Locate_Item_Location(CARDINAL_32		object_Classification
																, Str63			*item_Name
																, FSSpec		*file_Specification
																, CARDINAL_32	*item_Address
																, Safe_Boolean	*address_Instead_Of_File)
/*  */
/* Tries to locate in RAM first, then uses disk based information */
/* from scanned database */
/*  */
{
	Safe_Boolean				function_Result;
	Safe_Boolean				working_Directory_In_Use;
	Handle					search_Buffer_Hdl;
	SignedByte				search_Buffer_Original_State;
	Memory_Allocation		search_Buffer_Hdl_Memory_Initialized;
	HParamBlockRec			hfs_Parameter_Block;
	INTEGER_16				working_Directory;
	CARDINAL_32			total_Items_Found;
	CARDINAL_32			cache_List_Classification;
	WDPBRec					working_Directory_PB;
	OSErr						os_Error;
	CSParam					search;
	CInfoPBRec				search_Spec_1;
	CInfoPBRec				search_Spec_2;
	Str63						volume_Name;
	Str255						gFileName;
	INTEGER_32				index;
	CARDINAL_32			preferred_Size;
	CARDINAL_32			type_To_Match;
	CARDINAL_32			items_Available_To_Search;
	Hierarchial_Location		location_HFS;
	Boolean					done_Searching;
	Boolean					still_Searching;
	FSSpec					search_Results[k_MAXIMUM_FILE_MATCHES];
	CARDINAL_8				*code_Name;
/* END VAR */


	code_Name = "\pLocate_Item_Location";
	function_Result = SAFE_FALSE;
	working_Directory_In_Use = SAFE_FALSE;
	*item_Address = 0;
	*address_Instead_Of_File = SAFE_FALSE;
	search_Buffer_Hdl_Memory_Initialized = MEMORY_NOT_INITIALIZED;
	



	/*  */
	/* First, scan through Items in memory, and if none found then consult */
	/* list of files available. */
	/* In this temporary version of this routine, a search is made local to the */
	/* application file as well */
	/*  */

	if (function_Result != SAFE_TRUE) {

		switch (object_Classification) {
			case	k_SCENE_SELECTION:
					items_Available_To_Search = dream.play.total_Scenes;
				
					if (items_Available_To_Search > 0) {
						still_Searching = TRUE;
						total_Items_Found = 0;
						for (index = 0; ((index < k_MAX_ACTIVE_SCENES)
									&& (total_Items_Found < items_Available_To_Search)
									&& (still_Searching == TRUE)); index++) {
									
							if (dream.play.scene_Memory_In_Use[index] == MEMORY_INITIALIZED) {
								total_Items_Found += 1;
								
								if (((**dream.play.scene[index]).initialized == SAFE_TRUE)
									&& ((**dream.play.scene[index]).items_Remaining_To_Load != SAFE_TRUE)) {
									
									if (Compare_Pascal_Strings_Ignoring_Case(&((**dream.play.scene[index]).file_Name[0])
																															, &(*item_Name)[0]) == SAFE_TRUE) {
									
										still_Searching = FALSE;
										function_Result = SAFE_TRUE;
										*address_Instead_Of_File = SAFE_TRUE;
										*item_Address = index;
										/*  */
										/* Paranoia clearing */
										/*  */
										file_Specification->vRefNum = 0;
										file_Specification->parID = 0;
										file_Specification->name[0] = 0;
									}
								}
							}
						}
					}
					break;
					
			case	k_PUPPET_SELECTION:
			case	k_PROP_SELECTION:
					items_Available_To_Search = dream.active_Puppets_Total;
				
					if (items_Available_To_Search > 0) {
						still_Searching = TRUE;
						total_Items_Found = 0;
						for (index = 0; ((index < k_MAXIMUM_PUPPETS)
									&& (total_Items_Found < items_Available_To_Search)
									&& (still_Searching == TRUE)); index++) {
									
							if (dream.puppet_Memory_Initialized[index] == MEMORY_INITIALIZED) {
								total_Items_Found += 1;
								
								if ((**dream.puppet[index]).puppet_Initialized == SAFE_TRUE) {
									if (Compare_Pascal_Strings_Ignoring_Case(&((**dream.puppet[index]).file_Name[0])
																															, &(*item_Name)[0]) == SAFE_TRUE) {
									
										still_Searching = FALSE;
										function_Result = SAFE_TRUE;
										*address_Instead_Of_File = SAFE_TRUE;
										*item_Address = index;
										/*  */
										/* Paranoia clearing */
										/*  */
										file_Specification->vRefNum = 0;
										file_Specification->parID = 0;
										file_Specification->name[0] = 0;
									}
								}
							}
						}
					}
					break;
					
			case	k_BACKDROP_SELECTION:
			
					items_Available_To_Search = dream.backdrops.total_Backdrops;
				
					if (items_Available_To_Search > 0) {
						still_Searching = TRUE;
						total_Items_Found = 0;
						for (index = 0; ((index < k_MAX_ACTIVE_BACKDROPS)
									&& (total_Items_Found < items_Available_To_Search)
									&& (still_Searching == TRUE)); index++) {
									
							if (dream.backdrops.backdrop_Memory_In_Use[index] == MEMORY_INITIALIZED) {
								total_Items_Found += 1;
								
								if (((**dream.backdrops.backdrop[index]).initialized == SAFE_TRUE)
									&& ((**dream.backdrops.backdrop[index]).backdrop_Remaining_To_Load != SAFE_TRUE)) {
									
									if (Compare_Pascal_Strings_Ignoring_Case(&((**dream.backdrops.backdrop[index]).backdrop_Name[0])
																															, &(*item_Name)[0]) == SAFE_TRUE) {
									
										if ((**dream.backdrops.backdrop[index]).special_Effect_Backdrop == 0) {
											still_Searching = FALSE;
											function_Result = SAFE_TRUE;
											*address_Instead_Of_File = SAFE_TRUE;
											*item_Address = index;
											/*  */
											/* Paranoia clearing */
											/*  */
											file_Specification->vRefNum = 0;
											file_Specification->parID = 0;
											file_Specification->name[0] = 0;
										}
										else {
											/*  */
											/* We do not support special effect backdrops yet */
											/*  */
										}
									}
								}
							}
						}
					}
					break;



			case	k_SPIN_OBJECT_SELECTION:
			
					items_Available_To_Search = dream.spin_Objects.total_Spin_Objects;
				
					if (items_Available_To_Search > 0) {
						still_Searching = TRUE;
						total_Items_Found = 0;
						for (index = 0; ((index < k_MAX_ACTIVE_SPIN_OBJECTS)
									&& (total_Items_Found < items_Available_To_Search)
									&& (still_Searching == TRUE)); index++) {
									
							if (dream.spin_Objects.spin_Object_Memory_In_Use[index] == MEMORY_INITIALIZED) {
								total_Items_Found += 1;
								
								if (((**dream.spin_Objects.spin_Object[index]).initialized == SAFE_TRUE)
									&& ((**dream.spin_Objects.spin_Object[index]).spin_Object_Remaining_To_Load != SAFE_TRUE)) {
									
									if (Compare_Pascal_Strings_Ignoring_Case(&((**dream.spin_Objects.spin_Object[index]).spin_Object_Name[0])
																															, &(*item_Name)[0]) == SAFE_TRUE) {
									
										if ((**dream.spin_Objects.spin_Object[index]).special_Effect_Spin_Object == 0) {
											still_Searching = FALSE;
											function_Result = SAFE_TRUE;
											*address_Instead_Of_File = SAFE_TRUE;
											*item_Address = index;
											/*  */
											/* Paranoia clearing */
											/*  */
											file_Specification->vRefNum = 0;
											file_Specification->parID = 0;
											file_Specification->name[0] = 0;
										}
										else {
											/*  */
											/* We do not support special effect spin_Objects yet */
											/*  */
										}
									}
								}
							}
						}
					}
					break; /* k_SPIN_OBJECT_SELECTION */


			default:
				Display_Serious_Error_Message(unit_Name, code_Name, "\pUnexpected case", "\pBad programming");
				goto Error_Handler;
		}
	}	

	if (function_Result != SAFE_TRUE) {

		switch (object_Classification) {
			case	k_SCENE_SELECTION:
						cache_List_Classification = k_SCENE_FILE_SELECTION;
					break;
					
			case	k_PUPPET_SELECTION:
						cache_List_Classification = k_PUPPET_FILE_SELECTION;
					break;
					
			case	k_PROP_SELECTION:
						cache_List_Classification = k_PROP_FILE_SELECTION;
					break;
					
			case	k_BACKDROP_SELECTION:
						cache_List_Classification = k_BACKDROP_FILE_SELECTION;
					break;

			case	k_SPIN_OBJECT_SELECTION:
						cache_List_Classification = k_SPIN_OBJECT_FILE_SELECTION;
					break;

/* ===
			case	k_AUDIO_OBJECT_SELECTION:
						cache_List_Classification = k_AUDIO_OBJECT_FILE_SELECTION;
					break;
=== */


			default:
				Display_Serious_Error_Message(unit_Name, code_Name, "\pUnexpected case", "\pBad programming");
				goto Error_Handler;
		}
	}	

	/*  */
	/* If still not found, try in-memory list of cached available files */
	/*  */
	
	if (function_Result != SAFE_TRUE) {
	
		switch (cache_List_Classification) {
			case	k_SCENE_FILE_SELECTION:
						type_To_Match = k_SCENE_FILE_TYPE;
	
						if (g_Remote_Scenes.total_Available > 0) {
							still_Searching = TRUE;
							total_Items_Found = 0;
							for (index = 0; ((index < k_MAXIMUM_DATABASE_SCENES)
										&& (total_Items_Found < g_Remote_Scenes.total_Available)
										&& (still_Searching == TRUE)); index++) {
								if ((**g_Remote_Scenes.info.scenes.handle).scenes[index].active == SAFE_TRUE) {
									total_Items_Found += 1;
									if (Compare_Pascal_Strings_Ignoring_Case((**g_Remote_Scenes.info.scenes.handle).scenes[index].scene_Name, &(*item_Name)[0]) == SAFE_TRUE) {
										if ((**g_Remote_Scenes.info.scenes.handle).scenes[index].type_Of_Source == k_FROM_FILE) {
											still_Searching = FALSE;
											function_Result = SAFE_TRUE;
											*file_Specification = (**g_Remote_Scenes.info.scenes.handle).scenes[index].file_Info;
											*address_Instead_Of_File = SAFE_FALSE;
										}
									}
								}
							}
						}
					break;
	
			case	k_PUPPET_FILE_SELECTION:
			case	k_PROP_FILE_SELECTION:
						type_To_Match = k_PUPPET_FILE_TYPE;
	
						if (g_Puppets.total_Available > 0) {
							still_Searching = TRUE;
							total_Items_Found = 0;
							for (index = 0; ((index < k_MAXIMUM_DATABASE_PUPPETS) 
										&& (total_Items_Found < g_Puppets.total_Available)
										&& (still_Searching == TRUE)); index++) {
								if ((**g_Puppets.info.puppets.handle).puppets[index].active == SAFE_TRUE) {
									total_Items_Found += 1;
									if (Compare_Pascal_Strings_Ignoring_Case((**g_Puppets.info.puppets.handle).puppets[index].puppet_Name, &(*item_Name)[0]) == SAFE_TRUE) {
										if ((**g_Puppets.info.puppets.handle).puppets[index].type_Of_Source == k_FROM_FILE) {
											still_Searching = FALSE;
											function_Result = SAFE_TRUE;
											*file_Specification = (**g_Puppets.info.puppets.handle).puppets[index].file_Info;
											*address_Instead_Of_File = SAFE_FALSE;
										}
									}
								}
							}
						}
					break;
	
			case	k_BACKDROP_FILE_SELECTION:
						type_To_Match = k_BACKDROP_FILE_TYPE;
	
						if (g_Backdrops.total_Available > 0) {
							still_Searching = TRUE;
							total_Items_Found = 0;
							for (index = 0; ((index < k_MAXIMUM_DATABASE_BACKDROPS) 
										&& (total_Items_Found < g_Backdrops.total_Available)
										&& (still_Searching == TRUE)); index++) {
								if ((**g_Backdrops.info.backdrops.handle).backdrops[index].active == SAFE_TRUE) {
									total_Items_Found += 1;
									if (Compare_Pascal_Strings_Ignoring_Case((**g_Backdrops.info.backdrops.handle).backdrops[index].backdrop_Name, &(*item_Name)[0]) == SAFE_TRUE) {
										if ((**g_Backdrops.info.backdrops.handle).backdrops[index].type_Of_Source == k_FROM_FILE) {
											still_Searching = FALSE;
											function_Result = SAFE_TRUE;
											*file_Specification = (**g_Backdrops.info.backdrops.handle).backdrops[index].file_Info;
											*address_Instead_Of_File = SAFE_FALSE;
										}
									}
								}
							}
						}
					break;				
					
			case	k_SPIN_OBJECT_FILE_SELECTION:
						type_To_Match = k_SPIN_OBJECT_FILE_TYPE;
	
						if (g_Spin_Objects.total_Available > 0) {
							still_Searching = TRUE;
							total_Items_Found = 0;
							for (index = 0; ((index < k_MAXIMUM_DATABASE_SPIN_OBJECTS) 
										&& (total_Items_Found < g_Spin_Objects.total_Available)
										&& (still_Searching == TRUE)); index++) {
								if ((**g_Spin_Objects.info.spin_Objects.handle).spin_Objects[index].active == SAFE_TRUE) {
									total_Items_Found += 1;
									if (Compare_Pascal_Strings_Ignoring_Case((**g_Spin_Objects.info.spin_Objects.handle).spin_Objects[index].spin_Object_Name, &(*item_Name)[0]) == SAFE_TRUE) {
										if ((**g_Spin_Objects.info.spin_Objects.handle).spin_Objects[index].type_Of_Source == k_FROM_FILE) {
											still_Searching = FALSE;
											function_Result = SAFE_TRUE;
											*file_Specification = (**g_Spin_Objects.info.spin_Objects.handle).spin_Objects[index].file_Info;
											*address_Instead_Of_File = SAFE_FALSE;
										}
									}
								}
							}
						}
					break;				
					
			case	k_AUDIO_OBJECT_FILE_SELECTION:
						type_To_Match = k_AIFF_TYPE;
	
						if (g_Audio_Files.total_Available > 0) {
							still_Searching = TRUE;
							total_Items_Found = 0;
							for (index = 0; ((index < k_MAXIMUM_DATABASE_SOUND_OBJECTS) 
										&& (total_Items_Found < g_Audio_Files.total_Available)
										&& (still_Searching == TRUE)); index++) {
								if ((**g_Audio_Files.info.sounds.handle).sound_Objects[index].active == SAFE_TRUE) {
									total_Items_Found += 1;
									if (Compare_Pascal_Strings_Ignoring_Case((**g_Audio_Files.info.sounds.handle).sound_Objects[index].sound_Object_Name, &(*item_Name)[0]) == SAFE_TRUE) {
										if ((**g_Audio_Files.info.sounds.handle).sound_Objects[index].type_Of_Source == k_FROM_FILE) {
											still_Searching = FALSE;
											function_Result = SAFE_TRUE;
											*file_Specification = (**g_Audio_Files.info.sounds.handle).sound_Objects[index].file_Info;
											*address_Instead_Of_File = SAFE_FALSE;
										}
									}
								}
							}
						}
					break;				

					
			default:
				goto Error_Handler;
		}
	} /* if (function_Result == SAFE_TRUE) */
		
	
	/*  */
	/* If still not found, try recursive disk scan */
	/*  */
	if (function_Result != SAFE_TRUE) {
					
		preferred_Size = k_SEARCH_BUFFER_SIZE;
		if (Create_Handle(&preferred_Size
							, (CARDINAL_32)(10240L)
							, ALLOW_PURGING
							, DISALLOW_PARTIAL_AMOUNT
							, (Handle *)&search_Buffer_Hdl
							, &search_Buffer_Hdl_Memory_Initialized) != SAFE_TRUE) {
							
			Display_Low_Memory_Error_Message(unit_Name, code_Name, k_SEARCH_BUFFER_SIZE);
			
			function_Result = SAFE_FALSE;
			goto Error_Handler;
		}
		
		if (Begin_Use_Of_Handle((Handle)search_Buffer_Hdl
										, MOVE_HIGH_NOT_REQUIRED
										, &search_Buffer_Original_State) != SAFE_TRUE) {
			goto Error_Handler;
		}
		
		/*  */
		/* Crappy deficient PBCatSearch needs ancient Working Directory */
		/*  */
		working_Directory_PB.ioCompletion = nil;
		working_Directory_PB.ioNamePtr = nil;
		working_Directory_PB.ioWDProcID = FILE_CREATOR;
		working_Directory_PB.ioVRefNum = ns_Shell_Globals.application_Volume_Reference;
		working_Directory_PB.ioWDDirID = ns_Shell_Globals.application_Directory_ID;
		
		os_Error = PBOpenWD(&working_Directory_PB, FALSE);
		if (os_Error != noErr) {
			goto Error_Handler;
		}
		working_Directory_In_Use = SAFE_TRUE;
		
		/*  */
		/* Crappy deficient PBCatSearch needs ancient Working Directory and */
		/* for some reason also needs a damned actual volume name! */
		/*  */
		working_Directory = working_Directory_PB.ioVRefNum;
		hfs_Parameter_Block.volumeParam.ioCompletion = nil;
		hfs_Parameter_Block.volumeParam.ioNamePtr = (StringPtr)&volume_Name[0];
		hfs_Parameter_Block.volumeParam.ioVRefNum = ns_Shell_Globals.application_Volume_Reference;
		hfs_Parameter_Block.volumeParam.ioVolIndex = 0;
		
		os_Error = PBHGetVInfo(&hfs_Parameter_Block, FALSE);
		if (os_Error != noErr) {
			goto Error_Handler;
		}
					
		Copy_Pascal_String((CARDINAL_8 *)*item_Name, gFileName);
		
		search.ioCompletion = nil;
		search.ioNamePtr = &volume_Name[0];
		search.ioVRefNum = working_Directory;
		search.ioMatchPtr = search_Results;
		search.ioReqMatchCount = k_MAXIMUM_FILE_MATCHES;
		
		search.ioSearchBits = fsSBFullName + fsSBFlAttrib + fsSBFlFndrInfo;
		
		search.ioSearchInfo1 = &search_Spec_1; 
		search.ioSearchInfo2 = &search_Spec_2;
		search.ioSearchTime = 0;
		search.ioCatPosition.initialize = 0;
		
		search.ioOptBuffer = *search_Buffer_Hdl;
		search.ioOptBufSize  = k_SEARCH_BUFFER_SIZE;
	
		search_Spec_1.hFileInfo.ioNamePtr = &(gFileName[0]);
		search_Spec_1.hFileInfo.ioFlAttrib = 0x00;
		search_Spec_1.hFileInfo.ioFlFndrInfo.fdType = type_To_Match;
		
		/*  */
		/* Set mask data and high limits */
		/*  */
		search_Spec_2.hFileInfo.ioNamePtr = nil;
		search_Spec_2.hFileInfo.ioFlAttrib  = 0x10;
		search_Spec_2.hFileInfo.ioFlFndrInfo.fdType = 0xFFFF;
		search_Spec_2.hFileInfo.ioFlFndrInfo.fdCreator = 0;
		search_Spec_2.hFileInfo.ioFlFndrInfo.fdFlags = 0;
		search_Spec_2.hFileInfo.ioFlFndrInfo.fdLocation.h = 0;
		search_Spec_2.hFileInfo.ioFlFndrInfo.fdLocation.v = 0;
		search_Spec_2.hFileInfo.ioFlFndrInfo.fdFldr = 0;

		done_Searching = FALSE;
		do {
			search.ioActMatchCount = 0;
			os_Error = PBCatSearchSync(&search);			
			if (os_Error == eofErr) {
				done_Searching = TRUE;
			}
			
			if (((os_Error == noErr) || (done_Searching == TRUE)) && (search.ioActMatchCount > 0)) {
				/*  */
				/* Something is in the match buffers */
				/* Check if they are trhe ones we want ( a good routine would */
				/* open them up and serach for internal file name) */
				/*  */
				for (index = 0; (index < search.ioActMatchCount); index++) {
				
					/*  */
					/* Just take first one we find */
					/*  */
					*file_Specification = search_Results[index];
					done_Searching = TRUE;
					*address_Instead_Of_File = SAFE_FALSE;
					function_Result = SAFE_TRUE;
					
				}
			}
		} while (done_Searching != TRUE);
			
	} /* if (function_Result != SAFE_TRUE) */
	
	/*  */
	/* At this point, result has been determined and set already */
	/*  */

	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

		/*  */
		/* Placed here to ensure not accidentally ever left allocated */
		/*  */
		if (working_Directory_In_Use == SAFE_TRUE) {
			os_Error = CloseWD(working_Directory);
			if (os_Error != noErr) {
				goto Error_Handler;
			}
		}
			
		if (search_Buffer_Hdl_Memory_Initialized == MEMORY_INITIALIZED) {
			if (Destroy_Handle((Handle *)&search_Buffer_Hdl
								, &search_Buffer_Hdl_Memory_Initialized) != SAFE_TRUE) {
								
				Display_Serious_Error_Message(unit_Name, code_Name, "\pError with Destroy_Handle", "\pSystem Corrupted?");
			}
		}

	return(function_Result);
	
} /* Safe_Boolean Locate_Item_Location ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Test_Reentrant_Search(INTEGER_16 volume_Ref_Num_To_Search
								, INTEGER_32 directory_To_Search
								, Recursive_Search_Record	*pre_Cache)
{
	/*  */
	/* These two are superfluous to set, but are vital for */
	/* standalone code */
	/*  */
	(*pre_Cache).data_Valid = SAFE_TRUE;
	(*pre_Cache).volume_Reference = volume_Ref_Num_To_Search;
	(*pre_Cache).param_Block.hFileInfo.ioVRefNum = volume_Ref_Num_To_Search;
	
	(*pre_Cache).initialization_Directory = directory_To_Search;
	(*pre_Cache).initialization_Index = 1;
	(*pre_Cache).stack_Total = 0;
	
	(*pre_Cache).file_Type_To_Match[0] = 'TEXT';
	(*pre_Cache).file_Type_To_Match_Total = 1;
	
	
	
	do {
	
		Continue_Reentrant_Search(pre_Cache);
		
		/*  */
		/* "1" means do until first timeout, else "0" */
		/*  */
	} while ((*pre_Cache).return_Value_Determined == 1);
	

			
} /* void Test_Reentrant_Search(INTEGER_16 vRefNumToSearch ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Continue_Reentrant_Search (Recursive_Search_Record	*pre_Cache)
{
	INTEGER_16		index;
	INTEGER_16		loop;
	INTEGER_32 		dirIDToSearch;
	CARDINAL_32	type_To_Match[k_MAX_FILE_MATCH_TYPES];
	CARDINAL_32	type_To_Match_Total;
	CARDINAL_32	this_File_Type;
   	OSErr        			os_Error;
   	OSErr        			param_Build_Error;
	INTEGER_16		starting_Index;
	INTEGER_16		vRefNum_To_Search;
	Boolean			timeout_Search_Upon_Next_Popup;
	FSSpec			file_Specification;
	Boolean			database_Full;
	Boolean			found_Possible_Match;

	CInfoPBRec			*temp_PB_Ptr;
	INTEGER_16			*temp_Stack_Ptr;
	Str63          			item_Name;
/* END VAR */
		
		
	if ((*pre_Cache).data_Valid != SAFE_TRUE) {
		/*  */
		/* Simple paranoia check */
		/* Because a boolean is not passed back we cannot use the */
		/* normal treatment of "return_Value_Determined" to warn the caller. */
		/*  */
		goto Normal_Function_Exit;
	}
	
	/*  */
	/* temp_PB_Ptr was added to enhance speed of not needing */
	/* to dereference as often. The overhead of the Mac OS calls makes */
	/* the optimization not that significant though. */
	/*  */
		
	temp_PB_Ptr	= &((*pre_Cache).param_Block);
	temp_Stack_Ptr = &((*pre_Cache).stack_Of_Indices[0]);

	temp_PB_Ptr->hFileInfo.ioNamePtr = &item_Name[0];
	index = (*pre_Cache).initialization_Index;
	dirIDToSearch = (*pre_Cache).initialization_Directory;
	
	
	type_To_Match_Total = (*pre_Cache).file_Type_To_Match_Total;
	for (loop = 0 ; (loop < type_To_Match_Total) ; loop++) {
		type_To_Match[loop] = (*pre_Cache).file_Type_To_Match[loop];
	}
	
	(*pre_Cache).stop_Search_Time = (CARDINAL_32)TickCount();
	(*pre_Cache).stop_Search_Time += (CARDINAL_32)k_RECURSIVE_SEARCH_ACTIVE_TIMEOUT;
														
	(*pre_Cache).return_Value_Determined = 0;
	
	/*  */
	/* Store current volume for use when PB is blasted */
	/*  */
	vRefNum_To_Search = (*pre_Cache).volume_Reference;

	/*  */
	/* 1993.02.19 JAM : bug fix?, restore current volume for */
	/* use when PB is blasted */
	/*  */
	temp_PB_Ptr->hFileInfo.ioVRefNum = vRefNum_To_Search;

	/*  */
	/* Flag allows graceful yet strict timeout condition */
	/*  */
	timeout_Search_Upon_Next_Popup = FALSE;

Start_Indexing_Again:

	do {
		
		temp_PB_Ptr->hFileInfo.ioFDirIndex= index;
		
		/*  */
		/* We need to do this every time through, since*/
		/* GetCatInfo returns ioFlNum in this field */
		/*  */
		temp_PB_Ptr->hFileInfo.ioDirID= dirIDToSearch; 
										
		/*  */
		/* Clear the ioACUser byte if search is interested in it.*/
		/*  Nonserver volumes won't clear it for you and*/
		/*  the value returned is meaningless.*/
		/*  */
		temp_PB_Ptr->hFileInfo.filler2= 0; 
							
		os_Error= PBGetCatInfo(temp_PB_Ptr,false);
		if (os_Error == noErr) {
		
		/*  */
		/* We have a directory */
		/*  */
			if ((temp_PB_Ptr->hFileInfo.ioFlAttrib & ioDirMask) != 0) {
			
				if ((*pre_Cache).stack_Total < k_MAX_STACK_DEPTH_ALLOWED) {
										
					(*pre_Cache).stack_Of_Indices[(*pre_Cache).stack_Total] = index +1;
					(*pre_Cache).stack_Total += 1;

					dirIDToSearch = temp_PB_Ptr->dirInfo.ioDrDirID;
					index = 1;
					goto Start_Indexing_Again;
				}
				else {
					/*  */
					/* We have to ignore this directory, we are not allowed to recurse any deeper */
					/*  */
					
					/*  */
					/* WARNING: IF CRITICAL APPLICATION, THIS IS AN ERROR */
					/*  */
					
				}
			}
			else {
				/*  */
				/* We have a File */
				/*  */
				
			/* ===
				temp_PB_Ptr->hFileInfo.ioFlFndrInfo.fdCreator
			=== */
				
				this_File_Type = temp_PB_Ptr->hFileInfo.ioFlFndrInfo.fdType;
				
				found_Possible_Match = FALSE;
				for (loop = 0 ; ((loop < type_To_Match_Total) && (found_Possible_Match != TRUE)); loop++) {
					if (type_To_Match[loop] == this_File_Type) {
						found_Possible_Match = TRUE;
					}
				}
				
				if (found_Possible_Match == TRUE) {
				
					/*  */
					/* Found what we want! (Should safety check the item) */
					/*  */
					
					/*  */
					/* FSMakeFSSpec is a slow wasteful step but not too bad */
					/* for the purposes it serves (we already know we want this item */
					/*  */
					param_Build_Error = FSMakeFSSpec(temp_PB_Ptr->hFileInfo.ioVRefNum
									, temp_PB_Ptr->hFileInfo.ioFlParID
									, temp_PB_Ptr->hFileInfo.ioNamePtr
									,&file_Specification);
												
					if (param_Build_Error == noErr) {
					
						database_Full = FALSE;
						if (Add_Potential_File_Item_To_Database(this_File_Type, &file_Specification, &database_Full) != SAFE_TRUE) {
							/*  */
							/* Fatal errors not ever expected */
							/*  */
							database_Full = TRUE;
						}
						
						if (database_Full == TRUE) {
							/*  */
							/* ABORT ABORT ABORT : The one place where it is */
							/* actually aborted. Do so by setting "return_Value_Determined" to 0 */
							/*  */
							(*pre_Cache).return_Value_Determined = 0;
							(*pre_Cache).initialization_Index = index;
							(*pre_Cache).initialization_Directory = dirIDToSearch;
							(*pre_Cache).stack_Total = 0;
							goto Normal_Function_Exit;
						}
						
					}
					else {
						/* Odd! But "param_Build_Error" holds our FSSpec error. Ignore */
						; /* NULL */
					}
				}
												
			} /* if ((temp_PB_Ptr->hFileInfo.ioFlAttrib & ioDirMask) != 0) */
		}
		else {
			/*  */
			/* This error condition at this point simply means that no more */
			/* files are indexable from here (no more in list, and it should be */
			/* handled further on */
			/*  */
			
			/*  */
			/* 1993.02.19 JAM : bug fix?, restore current volume for */
			/* use when PB may be blasted */
			/*  */
			temp_PB_Ptr->hFileInfo.ioVRefNum = vRefNum_To_Search;
		}
		
		/*  */
		/* If current item yielded error, this logic will treat the directory as */
		/* if it has no more items. Could potentially be bad if this */
		/* code is used in a CRITICAL APPLICATION */
		/*  */
		
		index +=1;
		
		if ((CARDINAL_32)TickCount() >= (*pre_Cache).stop_Search_Time) {
				
			/*  */
			/* 1993.02.19 JAM : bug fix?, Was not honoring "os_Error" and could have */
			/* sent a false "1" value back to the caller when an osError could have been significant*/
			/* and, as can be seen, the index would infinitely increment higher and higher */
			/* exiting out each time if the timeout requirements were met */
			/*  */
			if (os_Error == noErr) {
				if ((*pre_Cache).return_Value_Determined == 0) {
					(*pre_Cache).return_Value_Determined = 1;
					(*pre_Cache).initialization_Index = index;
					(*pre_Cache).initialization_Directory = dirIDToSearch;
					goto Normal_Function_Exit;
				}
			}
			else {
				timeout_Search_Upon_Next_Popup = TRUE;
			}
		}		
		
	} while (os_Error == noErr);
	
		
	/*  */
	/* Reset error flag */
	/*  */
	os_Error = noErr;

	/*  */
	/* Pop Up a directory, if anything is on the stack. */
	/*  */	
	
	if ((*pre_Cache).stack_Total > 0) {
	
		/*  */
		/* 1993.02.19 JAM : bug fix?, store current volume for */
		/* use when PB is blasted */
		/*  */
		temp_PB_Ptr->hFileInfo.ioVRefNum = vRefNum_To_Search;

		temp_PB_Ptr->dirInfo.ioDrDirID = dirIDToSearch;
		temp_PB_Ptr->hFileInfo.ioFDirIndex = -1;
		
		os_Error= PBGetCatInfo(temp_PB_Ptr ,false);
		if (os_Error == noErr)
		{
		
			dirIDToSearch = temp_PB_Ptr->dirInfo.ioDrParID;
			(*pre_Cache).stack_Total -=1;
			index = (*pre_Cache).stack_Of_Indices[(*pre_Cache).stack_Total];
			
	
			if (timeout_Search_Upon_Next_Popup == TRUE) {
				(*pre_Cache).return_Value_Determined = 1;
				(*pre_Cache).initialization_Index = index;
				(*pre_Cache).initialization_Directory = dirIDToSearch;
				goto Normal_Function_Exit;
			}
			
			goto Start_Indexing_Again;
	
		}
		else {
			/*  */
			/* Should not ever fail, especially on locked media such as CD-ROM! */
			/* Oh well, tumble out as error */
			/*  */
			
		}	
	}
	else {
		/*  */
		/* Nothing left on the stack, tumble out, we are done. */
		/*  */
		
	}
	
	/*  */
	/* These are set for debugging purposes, and are not vital */
	/*  */
	(*pre_Cache).initialization_Index = index;
	(*pre_Cache).initialization_Directory = dirIDToSearch;
	
	
Normal_Function_Exit:
	; /* NULL */
	
}/* void Continue_Reentrant_Search (INTEGER_32 dirIDToSearch) */
/*=**************************************************************************=*/		

		
/*=**************************************************************************=*/
Safe_Boolean	Start_Scanning_For_Files(CARDINAL_32 object_Classification)
/*  */
/* In this temporary version of this routine, a search is made local to the */
/* application file as well */
/*  */
{
	Safe_Boolean				function_Result;
	SignedByte				search_Buffer_Original_State;
	SignedByte				search_Results_Original_State;
	SignedByte				stack_Search_Original_State;
	OSErr						os_Error;
	Str63						volume_Name;
	INTEGER_32				index;
	CARDINAL_32			preferred_Size;
	CARDINAL_32			type_To_Match;
	Hierarchial_Location		location_HFS;
	CARDINAL_8				*code_Name;
	Boolean					match_File_Name;
/* END VAR */
	
	
	code_Name = "\pStart_Scanning_For_Files";
	function_Result = SAFE_FALSE;
	match_File_Name = FALSE;

	if (g_Scanning_Active == SAFE_TRUE) {
	/*  */
	/* Ignore if busy */
	/*  */
		function_Result = SAFE_TRUE;
		goto Normal_Function_Exit;
	}
	
	g_Scanning_Active = SAFE_FALSE;
	
	if (g_Scanning_Needs_Run == SAFE_TRUE) {
		g_Scanning_Needs_Run = SAFE_FALSE;
	}


#if (k_SEARCH_ENTIRE_VOLUME_FAST == 1)
	g_Search_Entire_Volume = SAFE_TRUE;
#else
	g_Search_Entire_Volume = SAFE_FALSE;
#endif

	g_Working_Directory_In_Use = SAFE_FALSE;
	
	g_Search_Buffer_Hdl = nil;
	g_Search_Buffer_Hdl_Memory_Initialized = MEMORY_NOT_INITIALIZED;

	g_Search_Results = nil;
	g_Search_Results_Memory_Initialized = MEMORY_NOT_INITIALIZED;

	g_Stack_Search_Hdl = nil;
	g_Stack_Search_Memory_Initialized = MEMORY_NOT_INITIALIZED;


	switch (object_Classification) {
	
		case	k_ALL_SELECTIONS:
					type_To_Match = 0;
				break;
				
		case	k_PICT_SELECTION:
					type_To_Match = k_PICT_FILE_TYPE;
				break;

		case	k_SCENE_SELECTION:
					type_To_Match = k_SCENE_FILE_TYPE;
				break;

		case	k_PROP_SELECTION:
					type_To_Match = k_PUPPET_FILE_TYPE;
				break;

		case	k_BACKDROP_SELECTION:
					type_To_Match = k_BACKDROP_FILE_TYPE;
				break;
				
		case	k_SPIN_OBJECT_SELECTION:
					type_To_Match = k_SPIN_OBJECT_FILE_TYPE;
				break;
				
		case	k_AUDIO_OBJECT_FILE_SELECTION:
					type_To_Match = k_AIFF_TYPE;
				break;
				
		case	k_PUPPET_SELECTION:
					type_To_Match = k_PUPPET_FILE_TYPE;
				break;

				
		default:
			goto Error_Handler;
	}


	if (g_Search_Entire_Volume == SAFE_TRUE) {
		preferred_Size = k_SEARCH_BUFFER_SIZE;
		if (Create_Handle(&preferred_Size
							, (CARDINAL_32)(10240L)
							, ALLOW_PURGING
							, DISALLOW_PARTIAL_AMOUNT
							, (Handle *)&g_Search_Buffer_Hdl
							, &g_Search_Buffer_Hdl_Memory_Initialized) != SAFE_TRUE) {
							
			Display_Low_Memory_Error_Message(unit_Name, code_Name, k_SEARCH_BUFFER_SIZE);
			
			function_Result = SAFE_FALSE;
			goto Error_Handler;
		}
		
		if (Begin_Use_Of_Handle((Handle)g_Search_Buffer_Hdl
										, MOVE_HIGH_REQUIRED
										, &search_Buffer_Original_State) != SAFE_TRUE) {
			goto Error_Handler;
		}
		
		preferred_Size = sizeof(Search_Results_Rec);
		if (Create_Handle(&preferred_Size
							, (CARDINAL_32)(10240L)
							, ALLOW_PURGING
							, DISALLOW_PARTIAL_AMOUNT
							, (Handle *)&g_Search_Results
							, &g_Search_Results_Memory_Initialized) != SAFE_TRUE) {
							
			Display_Low_Memory_Error_Message(unit_Name, code_Name, k_SEARCH_BUFFER_SIZE);
			
			function_Result = SAFE_FALSE;
			goto Error_Handler;
		}
		
		if (Begin_Use_Of_Handle((Handle)g_Search_Results
										, MOVE_HIGH_REQUIRED
										, &search_Results_Original_State) != SAFE_TRUE) {
			goto Error_Handler;
		}
				
		/*  */
		/* Crappy deficient PBCatSearch needs ancient Working Directory. */
		/* Done in hope that one day the thing MIGHT start from the Working Directory */
		/* (Currently, PBCatSearch only scans entire volumes) */
		/*  */
		g_Working_Directory_PB.ioCompletion = nil;
		g_Working_Directory_PB.ioNamePtr = nil;
		g_Working_Directory_PB.ioWDProcID = FILE_CREATOR;
		g_Working_Directory_PB.ioVRefNum = ns_Shell_Globals.application_Volume_Reference;
		g_Working_Directory_PB.ioWDDirID = ns_Shell_Globals.application_Directory_ID;
		
		os_Error = PBOpenWD(&g_Working_Directory_PB, FALSE);
		if (os_Error != noErr) {
			goto Error_Handler;
		}
		g_Working_Directory_In_Use = SAFE_TRUE;
		
		/*  */
		/* Crappy deficient PBCatSearch needs ancient Working Directory and */
		/* for some reason also needs a damned actual volume name! */
		/*  */
		g_Working_Directory = g_Working_Directory_PB.ioVRefNum;
		g_Hfs_Parameter_Block.volumeParam.ioCompletion = nil;
		g_Hfs_Parameter_Block.volumeParam.ioNamePtr = (StringPtr)&volume_Name[0];
		g_Hfs_Parameter_Block.volumeParam.ioVRefNum = ns_Shell_Globals.application_Volume_Reference;
		g_Hfs_Parameter_Block.volumeParam.ioVolIndex = 0;
		
		os_Error = PBHGetVInfo(&g_Hfs_Parameter_Block, FALSE);
		if (os_Error != noErr) {
			goto Error_Handler;
		}
					
		
		g_Search.ioCompletion = nil;
		g_Search.ioNamePtr = &volume_Name[0];
		g_Search.ioVRefNum = g_Working_Directory;
		g_Search.ioMatchPtr = &((**g_Search_Results).search_Results[0]);
		g_Search.ioReqMatchCount = k_MAXIMUM_FILE_MATCHES;
		
		if (match_File_Name == TRUE) {
		/* ===
			HEY!!! We neem some file name to search from! 
			Copy_Pascal_String((CARDINAL_8 *)*item_Name, g_File_Name);
		=== */
			g_File_Name[0] = 0;
			
			g_Search.ioSearchBits = fsSBFullName + fsSBFlAttrib + fsSBFlFndrInfo;
		}
		else {
			g_File_Name[0] = 0;
			g_Search.ioSearchBits =  fsSBFlAttrib + fsSBFlFndrInfo;
		}
		
		g_Search.ioSearchInfo1 = &g_Search_Spec_1; 
		g_Search.ioSearchInfo2 = &g_Search_Spec_2;
		g_Search.ioSearchTime = (k_SEARCH_TIMEOUT * k_MILLISECOND_CONVERSION);
		g_Search.ioCatPosition.initialize = 0;
		
		g_Search.ioOptBuffer = *g_Search_Buffer_Hdl;
		g_Search.ioOptBufSize  = k_SEARCH_BUFFER_SIZE;
	
		if (match_File_Name == TRUE) {
			g_Search_Spec_1.hFileInfo.ioNamePtr = &(g_File_Name[0]);
		}
		else {
			g_Search_Spec_1.hFileInfo.ioNamePtr = nil;
		}
		
		g_Search_Spec_1.hFileInfo.ioFlAttrib = 0x00;
		g_Search_Spec_1.hFileInfo.ioFlFndrInfo.fdType = type_To_Match;
		
		/*  */
		/* Set mask data and high limits */
		/*  */
		g_Search_Spec_2.hFileInfo.ioNamePtr = nil;
		g_Search_Spec_2.hFileInfo.ioFlAttrib  = 0x10;
		g_Search_Spec_2.hFileInfo.ioFlFndrInfo.fdType = 0xFFFF;
		g_Search_Spec_2.hFileInfo.ioFlFndrInfo.fdCreator = 0;
		g_Search_Spec_2.hFileInfo.ioFlFndrInfo.fdFlags = 0;
		g_Search_Spec_2.hFileInfo.ioFlFndrInfo.fdLocation.h = 0;
		g_Search_Spec_2.hFileInfo.ioFlFndrInfo.fdLocation.v = 0;
		g_Search_Spec_2.hFileInfo.ioFlFndrInfo.fdFldr = 0;
		
	} /* if (g_Search_Entire_Volume == SAFE_TRUE) */
	else {
		/*  */
		/* Non-PBCatsearch technique, re-entrant recursion stack method*/
		/*  */

		preferred_Size = sizeof(Recursive_Search_Record);
		if (Create_Handle(&preferred_Size
							, (CARDINAL_32)(10240L)
							, ALLOW_PURGING
							, DISALLOW_PARTIAL_AMOUNT
							, (Handle *)&g_Stack_Search_Hdl
							, &g_Stack_Search_Memory_Initialized) != SAFE_TRUE) {
							
			Display_Low_Memory_Error_Message(unit_Name, code_Name, k_SEARCH_BUFFER_SIZE);
			
			function_Result = SAFE_FALSE;
			goto Error_Handler;
		}
		
		if (Begin_Use_Of_Handle((Handle)g_Stack_Search_Hdl
										, MOVE_HIGH_REQUIRED
										, &stack_Search_Original_State) != SAFE_TRUE) {
			goto Error_Handler;
		}

		(**g_Stack_Search_Hdl).data_Valid = SAFE_TRUE;
		(**g_Stack_Search_Hdl).volume_Reference = ns_Shell_Globals.application_Volume_Reference;
		(**g_Stack_Search_Hdl).param_Block.hFileInfo.ioVRefNum = ns_Shell_Globals.application_Volume_Reference;
		
		(**g_Stack_Search_Hdl).initialization_Directory = ns_Shell_Globals.application_Directory_ID;
		(**g_Stack_Search_Hdl).initialization_Index = 1;
		(**g_Stack_Search_Hdl).stack_Total = 0;

		(**g_Stack_Search_Hdl).return_Value_Determined = 0;

		if (type_To_Match != 0) {
			(**g_Stack_Search_Hdl).file_Type_To_Match[0] = type_To_Match;
			(**g_Stack_Search_Hdl).file_Type_To_Match_Total = 1;
		}
		else {
			(**g_Stack_Search_Hdl).file_Type_To_Match_Total = 0;
			(**g_Stack_Search_Hdl).file_Type_To_Match[(**g_Stack_Search_Hdl).file_Type_To_Match_Total] = k_PICT_FILE_TYPE;
			(**g_Stack_Search_Hdl).file_Type_To_Match_Total += 1;
			(**g_Stack_Search_Hdl).file_Type_To_Match[(**g_Stack_Search_Hdl).file_Type_To_Match_Total] = k_SCENE_FILE_TYPE;
			(**g_Stack_Search_Hdl).file_Type_To_Match_Total += 1;
			(**g_Stack_Search_Hdl).file_Type_To_Match[(**g_Stack_Search_Hdl).file_Type_To_Match_Total] = k_PUPPET_FILE_TYPE;
			(**g_Stack_Search_Hdl).file_Type_To_Match_Total += 1;
			(**g_Stack_Search_Hdl).file_Type_To_Match[(**g_Stack_Search_Hdl).file_Type_To_Match_Total] = k_BACKDROP_FILE_TYPE;
			(**g_Stack_Search_Hdl).file_Type_To_Match_Total += 1;
			(**g_Stack_Search_Hdl).file_Type_To_Match[(**g_Stack_Search_Hdl).file_Type_To_Match_Total] = k_SPIN_OBJECT_FILE_TYPE;
			(**g_Stack_Search_Hdl).file_Type_To_Match_Total += 1;
			(**g_Stack_Search_Hdl).file_Type_To_Match[(**g_Stack_Search_Hdl).file_Type_To_Match_Total] = k_AIFF_TYPE;
			(**g_Stack_Search_Hdl).file_Type_To_Match_Total += 1;
		}



	} /* NOT if (g_Search_Entire_Volume == SAFE_TRUE) */
	

	g_Scanning_Active = SAFE_TRUE;
	
	/*  */
	/* If we did it async style we would start async chain here */
	/*  */
	

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	if (g_Scanning_Active != SAFE_TRUE) {
		if (Shutdown_File_Scanning() != SAFE_TRUE) {
			/* Ignore, quitting anyway */
		}
	}
	
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/		

		
/*=**************************************************************************=*/
Safe_Boolean	Add_Potential_File_Item_To_Database(CARDINAL_32 file_Type, FSSpec *file_Specification, Boolean	*database_Full)
{
	Safe_Boolean					function_Result;
	INTEGER_16					loop;
	INTEGER_16					offset;
	Boolean						located;
	Str63							temp_String;
	Safe_Boolean					resource_Sources_Swapped;
	INTEGER_16					save_Ref;
	OSErr							os_Error;
	Scene_File_Header_Hdl		scene_Header_Hdl;
	Puppet_File_Header_Hdl	puppet_Header_Hdl;
	Backdrop_File_Header_Hdl	backdrop_Header_Hdl;
	Spin_Object_File_Header_Hdl	spin_Object_Header_Hdl;
	INTEGER_16					input_File_Reference;
	CARDINAL_8				*code_Name;
/* END VAR */


	code_Name = "\pLocate_Item_Location";
	function_Result = SAFE_FALSE;
	*database_Full = FALSE;
	resource_Sources_Swapped = SAFE_FALSE;

	switch(file_Type) {
					
		case k_PICT_FILE_TYPE:
			located = FALSE;
			for(loop= 0; ((loop < k_MAXIMUM_SLIDES) && (located != TRUE)); loop++) {
				if (g_Slides_Ptr->slides[loop].active != SAFE_TRUE) {
					located = TRUE;
					offset = loop;
				}
			}
			
			if (located != TRUE) {
/* ===			
				*database_Full = TRUE;
				function_Result = SAFE_TRUE;
				goto Normal_Function_Exit;
==== */
				
			}
			else {
				g_Slides_Ptr->slides[offset].active = SAFE_TRUE;
				g_Slides_Ptr->slides[offset].type_Of_Source = k_FROM_FILE;
				g_Slides_Ptr->slides[offset].algorithm_Id = k_BLACK_WINDOW; /* Unused */
				g_Slides_Ptr->slides[offset].file_Info = *file_Specification;
				
				g_Slides_Ptr->slides[offset].file_Info = *file_Specification;
				Copy_Pascal_String(&((*file_Specification).name[0]), &temp_String[0]);
				if (temp_String[0] > 31) {
					temp_String[0] = 31;
				}
				Copy_Pascal_String(&(temp_String[0]), &(g_Slides_Ptr->slides[offset].slide_Name[0]));
			
				g_Total_Picts_Available += 1;
			}
		break;
				
		case k_SCENE_FILE_TYPE:
			located = FALSE;

			for(loop= 0; ((loop < k_MAXIMUM_DATABASE_SCENES) && (located != TRUE)); loop++) {
				if (g_Remote_Scenes.info.scenes.pointer->scenes[loop].active != SAFE_TRUE) {
					located = TRUE;
					offset = loop;
				}
			}
			
			if (located != TRUE) {
/* ===			
				*database_Full = TRUE;
				function_Result = SAFE_TRUE;
				goto Normal_Function_Exit;
==== */
				
			}
			else {
																
				/*  */
				/* Open for read only permission */
				/*  */
				
				save_Ref = CurResFile();
				input_File_Reference = HOpenResFile (file_Specification->vRefNum
												, file_Specification->parID
												, file_Specification->name
												, fsRdPerm );
									
				if ((input_File_Reference == -1) || (ResError() != noErr)) {
					goto Error_Handler;
				}
																
				UseResFile(input_File_Reference);
				os_Error = ResError();
				if (os_Error != noErr) {
					Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal using resource", "\pCorrupted?");
					goto Error_Handler;
				}
				
				resource_Sources_Swapped = SAFE_TRUE;
			
				scene_Header_Hdl = (Scene_File_Header_Hdl)Get1Resource(k_SCENE_MAIN_RESOURCE_TYPE
																						, k_BASE_SCENE_RESOURCE_ID);
				if ((ResError() != noErr) || (scene_Header_Hdl == nil)) {
					goto Error_Handler;
				}
			
				Copy_Pascal_String(&((*scene_Header_Hdl)->file_Name[0]), &(temp_String[0]));
			
				g_Remote_Scenes.info.scenes.pointer->scenes[offset].active = SAFE_TRUE;
				g_Remote_Scenes.info.scenes.pointer->scenes[offset].type_Of_Source = k_FROM_FILE;
				g_Remote_Scenes.info.scenes.pointer->scenes[offset].algorithm_Id = 0; /* Unused */
				g_Remote_Scenes.info.scenes.pointer->scenes[offset].file_Info = *file_Specification;
				
				if (temp_String[0] > 31) {
					temp_String[0] = 31;
				}
				Copy_Pascal_String(&(temp_String[0]), &(g_Remote_Scenes.info.scenes.pointer->scenes[offset].scene_Name[0]));
			
				g_Remote_Scenes.total_Available += 1;
			}
		break;
		
		case k_PUPPET_FILE_TYPE:
			located = FALSE;
			for(loop= 0; ((loop < k_MAXIMUM_DATABASE_PUPPETS) && (located != TRUE)); loop++) {
				if (g_Puppets.info.puppets.pointer->puppets[loop].active != SAFE_TRUE) {
					located = TRUE;
					offset = loop;
				}
			}
			
			if (located != TRUE) {
/* ===			
				*database_Full = TRUE;
				function_Result = SAFE_TRUE;
				goto Normal_Function_Exit;
==== */
				
			}
			else {
																
				/*  */
				/* Open for read only permission */
				/*  */
				
				save_Ref = CurResFile();
				input_File_Reference = HOpenResFile (file_Specification->vRefNum
												, file_Specification->parID
												, file_Specification->name
												, fsRdPerm );
									
				if ((input_File_Reference == -1) || (ResError() != noErr)) {
					goto Error_Handler;
				}
																
				UseResFile(input_File_Reference);
				os_Error = ResError();
				if (os_Error != noErr) {
					Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal using resource", "\pCorrupted?");
					goto Error_Handler;
				}
				
				resource_Sources_Swapped = SAFE_TRUE;
			
				puppet_Header_Hdl = (Puppet_File_Header_Hdl)Get1Resource(k_PUPPET_MAIN_RESOURCE_TYPE
																						, k_BASE_PUPPET_RESOURCE_ID);
				if ((ResError() != noErr) || (puppet_Header_Hdl == nil)) {
					goto Error_Handler;
				}
			
				Copy_Pascal_String(&((*puppet_Header_Hdl)->file_Name[0]), &(temp_String[0]));
			
				g_Puppets.info.puppets.pointer->puppets[offset].active = SAFE_TRUE;
				g_Puppets.info.puppets.pointer->puppets[offset].type_Of_Source = k_FROM_FILE;
				g_Puppets.info.puppets.pointer->puppets[offset].algorithm_Id = 0; /* Unused */
				g_Puppets.info.puppets.pointer->puppets[offset].file_Info = *file_Specification;
				
				if (temp_String[0] > 31) {
					temp_String[0] = 31;
				}
				Copy_Pascal_String(&(temp_String[0]), &(g_Puppets.info.puppets.pointer->puppets[offset].puppet_Name[0]));
			
				g_Puppets.total_Available += 1;
			}
		break;
		
		case k_BACKDROP_FILE_TYPE:
			located = FALSE;
			for(loop= 0; ((loop < k_MAXIMUM_DATABASE_BACKDROPS) && (located != TRUE)); loop++) {
				if (g_Backdrops.info.backdrops.pointer->backdrops[loop].active != SAFE_TRUE) {
					located = TRUE;
					offset = loop;
				}
			}
			
			if (located != TRUE) {
/* ===			
				*database_Full = TRUE;
				function_Result = SAFE_TRUE;
				goto Normal_Function_Exit;
==== */
				
			}
			else {
																
				/*  */
				/* Open for read only permission */
				/*  */
				
				save_Ref = CurResFile();
				input_File_Reference = HOpenResFile (file_Specification->vRefNum
												, file_Specification->parID
												, file_Specification->name
												, fsRdPerm );
									
				if ((input_File_Reference == -1) || (ResError() != noErr)) {
					goto Error_Handler;
				}
																
				UseResFile(input_File_Reference);
				os_Error = ResError();
				if (os_Error != noErr) {
					Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal using resource", "\pCorrupted?");
					goto Error_Handler;
				}
				
				resource_Sources_Swapped = SAFE_TRUE;
			
				backdrop_Header_Hdl = (Backdrop_File_Header_Hdl)Get1Resource(k_BACKDROP_MAIN_RESOURCE_TYPE
																						, k_BASE_BACKDROP_RESOURCE_ID);
				if ((ResError() != noErr) || (backdrop_Header_Hdl == nil)) {
					goto Error_Handler;
				}
			
				Copy_Pascal_String(&((*backdrop_Header_Hdl)->file_Name[0]), &(temp_String[0]));
			
				g_Backdrops.info.backdrops.pointer->backdrops[offset].active = SAFE_TRUE;
				g_Backdrops.info.backdrops.pointer->backdrops[offset].type_Of_Source = k_FROM_FILE;
				g_Backdrops.info.backdrops.pointer->backdrops[offset].algorithm_Id = 0; /* Unused */
				g_Backdrops.info.backdrops.pointer->backdrops[offset].file_Info = *file_Specification;
				
				if (temp_String[0] > 31) {
					temp_String[0] = 31;
				}
				Copy_Pascal_String(&(temp_String[0]), &(g_Backdrops.info.backdrops.pointer->backdrops[offset].backdrop_Name[0]));
			
				g_Backdrops.total_Available += 1;
			}
		break;
		
		
		case k_SPIN_OBJECT_FILE_TYPE:
			located = FALSE;
			for(loop= 0; ((loop < k_MAXIMUM_DATABASE_SPIN_OBJECTS) && (located != TRUE)); loop++) {
				if (g_Spin_Objects.info.spin_Objects.pointer->spin_Objects[loop].active != SAFE_TRUE) {
					located = TRUE;
					offset = loop;
				}
			}
			
			if (located != TRUE) {
/* ===			
				*database_Full = TRUE;
				function_Result = SAFE_TRUE;
				goto Normal_Function_Exit;
==== */
				
			}
			else {
																
				/*  */
				/* Open for read only permission */
				/*  */
				
				save_Ref = CurResFile();
				input_File_Reference = HOpenResFile (file_Specification->vRefNum
												, file_Specification->parID
												, file_Specification->name
												, fsRdPerm );
									
				if ((input_File_Reference == -1) || (ResError() != noErr)) {
					goto Error_Handler;
				}
																
				UseResFile(input_File_Reference);
				os_Error = ResError();
				if (os_Error != noErr) {
					Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal using resource", "\pCorrupted?");
					goto Error_Handler;
				}
				
				resource_Sources_Swapped = SAFE_TRUE;
			
				spin_Object_Header_Hdl = (Spin_Object_File_Header_Hdl)Get1Resource(k_SPIN_OBJECT_MAIN_RESOURCE_TYPE
																						, k_BASE_SPIN_OBJECT_RESOURCE_ID);
				if ((ResError() != noErr) || (spin_Object_Header_Hdl == nil)) {
					goto Error_Handler;
				}
			
				Copy_Pascal_String(&((*spin_Object_Header_Hdl)->file_Name[0]), &(temp_String[0]));
			
				g_Spin_Objects.info.spin_Objects.pointer->spin_Objects[offset].active = SAFE_TRUE;
				g_Spin_Objects.info.spin_Objects.pointer->spin_Objects[offset].type_Of_Source = k_FROM_FILE;
				g_Spin_Objects.info.spin_Objects.pointer->spin_Objects[offset].algorithm_Id = 0; /* Unused */
				g_Spin_Objects.info.spin_Objects.pointer->spin_Objects[offset].file_Info = *file_Specification;
				
				if (temp_String[0] > 31) {
					temp_String[0] = 31;
				}
				Copy_Pascal_String(&(temp_String[0]), &(g_Spin_Objects.info.spin_Objects.pointer->spin_Objects[offset].spin_Object_Name[0]));
			
				g_Spin_Objects.total_Available += 1;
			}
		break; /* case k_SPIN_OBJECT_FILE_TYPE */
		

		case k_AIFF_TYPE:
			located = FALSE;
			for(loop= 0; ((loop < k_MAXIMUM_DATABASE_SOUND_OBJECTS) && (located != TRUE)); loop++) {
				if (g_Audio_Files.info.sounds.pointer->sound_Objects[loop].active != SAFE_TRUE) {
					located = TRUE;
					offset = loop;
				}
			}
			
			if (located != TRUE) {
/* ===			
				*database_Full = TRUE;
				function_Result = SAFE_TRUE;
				goto Normal_Function_Exit;
==== */
				
			}
			else {
				/*  */
				/* Just blindly accept it. */
				/*  */
				
				g_Audio_Files.info.sounds.pointer->sound_Objects[offset].active = SAFE_TRUE;
				g_Audio_Files.info.sounds.pointer->sound_Objects[offset].type_Of_Source = k_FROM_FILE;
				g_Audio_Files.info.sounds.pointer->sound_Objects[offset].algorithm_Id = 0; /* Unused */
				g_Audio_Files.info.sounds.pointer->sound_Objects[offset].file_Info = *file_Specification;
				Copy_Pascal_String(&((*file_Specification).name[0]), &temp_String[0]);
				if (temp_String[0] > 31) {
					temp_String[0] = 31;
				}
				Copy_Pascal_String(&(temp_String[0]), &(g_Audio_Files.info.sounds.pointer->sound_Objects[offset].sound_Object_Name[0]));
			
				g_Audio_Files.total_Available += 1;
			}
		break; /* case k_AIFF_TYPE */
		
		
		default:
			; /* NULL */
	
	}
	

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
	
Normal_Function_Exit:
	if (resource_Sources_Swapped == SAFE_TRUE) {
		CloseResFile(input_File_Reference);
		if (ResError() != noErr) {
			/*  */
			/* Ignore, we are exiting.  */
			/*  */
		}
		
		UseResFile(save_Ref);
		if (ResError() != noErr) {
			/*  */
			/* This should be recoverable from, enhance one day  */
			/*  */
		}
		resource_Sources_Swapped = SAFE_FALSE;
	}
			

	return(function_Result);
}
/*=**************************************************************************=*/		

		
/*=**************************************************************************=*/
Safe_Boolean	Shutdown_File_Scanning(void)
{
	Safe_Boolean			function_Result;
	OSErr					os_Error;
	CARDINAL_8			*code_Name;
/* END VAR */


	code_Name = "\pContinue_Scanning_For_Files";
	function_Result = SAFE_FALSE;
	
	g_Scanning_Active = SAFE_FALSE;
	g_Scanning_Needs_Run = SAFE_FALSE;


	if (g_Search_Buffer_Hdl_Memory_Initialized == MEMORY_INITIALIZED) {
		if (Destroy_Handle((Handle *)&g_Search_Buffer_Hdl
							, &g_Search_Buffer_Hdl_Memory_Initialized) != SAFE_TRUE) {
							
			Display_Serious_Error_Message(unit_Name, code_Name, "\pError with Destroy_Handle", "\pSystem Corrupted?");
		}
	}
	
	if (g_Search_Results_Memory_Initialized == MEMORY_INITIALIZED) {
		if (Destroy_Handle((Handle *)&g_Search_Results
							, &g_Search_Results_Memory_Initialized) != SAFE_TRUE) {
							
			Display_Serious_Error_Message(unit_Name, code_Name, "\pError with Destroy_Handle", "\pSystem Corrupted?");
		}
	}
	
	if (g_Stack_Search_Memory_Initialized == MEMORY_INITIALIZED) {
		if (Destroy_Handle((Handle *)&g_Stack_Search_Hdl
							, &g_Stack_Search_Memory_Initialized) != SAFE_TRUE) {
							
			Display_Serious_Error_Message(unit_Name, code_Name, "\pError with Destroy_Handle", "\pSystem Corrupted?");
		}
	}
							
	/*  */
	/* Placed here to ensure not accidentally ever left allocated */
	/*  */
	if (g_Working_Directory_In_Use == SAFE_TRUE) {
		os_Error = CloseWD(g_Working_Directory);
		if (os_Error != noErr) {
		/* ===
			goto Error_Handler;
		=== */
		}
	}
		
	
Normal_Function_Exit:

	return(function_Result);
}
/*=**************************************************************************=*/		

		
/*=**************************************************************************=*/	
Safe_Boolean	Continue_Scanning_For_Files(void)
/*  */
/* In this temporary version of this routine, a search is made local to the */
/* application file as well */
/*  */
{
	OSErr					os_Error;
	Boolean				database_Full;
	CARDINAL_32		this_File_Type;
	INTEGER_16			index;
	Safe_Boolean			function_Result;
	INTEGER_16			loop;
	Boolean				found_Possible_Match;
	Boolean				done_Searching;
	FSSpec				file_Specification;
/* END VAR */
	
	
	function_Result = SAFE_FALSE;
	
	if (g_Scanning_Active!= SAFE_TRUE) {
		goto Error_Handler;
	}
	
	if (g_Search_Entire_Volume == SAFE_TRUE) {
	
		g_Search.ioActMatchCount = 0;
		os_Error = PBCatSearchSync(&g_Search);			
		if (os_Error == eofErr) {
			done_Searching = TRUE;
		}
		else {
			done_Searching = FALSE;
		}
		
		if (((os_Error == noErr) || (done_Searching == TRUE)) && (g_Search.ioActMatchCount > 0)) {
			/*  */
			/* Something is in the match buffers */
			/* Check if they are the ones we want ( a good routine would */
			/* open them up and serach for internal file name) */
			/*  */
			for (index = 0; (index < g_Search.ioActMatchCount); index++) {
			
				/*  */
				/* Just take first one we find */
				/*  */
				file_Specification = ((**g_Search_Results).search_Results[index]);
				
				
			/*  HACK ===
				this_File_Type = temp_PB_Ptr->hFileInfo.ioFlFndrInfo.fdType;
			==== */
				this_File_Type = (**g_Stack_Search_Hdl).file_Type_To_Match[0];
				
				found_Possible_Match = FALSE;
				for (loop = 0 ; ((loop < (**g_Stack_Search_Hdl).file_Type_To_Match_Total) && (found_Possible_Match != TRUE)); loop++) {
					if ((**g_Stack_Search_Hdl).file_Type_To_Match[loop] == this_File_Type) {
						found_Possible_Match = TRUE;
					}
				}
				
				if (found_Possible_Match == TRUE) {
				
					/*  */
					/* Before Accepting it we must validate it */
					/*  */
					database_Full = FALSE;
					if (Add_Potential_File_Item_To_Database(this_File_Type, &file_Specification, &database_Full) != SAFE_TRUE) {
						done_Searching = TRUE;
						goto Error_Handler;
					}
					else {
						if (database_Full == TRUE) {
							done_Searching = TRUE;
						}
					}
				}
			}
		}
		
	} /* if (g_Search_Entire_Volume == SAFE_TRUE) */
	else {
	
		/*  */
		/* Assume "Recursive_Search_Hdl" already locked down */
		/*  */
		Continue_Reentrant_Search(*g_Stack_Search_Hdl);
	
		if ((**g_Stack_Search_Hdl).return_Value_Determined == 0) {
			done_Searching = TRUE;
		}
		
	} /* if (g_Search_Entire_Volume == SAFE_TRUE) */
	
	
	if (done_Searching == TRUE) {
		g_Scanning_Active = SAFE_FALSE;
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
	
	g_Scanning_Active = SAFE_FALSE;
	
	
Normal_Function_Exit:

	if (g_Scanning_Active != SAFE_TRUE) {
		if (Shutdown_File_Scanning() != SAFE_TRUE) {
			/* Ignore, quitting anyway */
		}
	}

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Object_Inventory.c */
